// file: $isip/class/algo/AlgorithmUndefined/AlgorithmUndefined.h
// version: $Id: AlgorithmUndefined.h 8226 2002-06-25 01:42:42Z picone $
//

// make sure definitions are only made once
//
#ifndef ISIP_ALGORITHM_UNDEFINED
#define ISIP_ALGORITHM_UNDEFINED

// isip include files
//
#ifndef ISIP_ALGORITHM_BASE
#include <AlgorithmBase.h>
#endif


// AlgorithmUndefined: a class used to represent an empty choice
//  for an algorithm.
//
class AlgorithmUndefined : public AlgorithmBase {

  //---------------------------------------------------------------------------
  //
  // public constants
  //
  //---------------------------------------------------------------------------
public:

  // define the class name
  //
  static const String CLASS_NAME;

  //----------------------------------------
  //
  // other important constants
  //
  //----------------------------------------

  //----------------------------------------
  //
  // default values and arguments
  //
  //----------------------------------------  

  // define the default value(s) of the class data
  //
  
  //----------------------------------------
  //
  // error codes
  //
  //----------------------------------------  

  //---------------------------------------------------------------------------
  //
  // protected data
  //
  //---------------------------------------------------------------------------
protected:

  //---------------------------------------------------------------------------
  //
  // required public methods
  //
  //---------------------------------------------------------------------------
public:
  
  // method: name
  //
  static const String& name() {
    return CLASS_NAME;
  }

  // method: diagnose
  //
  static bool8 diagnose(const unichar* msg) {
    return Error::handle(name(), L"diagnose",
			 Error::VIRTUAL_PTR, __FILE__, __LINE__);
  }
  
  // method: debug
  //  setDebug method is inherited from the base class
  //
  bool8 debug(const unichar* msg) const;

  // method: destructor
  //
  ~AlgorithmUndefined() {}

  // method: default constructor
  //
  AlgorithmUndefined() {}

  // method: copy constructor
  //
  AlgorithmUndefined(const AlgorithmUndefined& arg) {
    assign(arg);
  }

  // method: assign
  //
  bool8 assign(const AlgorithmUndefined& arg) {
    return Error::handle(name(), L"assign",
			 Error::VIRTUAL_PTR, __FILE__, __LINE__);
  }
  
  // method: sofSize
  //
  int32 sofSize() const {
    return Error::handle(name(), L"sofSize",
			 Error::VIRTUAL_PTR, __FILE__, __LINE__);
  }
  
  // method: read
  //
  bool8 read(Sof& sof, int32 tag, const String& name) {
    return Error::handle(CLASS_NAME, L"read",
			 Error::VIRTUAL_PTR, __FILE__, __LINE__);
  }

  // method: write
  //
  bool8 write(Sof& sof, int32 tag, const String& name) const {
    return Error::handle(CLASS_NAME, L"write",
			 Error::VIRTUAL_PTR, __FILE__, __LINE__);
  }

  // method: readData
  //
  bool8 readData(Sof& sof, const String& pname,
		   int32 size, bool8 param, bool8 nested) {
    return Error::handle(CLASS_NAME, L"readData",
			 Error::VIRTUAL_PTR, __FILE__, __LINE__);
  }

  // method: writeData
  //
  bool8 writeData(Sof& sof, const String& pname) const {
    return Error::handle(CLASS_NAME, L"writeData",
			 Error::VIRTUAL_PTR, __FILE__, __LINE__);
  }
  
  // method: eq
  //
  bool8 eq(const AlgorithmUndefined& arg) const {
    return Error::handle(CLASS_NAME, L"eq",
			 Error::VIRTUAL_PTR, __FILE__, __LINE__);
  }

  // method: clear
  //
  bool8 clear() {
    return Error::handle(CLASS_NAME, L"clear",
			 Error::VIRTUAL_PTR, __FILE__, __LINE__);
  }

  //---------------------------------------------------------------------------
  //
  // class-specific public methods:
  //  set methods
  //
  //---------------------------------------------------------------------------

  //  the set methods are inherited from the base class
  //

  //---------------------------------------------------------------------------
  //
  // class-specific public methods:
  //  get methods
  //
  //---------------------------------------------------------------------------  
  //  the get methods are inherited from the base class
  //
  
  //---------------------------------------------------------------------------
  //
  // class-specific public methods:  
  //  interface contract methods
  //
  //---------------------------------------------------------------------------
  
  // method: assign
  //
  bool8 assign(const AlgorithmBase& arg) {
    return Error::handle(name(), L"assign",
			 Error::VIRTUAL_PTR, __FILE__, __LINE__);
  }

  // method: eq
  //
  bool8 eq(const AlgorithmBase& arg) const {
    return Error::handle(name(), L"eq",
			 Error::VIRTUAL_PTR, __FILE__, __LINE__);
  }

  // method: className
  //
  const String& className() const {
    Error::handle(CLASS_NAME, L"className",
		  Error::VIRTUAL_PTR, __FILE__, __LINE__);
    return name();
  }
    
  // method: init
  //
  bool8 init() {
    return true;
  }

  // method: apply
  //
  bool8 apply(Vector<AlgorithmData>& output,
		const Vector< CircularBuffer<AlgorithmData> >& input) {
    return Error::handle(CLASS_NAME, L"apply",
			 Error::VIRTUAL_PTR, __FILE__, __LINE__);
  }
  
  // method: getLeadingPad
  //
  int32 getLeadingPad() const {
    return Error::handle(CLASS_NAME, L"getLeadingPad",
			 Error::VIRTUAL_PTR, __FILE__, __LINE__);
  }
  
  // method: getTrailingPad
  //
  int32 getTrailingPad() const {
    return Error::handle(CLASS_NAME, L"getTrailingPad",
			 Error::VIRTUAL_PTR, __FILE__, __LINE__);
  }
  
  // method: setParser
  //
  bool8 setParser(SofParser* parser) {
    return Error::handle(CLASS_NAME, L"setParser",
			 Error::VIRTUAL_PTR, __FILE__, __LINE__);
  }
  
  //---------------------------------------------------------------------------
  //
  // private methods
  //
  //---------------------------------------------------------------------------
private:

};

// end of include file
// 
#endif
