// file: $isip/class/algo/Cepstrum/cep_04.cc
// version: $Id: cep_04.cc 8417 2002-07-17 21:59:48Z parihar $
//

// isip include files
//
#include "Cepstrum.h"

// method: sofSize
//
// arguments: none 
//
// return: size of the object
//
// this method returns the size of the object
//
int32 Cepstrum::sofSize() const {

  // start with enough space for the number of coefficients
  //
  int32 bytes = order_d.sofSize();

  // add the space required for the algorithm name and debug level object
  //
  bytes += debug_level_d.sofSize();  
  bytes += ALGO_MAP.elementSofSize();  

  // add the space required for the implementation name
  //
  bytes += IMPL_MAP.elementSofSize();

  // add the space required for the minimum amplitude
  //
  bytes += min_amp_d.sofSize();

  // add the space required for the minimum amplitude flag
  //
  bytes += flag_min_amp_d.sofSize();

  // return the size
  //
  return bytes;
}

// method: write
//
// arguments:
//  Sof& sof: (input) sof file object
//  int32 tag: (input) sof object instance tag
//  const String& name: (input) sof object instance name
//
// return: a bool8 value indicating status
//
// this method has the object write itself, with specified name and
// tag, to an Sof file
//
bool8 Cepstrum::write(Sof& sof_a, int32 tag_a, const String& name_a) const {

  // declare local variables
  //
  int32 obj_size = 0;
  
  // write the instance of the object into the Sof file
  //
  if (sof_a.isText()) {

    // set the size to by dynamic
    //
    obj_size = Sof::ANY_SIZE;
  }
  else {

    // the size index and the size of each element
    //
    obj_size = sofSize();
  }

  // put the object into the sof file's index
  //
  if (!sof_a.put(name_a, tag_a, obj_size)) {
    return false;
  }
  
  // exit gracefully
  //
  return writeData(sof_a);
}

// method: writeData
//
// arguments:
//  Sof& sof: (input) sof file object
//  const String& pname: (input) parameter name
//
// return: a bool8 value indicating status
//
// this method has the object read itself from an Sof file. it assumes
// that the Sof file is already positioned correctly.
//
bool8 Cepstrum::writeData(Sof& sof_a,
			    const String& pname_a) const {

  // declare local variables
  //
  bool8 status = false;
  
  // write a start string if necessary
  //
  sof_a.writeLabelPrefix(pname_a);

  // write the algorithm type
  //
  ALGO_MAP.writeElementData(sof_a, PARAM_ALGORITHM, (int32)algorithm_d);
  
  // write debug level
  //
  debug_level_d.writeData(sof_a, PARAM_DBGL);
  
  // invoke the proper lower-level method
  //
  if ((algorithm_d == IDCT) || (algorithm_d == IDFT)) {
    status = writeDataCommon(sof_a, pname_a);
  }
  else {
    return Error::handle(name(), L"writeData", Error::ARG,
			 __FILE__, __LINE__);
  }
  
  // put an end string if necessary
  //
  sof_a.writeLabelSuffix(pname_a);
  
  // exit gracefully
  //
  return status;
}

// method: writeDataCommon
//
// arguments:
//  Sof& sof: (input) sof file object
//  const String& pname: (input) parameter name
//
// return: a bool8 value indicating status
//
// this method writes data to a file for several algorithms that
// share the same parameters.
//
bool8 Cepstrum::writeDataCommon(Sof& sof_a, const String& pname_a) const {

  // write the implementation type
  //
  IMPL_MAP.writeElementData(sof_a, PARAM_IMPLEMENTATION,
			    (int32)implementation_d);
  
  // write the number of coefficients
  //
  order_d.writeData(sof_a, PARAM_ORDER);

  // write the minimum amplitude value
  //
  min_amp_d.writeData(sof_a, PARAM_MIN_AMP);
  
  // write the minimum amplitude flag
  //
  flag_min_amp_d.writeData(sof_a, PARAM_FLAG_MIN_AMP);
  
  // exit gracefully
  //
  return true;
}
