// file: $isip/class/algo/CoefficientLabel/clab_02.cc
// version: $Id: clab_02.cc 6569 2001-03-30 15:49:39Z duncan $
//

// isip include files
//
#include <Console.h>
#include "CoefficientLabel.h"

// method: diagnose
//
// arguments:
//  Integral::DEBUG level: (input) debug level for diagnostics
//
// return: a bool8 value indicating status
//
bool8 CoefficientLabel::diagnose(Integral::DEBUG level_a) {

  //---------------------------------------------------------------------------
  //
  // 0. preliminaries
  //
  //---------------------------------------------------------------------------

  // output the class name
  //
  if (level_a > Integral::NONE) {
    SysString output(L"diagnosing class ");
    output.concat(CLASS_NAME);
    output.concat(L": ");
    Console::put(output);
    Console::increaseIndention();
  }
  
  //--------------------------------------------------------------------------
  //
  // 1. required public methods
  //
  //--------------------------------------------------------------------------

  // set indentation
  //
  if (level_a > Integral::NONE) {
    Console::put(L"testing required public methods...\n");
    Console::increaseIndention();
  }

  // declare CoefficientLabel objects for read/write methods
  //
  CoefficientLabel dbuf2;
  CoefficientLabel dbuf3;
  CoefficientLabel dbuf4;

  // we need binary and text sof files
  //
  String tmp_filename0;
  Integral::makeTemp(tmp_filename0);
  String tmp_filename1;
  Integral::makeTemp(tmp_filename1);

  // open files in write mode
  //
  Sof tmp_file0;
  tmp_file0.open(tmp_filename0, File::WRITE_ONLY, File::TEXT);
  Sof tmp_file1;
  tmp_file1.open(tmp_filename1, File::WRITE_ONLY, File::BINARY);

  // write to text and binary files
  //
  dbuf2.write(tmp_file0, (int32)0);
  dbuf2.write(tmp_file1, (int32)0);

  // close the files
  //
  tmp_file0.close();
  tmp_file1.close();

  // open the files in read mode
  //
  tmp_file0.open(tmp_filename0);
  tmp_file1.open(tmp_filename1);

  // read the value back
  //
  dbuf3.read(tmp_file0, (int32)0);
  if (!dbuf3.eq(dbuf2)) {
    return Error::handle(name(), L"read", Error::TEST, __FILE__, __LINE__);
  }
  dbuf4.read(tmp_file0, (int32)0);
  if (!dbuf4.eq(dbuf2)) {
    dbuf4.debug(L"dbuf4");
    return Error::handle(name(), L"read", Error::TEST, __FILE__, __LINE__);
  }

  // close and  delete the temporary files
  //
  tmp_file0.close();
  tmp_file1.close();
  File::remove(tmp_filename0);
  File::remove(tmp_filename1);

  // test the conditional i/o methods 
  //
  CoefficientLabel dbuf_0;
  
  Sof tmp_file2;
  String tmp_filename2;
  Integral::makeTemp(tmp_filename2);

  tmp_file2.open(tmp_filename2, File::WRITE_ONLY);
  
  tmp_file2.put(CLASS_NAME, 5, -1);
  String line;

  line.assign(L"variable = blah;\n");
  tmp_file2.puts(line);

  line.assign(L"type = input;\n");
  tmp_file2.puts(line);

  // close the file
  //
  tmp_file2.close();

  // open the file in read mode
  //
  tmp_file2.open(tmp_filename2);

  // read the values back
  //
  dbuf_0.read(tmp_file2, (int32)5);

  if ((dbuf_0.getVariable().ne(L"blah")) || (dbuf_0.getType() != INPUT)) {
    dbuf_0.debug(L"dbuf_0");
    return Error::handle(name(), L"readData", Error::TEST, __FILE__, __LINE__);
  }
  
  // close and  delete the temporary files
  //
  tmp_file2.close();
  File::remove(tmp_filename2);

  // check constructors and memory management methods
  //
  CoefficientLabel dbuf0;
  CoefficientLabel dbuf1(dbuf0);

  if (!dbuf1.eq(dbuf0)) {
    return Error::handle(name(), L"copy constructor", Error::TEST,
                         __FILE__, __LINE__);
  }

  // test large allocation construction and deletion
  //
  if (level_a == Integral::ALL) {
    
    Console::put(L"\ntesting large chunk memory allocation and deletion:\n");
    
    // set the memory to a strange block size so we can hopefully catch any
    // frame overrun errors
    //
    
    CoefficientLabel::setGrowSize((int32)500);
    
    CoefficientLabel* pft = new CoefficientLabel();
    
    for (int32 j = 1; j <= 100; j++) {
      CoefficientLabel** pfts = new CoefficientLabel*[j * 100];
      
      // create the objects
      //
      for (int32 i = 0; i < j * 100; i++) {
	pfts[i] = new CoefficientLabel();
      }
    
      // delete objects
      //
      for (int32 i = (j * 100) - 1; i >= 0; i--) {
	delete pfts[i];
      }
      delete [] pfts;
    } 
    delete pft;
  }

  // reset indentation
  //
  if (level_a > Integral::NONE) {
    Console::decreaseIndention();
  }
  
  //---------------------------------------------------------------------------
  //
  // 2. class-specific public methods:
  //     set and get methods     
  //
  //---------------------------------------------------------------------------

  // set indentation
  //
  if (level_a > Integral::NONE) {
    Console::put(L"testing class-specific public methods: set methods...\n");
    Console::increaseIndention();
  }

  // test set/get variable method
  //
  String str(L"my var");
  dbuf2.setVariable(str);

  if (dbuf2.getVariable().ne(L"my var")) {
    return Error::handle(name(), L"setVarName", Error::TEST,
                         __FILE__, __LINE__);
  }

  // test set/get variable method
  //
  dbuf2.setType(OUTPUT);

  if (dbuf2.getType() != OUTPUT) {
    return Error::handle(name(), L"setType", Error::TEST,
                         __FILE__, __LINE__);
  }
  // reset indentation
  //
  if (level_a > Integral::NONE) {
    Console::decreaseIndention();
  }
 
  //---------------------------------------------------------------------------
  //
  // 5. print completion message
  //
  //---------------------------------------------------------------------------

  // reset indentation
  //
  if (level_a > Integral::NONE) {
    Console::decreaseIndention();
  }
  
  if (level_a > Integral::NONE) {
    SysString output(L"diagnostics passed for class ");
    output.concat(name());
    output.concat(L"\n");
    Console::put(output);
  }

  // exit gracefully
  //
  return true;
}





