// file: $isip/class/algo/CoefficientLabel/clab_04.cc
// version: $Id: clab_04.cc 8251 2002-06-29 18:02:58Z picone $
//

// isip include files
//
#include "CoefficientLabel.h"

// method: sofSize
//
// arguments: none 
//
// return: size of the object
//
// this method returns the size of the object
//
int32 CoefficientLabel::sofSize() const {

  // accumulate the sizes
  //
  int32 bytes = ALGO_MAP.elementSofSize();
  bytes += IMPL_MAP.elementSofSize();
  bytes += variable_d.sofSize();
  bytes += TYPE_MAP.elementSofSize();
  bytes += AlgorithmData::CTYPE_MAP.elementSofSize();
  bytes += debug_level_d.sofSize();  

  // return the size
  //
  return bytes;
}

// method: write
//
// arguments:
//  Sof& sof: (input) sof file object
//  int32 tag: (input) sof object instance tag
//  const String& name: (input) sof object instance name
//
// return: a bool8 value indicating status
//
// this method has the object write itself, with specified name and
// tag, to an Sof file
//
bool8 CoefficientLabel::write(Sof& sof_a, int32 tag_a,
				const String& name_a) const {

  // declare local variables
  //
  int32 obj_size = 0;
  
  // write the instance of the object into the Sof file
  //
  if (sof_a.isText()) {

    // set the size to by dynamic
    //
    obj_size = Sof::ANY_SIZE;
  }
  else {

    // the size index and the size of each element
    //
    obj_size = sofSize();
  }

  // put the object into the sof file's index
  //
  if (!sof_a.put(name_a, tag_a, obj_size)) {
    return false;
  }
  
  // exit gracefully
  //
  return writeData(sof_a);
}

// method: writeData
//
// arguments:
//  Sof& sof: (input) sof file object
//  const String& pname: (input) parameter name
//
// return: a bool8 value indicating status
//
// this method has the object read itself from an Sof file. it assumes
// that the Sof file is already positioned correctly.
//
bool8 CoefficientLabel::writeData(Sof& sof_a,
			    const String& pname_a) const {

  // write a start string if necessary
  //
  sof_a.writeLabelPrefix(pname_a);

  // write the type
  //
  ALGO_MAP.writeElementData(sof_a, PARAM_ALGORITHM, (int32)algorithm_d);

  // write the type
  //
  IMPL_MAP.writeElementData(sof_a, PARAM_IMPLEMENTATION,
			    (int32)implementation_d);

  // write the variable name
  //
  variable_d.writeData(sof_a, PARAM_VARIABLE);

  // write the type
  //
  TYPE_MAP.writeElementData(sof_a, PARAM_TYPE, (int32)type_d);

  // write the type
  //
  AlgorithmData::CTYPE_MAP.writeElementData(sof_a, PARAM_COEF_TYPE,
					    (int32)coef_type_d);

  // write debug level
  //
  debug_level_d.writeData(sof_a, PARAM_DBGL);
  
  // put an end string if necessary
  //
  sof_a.writeLabelSuffix(pname_a);
  
  // exit gracefully
  //
  return true;
}
