// file: $isip/class/algo/Connection/conn_04.cc
// version: $Id: conn_04.cc 6807 2001-04-24 15:28:14Z peng $
//

// isip include files
//
#include "Connection.h"

// method: sofSize
//
// arguments: none 
//
// return: size of the object
//
// this method returns the size of the object
//
int32 Connection::sofSize() const {

  // start with enough space for the type
  //
  int32 bytes = ALGO_MAP.elementSofSize();

  bytes += IMPL_MAP.elementSofSize();

  bytes += channel_d.sofSize();
  
  // add the size of debug_level_d
  //
  bytes += debug_level_d.sofSize();
  
  // return the size
  //
  return bytes;
}

// method: write
//
// arguments:
//  Sof& sof: (input) sof file object
//  int32 tag: (input) sof object instance tag
//  const String& name: (input) sof object instance name
//
// return: logical error status
//
// this method has the object write itself to an Sof file
//
bool8 Connection::write(Sof& sof_a, int32 tag_a, const String& name_a) const {

  int32 obj_size = 0;
  
  // write the instance of the object into the Sof file
  //
  if (sof_a.isText()) {

    // set the size to by dynamic
    //
    obj_size = Sof::ANY_SIZE;
  }
  else {

    // the size index and the size of each element
    //
    obj_size = sofSize();
  }

  // write the object into the sof file's index
  //
  if (!sof_a.put(name_a, tag_a, obj_size)) {
    return false;
  }
  
  // exit gracefully
  //
  return writeData(sof_a);
}

// method: writeData
//
// arguments:
//  Sof& sof: (input) sof file object
//  const String& pname: (input) parameter name
//
// return: logical error status
//
// this method has the object writes itself from an Sof file. it should make
// sure that the Sof file is positioned correctly.
//
bool8 Connection::writeData(Sof& sof_a, const String& pname_a) const {

  // write a start string if necessary
  //
  sof_a.writeLabelPrefix(pname_a);

  // write the algorithm type
  //
  ALGO_MAP.writeElementData(sof_a, PARAM_ALGORITHM, (int32)algorithm_d);

  // write the implementation type
  //
  IMPL_MAP.writeElementData(sof_a, PARAM_IMPLEMENTATION,
			    (int32)implementation_d);

  // write the channel
  //
  channel_d.writeData(sof_a, PARAM_CHANNEL);
  
  // write debug level
  //
  debug_level_d.writeData(sof_a, PARAM_DBGL);

  // put an end string if necessary
  //
  sof_a.writeLabelSuffix(pname_a);
  
  // exit gracefully
  //
  return true;
}
