// file: $isip_ifc/class/algo/CorrelationDimension/cdim_04.cc
// version: $Id: cdim_04.cc 10555 2006-04-19 08:39:03Z pannuri $
//

// isip include files
//
#include "CorrelationDimension.h"

// method: sofSize
//
// arguments: none
//
// return: size of object
//
// this method returns the size of the object in the Sof file and is
// used for binary write
//
int32 CorrelationDimension::sofSize() const {
  
  // total things common to all algorithms
  //

  // start with the space required for the algorithm name
  //
  int32 bytes = ALGO_MAP.elementSofSize();

  // add the space required for the implementation name
  //
  bytes += IMPL_MAP.elementSofSize();
  
  // add the space required for the implementation name
  //
  bytes += SCAL_MAP.elementSofSize();

  // add the space required for minimum epsilon
  //
  bytes += epsilon_min_d.sofSize();

  // add the space required for maximum epsilon
  //
  bytes += epsilon_max_d.sofSize();

  // add the space required for the range of epsilon
  //
  bytes += epsilon_res_d.sofSize();
  
  // add space for delta
  //
  bytes += delta_d.sofSize();

  // add the space required for the Theiler's window
  //
  bytes += theiler_correction_d.sofSize();

  // add the space for the base class
  //
  bytes += debug_level_d.sofSize();

  // total things common to specific algorithms
  //

  // check the algorithm:
  //  at this point, there are no algorithm or implementation specific
  //  data to be accounted for
  //

  // return the size
  //
  return bytes;
}

// method: write
//
// arguments:
//  Sof& sof: (input) sof file object
//  int32 tag: (input) sof object instance tag
//  const String& name: (input) sof object instance name
//
// return: a bool8 value indicating status
//
// this method has the object write itself to an Sof file
//
bool8 CorrelationDimension::write(Sof& sof_a, int32 tag_a,
			   const String& name_a) const {

  // declare local variables
  //
  int32 obj_size;

  // check the file type
  //
  if (sof_a.isText()) {
    obj_size = Sof::ANY_SIZE;
  }
  else {
    obj_size = sofSize();
  }
  
  // put the object into the sof file's index
  //
  if (!sof_a.put(name_a, tag_a, obj_size)) {
    return false;
  }

  // exit gracefully
  //
  return writeData(sof_a);
}

// method: writeData
//
// arguments:
//  Sof& sof: (input) sof file object
//  const String& pname: (input) parameter name
//
// return: a bool8 value indicating status
//
// this method branches on the algorithm name
//
bool8 CorrelationDimension::writeData(Sof& sof_a, const String& pname_a) const {

  // declare local variables
  //
  bool8 status = true;
  
  // write a start string if necessary
  //
  sof_a.writeLabelPrefix(pname_a);

  // write the algorithm name
  //
  ALGO_MAP.writeElementData(sof_a, PARAM_ALGORITHM, (int32)algorithm_d);
    
  // write the implementation type
  //
  IMPL_MAP.writeElementData(sof_a, PARAM_IMPLEMENTATION,
			    (int32)implementation_d);

  // write the scaling type
  //
  SCAL_MAP.writeElementData(sof_a, PARAM_SCALING,
			    (int32)scaling_d);
  
  // write the minimum epsilon
  //
  epsilon_min_d.writeData(sof_a, PARAM_EPSILON_MIN);

  // write the maximum epsilon
  //
  epsilon_max_d.writeData(sof_a, PARAM_EPSILON_MAX);

  // write the resolution of epsilon
  //
  epsilon_res_d.writeData(sof_a, PARAM_EPSILON_RES);
  
  // write the order
  //
  delta_d.writeData(sof_a, PARAM_DELTA);

  // write the Theiler's correction
  //
  theiler_correction_d.writeData(sof_a, PARAM_THEILER_CORRECTION);  

  // write debug level (from the base class)
  //
  debug_level_d.writeData(sof_a, PARAM_DBGL);

  // check known algorithms: invoke the proper lower-level method
  //
  if ((algorithm_d == NORMAL)) {   
    status = writeDataCommon(sof_a, pname_a);
  }

  // exit ungracefully
  //
  else {
    return Error::handle(name(), L"writeData",
			 ERR_UNKALG, __FILE__, __LINE__);
  }

  // put an end string if necessary
  //
  sof_a.writeLabelSuffix(pname_a);
  
  // exit gracefully
  //
  return status;
}

// method: writeDataCommon
//
// arguments:
//  Sof& sof: (input) sof file object
//  const String& pname: (input) parameter name
//
// return: a bool8 value indicating status
//
// this method writes data to a file for several algorithms that
// share the same parameters. currently, no additional information
// needs to be written.
//
bool8 CorrelationDimension::writeDataCommon(Sof& sof_a,
				     const String& pname_a) const {

  // exit gracefully
  //
  return true;
}
