// file: $isip_ifc/class/algo/CorrelationEntropy/correntp_04.cc
// version: $Id: correntp_04.cc
//

// isip include files
//
#include "CorrelationEntropy.h"

// method: sofSize
//
// arguments: none
//
// return: size of object
//
// this method returns the size of the object in the Sof file and is
// used for binary write
//
int32 CorrelationEntropy::sofSize() const {
  
  // start with the space required for the algorithm name
  //
  int32 bytes = ALGO_MAP.elementSofSize();

  // add the space required for the implementation name
  //
  bytes += IMPL_MAP.elementSofSize();

  // add the space required for the embedding name
  //
  bytes += EMBED_MAP.elementSofSize();

  // add the space required for  epsilon
  //
  bytes += epsilon_d.sofSize();

  // add the space required for minimum embedding dimension
  //
  bytes += embed_dim_min_d.sofSize();

  // add the space required for maximum embedding dimension
  //
  bytes += embed_dim_max_d.sofSize();

  // add the space required for the Theiler's window
  //
  bytes += theiler_correction_d.sofSize();

  // add the space required for the svd window size 
  //
  bytes += svd_window_size_d.sofSize();

  // add the space required for the time delay 
  //
  bytes += delay_d.sofSize();

  // add the space required for the debug level
  //
  bytes += debug_level_d.sofSize();
  
  // return the size
  //
  return bytes;
}

// method: write
//
// arguments:
//  Sof& sof: (input) sof file object
//  int32 tag: (input) sof object instance tag
//  const String& name: (input) sof object instance name
//
// return: a bool8 value indicating status
//
// this method has the object write itself to an Sof file
//
bool8 CorrelationEntropy::write(Sof& sof_a, int32 tag_a,
			  const String& name_a) const {

  // declare local variable
  //
  int32 obj_size;

  if (sof_a.isText()) {

    // set the size to be dynamic
    //
    obj_size = Sof::ANY_SIZE;
  }
  else {
    
    // set the size to be the size of the object written to the Sof file
    //
    obj_size = sofSize();
  }
  
  // write the object into the sof file's index
  //
  if (!sof_a.put(name_a, tag_a, obj_size)) {
    return false;
  }

  // write data and exit gracefully
  //
  return writeData(sof_a);
}

// method: writeData
//
// arguments:
//  Sof& sof: (input) sof file object
//  const String& pname: (input) parameter name
//
// return: a bool8 value indicating status
//
// this method branches on the algorithm name
//
bool8 CorrelationEntropy::writeData(Sof& sof_a, const String& pname_a) const {

  // declare local variables
  //
  bool8 status = false;
  
  // write a start string if necessary
  //
  sof_a.writeLabelPrefix(pname_a);

  // write the algorithm name
  //
  ALGO_MAP.writeElementData(sof_a, PARAM_ALGORITHM, (int32)algorithm_d);

  // invoke the proper lower-level method
  //
  if (algorithm_d == NORMAL) {
    status = writeDataCommon(sof_a, pname_a);
  }

  // exit ungracefully
  //
  else {
    return Error::handle(name(), L"writeData",
			 ERR_UNKALG, __FILE__, __LINE__);
  }

  // put an end string if necessary
  //
  sof_a.writeLabelSuffix(pname_a);
  
  // exit gracefully
  //
  return status;
}

// method: writeDataCommon
//
// arguments:
//  Sof& sof: (input) sof file object
//  const String& pname: (input) parameter name
//
// return: a bool8 value indicating status
//
// this method writes data to a file for several algorithms that
// share the same parameters
//
bool8 CorrelationEntropy::writeDataCommon(Sof& sof_a,
				    const String& pname_a) const {

  // write the implementation type
  //
  IMPL_MAP.writeElementData(sof_a, PARAM_IMPLEMENTATION,
			    (int32)implementation_d);

  // write the embedding type
  //
  EMBED_MAP.writeElementData(sof_a, PARAM_EMBEDDING,
			    (int32)embedding_d);

  // write the value of epsilon
  //
  epsilon_d.writeData(sof_a, PARAM_EPSILON);

  // write the minimum embedding dimension
  //
  embed_dim_min_d.writeData(sof_a, PARAM_EMBED_DIM_MIN);

  // write the maximum embedding dimension
  //
  embed_dim_max_d.writeData(sof_a, PARAM_EMBED_DIM_MAX);

  // write the Theiler's correction
  //
  theiler_correction_d.writeData(sof_a, PARAM_THEILER_CORRECTION);

  // write the value of delay
  //
  delay_d.writeData(sof_a, PARAM_DELAY);

  // write the value of SVD window size
  //
  svd_window_size_d.writeData(sof_a, PARAM_SVD_WINDOW_SIZE);

  // write the debug level (from the base class)
  //
  debug_level_d.writeData(sof_a, PARAM_DBGL);

  // exit gracefully
  //
  return true;
}
