// file: $isip/class/algo/DisplayData/dspd_04.cc
// version: $Id: dspd_04.cc 6978 2001-05-11 14:47:44Z duncan $
//

// isip include files
//
#include "DisplayData.h"

// method: sofSize
//
// arguments: none 
//
// return: size of the object
//
// this method returns the size of the object
//
int32 DisplayData::sofSize() const {

  int32 bytes = ALGO_MAP.elementSofSize();
  bytes += IMPL_MAP.elementSofSize();

  bytes += number_d.sofSize();

  bytes += label_d.sofSize();
  bytes += prefix_d.sofSize();
  bytes += suffix_d.sofSize();

  bytes += debug_level_d.sofSize();  

  // return the size
  //
  return bytes;
}

// method: write
//
// arguments:
//  Sof& sof: (input) sof file object
//  int32 tag: (input) sof object instance tag
//  const String& name: (input) sof object instance name
//
// return: a bool8 value indicating status
//
// this method has the object write itself, with specified name and
// tag, to an Sof file
//
bool8 DisplayData::write(Sof& sof_a, int32 tag_a,
				const String& name_a) const {

  // declare local variables
  //
  int32 obj_size = 0;
  
  // write the instance of the object into the Sof file
  //
  if (sof_a.isText()) {

    // set the size to by dynamic
    //
    obj_size = Sof::ANY_SIZE;
  }
  else {

    // the size index and the size of each element
    //
    obj_size = sofSize();
  }

  // put the object into the sof file's index
  //
  if (!sof_a.put(name_a, tag_a, obj_size)) {
    return false;
  }
  
  // exit gracefully
  //
  return writeData(sof_a);
}

// method: writeData
//
// arguments:
//  Sof& sof: (input) sof file object
//  const String& pname: (input) parameter name
//
// return: a bool8 value indicating status
//
// this method has the object read itself from an Sof file. it assumes
// that the Sof file is already positioned correctly.
//
bool8 DisplayData::writeData(Sof& sof_a,
			    const String& pname_a) const {

  // write a start string if necessary
  //
  sof_a.writeLabelPrefix(pname_a);

  // write the type
  //
  ALGO_MAP.writeElementData(sof_a, PARAM_ALGORITHM, (int32)algorithm_d);

  // write the type
  //
  IMPL_MAP.writeElementData(sof_a, PARAM_IMPLEMENTATION,
			    (int32)implementation_d);

  // write the number
  //
  number_d.writeData(sof_a, PARAM_NUMBER);

  // write the label
  //
  label_d.writeData(sof_a, PARAM_LABEL);

  // write the prefix
  //
  prefix_d.writeData(sof_a, PARAM_PREFIX);

  // write the suffix
  //
  suffix_d.writeData(sof_a, PARAM_SUFFIX);

  // write debug level
  //
  debug_level_d.writeData(sof_a, PARAM_DBGL);
  
  // put an end string if necessary
  //
  sof_a.writeLabelSuffix(pname_a);
  
  // exit gracefully
  //
  return true;
}
