// file: $isip_ifc/class/algo/DynamicFilter/lyap_00.cc
// version: $Id: lyap_00.cc 10485 2006-03-15 17:22:34Z srinivas $
//

// isip include files
//
#include "DynamicFilter.h"

//------------------------------------------------------------------------
//
// required public methods
//
//-----------------------------------------------------------------------

// method: clear
//
// arguments:
//  Integral::CMODE ctype: (input) clear mode
//
// return: a bool8 value indicating status
//
// this method resets the data members to the default values
//
bool8 DynamicFilter::clear(Integral::CMODE ctype_a) {

  // reset all protected data
  //
  if (ctype_a != Integral::RETAIN) {
    algorithm_d = DEF_ALGORITHM;
    implementation_d = DEF_IMPLEMENTATION;
  }
  
  // call the base clear method
  //
  return AlgorithmBase::clear(ctype_a);
}

//---------------------------------------------------------------------------
//
// class-specific public methods:
//  public methods required by the AlgorithmBase interface contract
//
//---------------------------------------------------------------------------

// method: assign
//
// arguments:
//  const AlgorithmBase& arg: (input) object to be assigned
//
// return: a bool8 value indicating status
//
// this method assigns the input algorithm object to the current
// DynamicFilter object, if the input algorithm object is not an DynamicFilter
// object, it returns an error
//
bool8 DynamicFilter::assign(const AlgorithmBase& arg_a) {

  // case: input is a DynamicFilter object
  //
  if (typeid(arg_a) == typeid(DynamicFilter)) {
    return assign((DynamicFilter&)arg_a);
  }

  // case: other
  //  if the input algorithm object is not a DynamicFilter object, return
  //  an error.
  //    
  else {
    return Error::handle(name(), L"assign", Error::ARG, __FILE__, __LINE__);
  }
}

// method: eq
//
// arguments:
//  const AlgorithmBase& arg: (input) object to be compared
//
// return: a bool8 value indicating status
//
// this method checks whether the current DynamicFilter object is identical
// to the input algorithm object, if the input algorithm object is not
// an DynamicFilter object, it returns an error
//
bool8 DynamicFilter::eq(const AlgorithmBase& arg_a) const {
  
  // case: input is a DynamicFilter object
  //
  if (typeid(arg_a) == typeid(DynamicFilter)) {
    return eq((DynamicFilter&)arg_a);
  }

  // case: other
  //  if the input algorithm object is not a DynamicFilter object, return
  //  an error.
  //    
  else {
    return Error::handle(name(), L"eq", Error::ARG, __FILE__, __LINE__);
  }
}

//-----------------------------------------------------------------------------
//
// we define non-integral constants in the default constructor
//      
//-----------------------------------------------------------------------------

// constants: class name
//
const String DynamicFilter::CLASS_NAME(L"DynamicFilter");

// constants: i/o related constants
//
const String DynamicFilter::DEF_PARAM(L"");
const String DynamicFilter::PARAM_ALGORITHM(L"algorithm");
const String DynamicFilter::PARAM_IMPLEMENTATION(L"implementation");
const String DynamicFilter::PARAM_NUM_NEIGHBORS(L"num_neighbors");
const String DynamicFilter::PARAM_ALPHA_MIN(L"alpha_min");
const String DynamicFilter::PARAM_ALPHA_MAX(L"alpha_max");
const String DynamicFilter::PARAM_NUM_ITERATIONS(L"num_iterations");
const String DynamicFilter::PARAM_NUM_EIGENS(L"num_eigens");

// constants: NameMap(s) for the enumerated values
//
const NameMap DynamicFilter::ALGO_MAP(L"NORMAL, SVD_CLEAN");
const NameMap DynamicFilter::IMPL_MAP(L"REGULAR");

// static instantiations: memory manager
//
MemoryManager DynamicFilter::mgr_d(sizeof(DynamicFilter), DynamicFilter::name());


