// file: $isip_ifc/class/algo/DynamicFilter/lyap_02.cc
// version: $Id: lyap_02.cc 10485 2006-03-15 17:22:34Z srinivas $
//

// isip include files
//
#include "DynamicFilter.h"
#include <Rps.h>
#include <AudioFile.h>
#include <Console.h>

// method: diagnose
//
// arguments:
//  Integral::DEBUG level: (input) debug level for diagnostics
//
// return: a bool8 value indicating status
//

bool8 DynamicFilter::diagnose(Integral::DEBUG level_a) {
  
  //---------------------------------------------------------------------------
  //
  // 0. preliminaries
  //
  //---------------------------------------------------------------------------

  // output the class name
  //
  if (level_a > Integral::NONE) {
    SysString output(L"diagnosing class ");
    output.concat(CLASS_NAME);
    output.concat(L": ");
    Console::put(output);
    Console::increaseIndention();
  }

  // --------------------------------------------------------------------
  //
  // 1. required public methods
  //
  // --------------------------------------------------------------------
 
  // set indentation
  //
  if (level_a > Integral::NONE) {
    Console::put(L"testing required public methods...\n");
    Console::increaseIndention();
  }

  DynamicFilter dynfilt_1(SVD_CLEAN, DEF_IMPLEMENTATION, (int32)10, 
			  (float32)0.1, (float32)1.0, (int32)20, (int32)2);
  DynamicFilter dynfilt_2(dynfilt_1);
  
  if (!dynfilt_2.eq(dynfilt_1)) {
    return Error::handle(name(), L"copy constructor/eq", Error::TEST,
			 __FILE__, __LINE__);
  }


  // testing large chunk of memory allocation
  //
  DynamicFilter::setGrowSize((int32)500);

  for (int32 j = 1; j <= 10; j++) {
    DynamicFilter** dynfilt = new DynamicFilter*[j * 100];
    
    // create the objects
    //
    for (int32 i = 0; i < j * 100; i++) {
      dynfilt[i] = new DynamicFilter();
    }
    
    // delete objects
    //
    for (int32 i = (j * 100) - 1; i >= 0; i--) {
      delete dynfilt[i];
    }
      
    delete [] dynfilt;
  } 

  // test the i/o methods
  //
  DynamicFilter dynfilt_3(SVD_CLEAN, DEF_IMPLEMENTATION, (int32)10, (float32)0.1, (float32)1.0, (int32)20, (int32)2);
  DynamicFilter dynfilt_4;
  DynamicFilter dynfilt_5;

  // we need binary and text sof files
  //
  String tmp_filename0;
  Integral::makeTemp(tmp_filename0);
  String tmp_filename1;
  Integral::makeTemp(tmp_filename1);

  // open files in write mode
  //
  Sof tmp_file0;
  tmp_file0.open(tmp_filename0, File::WRITE_ONLY, File::TEXT);
  Sof tmp_file1;
  tmp_file1.open(tmp_filename1, File::WRITE_ONLY, File::BINARY);

  dynfilt_3.write(tmp_file0, (int32)0);
  dynfilt_3.write(tmp_file1, (int32)0);

  // close the files
  //
  tmp_file0.close();
  tmp_file1.close();

  // open the files in read mode
  //
  tmp_file0.open(tmp_filename0);
  tmp_file1.open(tmp_filename1);

  // read the value back
  //
  if (!dynfilt_4.read(tmp_file0, (int32)0) || !dynfilt_4.eq(dynfilt_3)) {
    return Error::handle(name(), L"read", Error::TEST, __FILE__, __LINE__);
  }
  
  if (!dynfilt_5.read(tmp_file1, (int32)0) || !dynfilt_5.eq(dynfilt_3)) {
    return Error::handle(name(), L"read", Error::TEST, __FILE__, __LINE__);
  }
  
  // close and  delete the temporary files
  //
  tmp_file0.close();
  tmp_file1.close();

  File::remove(tmp_filename0);
  File::remove(tmp_filename1);

  // reset indentation
  //
  if (level_a > Integral::NONE) {
    Console::decreaseIndention();
  }
  
  // --------------------------------------------------------------------
  //
  // 2. class specific public methods:
  //     set and get methods
  //
  // --------------------------------------------------------------------
  
  // set indentation
  //
  if (level_a > Integral::NONE) {  
    Console::put(L"testing class-specific public methods: set and get methods...\n");
    Console::increaseIndention();
  }
  

  DynamicFilter dynfilt_6;

  // set and get the number of iterations
  //
  dynfilt_6.setNumIterations((int32)10);
  if (dynfilt_6.getNumIterations() != (int32)10) {
    return Error::handle(name(), L"set/get number of iterations", Error::TEST,
			 __FILE__, __LINE__);
  }

  // set and get the number of neighbors
  //
  dynfilt_6.setNumNeighbors((int32)25);
  if (dynfilt_6.getNumNeighbors() != (int32)25) {
    return Error::handle(name(), L"set/get number of nearest neighbors", Error::TEST,
			 __FILE__, __LINE__);
  }

  // set and get the maximum alpha factor
  //
  dynfilt_6.setMaxAlpha((int32)1.0);
  if (dynfilt_6.getMaxAlpha() != (int32)1.0) {
    return Error::handle(name(), L"set/get maximum alpha factor", Error::TEST,
			 __FILE__, __LINE__);
  }

  // set and get the minimum alpha factor
  //
  dynfilt_6.setMinAlpha((int32)1.0);
  if (dynfilt_6.getMinAlpha() != (int32)1.0) {
    return Error::handle(name(), L"set/get minimum alpha factor", Error::TEST,
			 __FILE__, __LINE__);
  }

  // set and get the number of eigen values to use
  //
  dynfilt_6.setNumEigens((int32)25);
  if (dynfilt_6.getNumEigens() != (int32)25) {
    return Error::handle(name(), L"set/get number of eigen values", Error::TEST,
			 __FILE__, __LINE__);
  }

  // reset indentation
  //
  if (level_a > Integral::NONE) {
    Console::decreaseIndention();
  }
  
  // --------------------------------------------------------------------
  //
  // 3. class-specific public methods
  //     computation methods
  //
  // --------------------------------------------------------------------

  // set indentation
  //
  if (level_a > Integral::NONE) {
    Console::put(L"testing class-specific public methods: computation methods...\n");
    Console::increaseIndention();
  }

  AudioFile testdata;
  testdata.open(L"./noisy_sine.sof");

  Vector<VectorFloat> data;
  int32 current_sample = 0;

  testdata.readAudioData(data, 0, current_sample, 750);

  testdata.close();

  Rps reconstruct(Rps::TIME_DELAY_EMBEDDING, Rps::EMBEDDING, 3, 
		  Rps::DEF_SVD_WINDOW_SIZE, 0.0063, Rps::DEF_SVD_THRESHOLD);

  MatrixFloat reconstructed_data, filtered_data;
  
  // reconstruct data
  //
  reconstruct.compute(reconstructed_data, data(0));
  
  DynamicFilter d_filter(DynamicFilter::SVD_CLEAN, DynamicFilter::REGULAR, 10, 
			 0.1, 1.0, 10, 1);
  
  d_filter.compute(filtered_data, reconstructed_data);
  
  
  // reset indentation
  //
  if (level_a > Integral::NONE) {
    Console::decreaseIndention();
  }
  

  //---------------------------------------------------------------------------
  //
  // 4. print completion message
  //
  //---------------------------------------------------------------------------

  // reset indentation
  //
  if (level_a > Integral::NONE) {
    Console::decreaseIndention();
  }
  
  if (level_a > Integral::NONE) {
    SysString output(L"diagnostics passed for class ");
    output.concat(name());
    output.concat(L"\n");
    Console::put(output);
  }

  // exit gracefully
  //
  return true;
}
