// file: $isip_ifc/class/algo/MutualInformation/mi_04.cc
// version: $Id: mi_04.cc 10436 2006-02-23 20:12:47Z pannuri $
//

// isip include files
//
#include "MutualInformation.h"

// method: sofSize
//
// arguments: none
//
// return: size of object
//
// this method returns the size of the object in the Sof file and is
// used for binary write
//
int32 MutualInformation::sofSize() const {

  // start with the space required for the algorithm name
  //
  int32 bytes = ALGO_MAP.elementSofSize();
  
  // add the space required for the implementation name
  //
  bytes += IMPL_MAP.elementSofSize();
   
  // add the space required for the normalization name
  //
  bytes += NORM_MAP.elementSofSize();
  
  // add the space required for the order
  //
  bytes += order_d.sofSize();
  
  // add the space required for number of bins
  //
  bytes += num_bins_d.sofSize();
  
  // add the space for the base class
  //
  bytes += debug_level_d.sofSize();

  // check the algorithm:
  //  at this point, there are no algorithm or implementation specific
  //  data to be accounted for
  //
  
  // return the size
  //
  return bytes;
}

// method: write
//
// arguments:
//  Sof& sof: (input) sof file object
//  int32 tag: (input) sof object instance tag
//  const String& name: (input) sof object instance name
//
// return: a bool8 value indicating status
//
// this method has the object write itself to an Sof file
//
bool8 MutualInformation::write(Sof& sof_a, int32 tag_a,
				 const String& name_a) const {
  
  // declare local variables
  //
  int32 obj_size;
  
  // check the file type
  //
  if (sof_a.isText()) {

    // set the size to be dynamic
    //
    obj_size = Sof::ANY_SIZE;
  }
  else {

    // set the size to be the size of object written to the Sof file
    //
    obj_size = sofSize();
  }
  
  // put the object into the sof file's index
  //
  if (!sof_a.put(name_a, tag_a, obj_size)) {
    return false;
  }
  
  // exit gracefully
  //
  return writeData(sof_a);
}

// method: writeData
//
// arguments:
//  Sof& sof: (input) sof file object
//  const String& pname: (input) parameter name
//
// return: a bool8 value indicating status
//
// this method branches on the algorithm name
//
bool8 MutualInformation::writeData(Sof& sof_a, const String& pname_a) const {

  // declare local variables
  //
  bool8 status = true;
  
  // write a start string if necessary
  //
  sof_a.writeLabelPrefix(pname_a);
  
  // write the algorithm name
  //
  ALGO_MAP.writeElementData(sof_a, PARAM_ALGORITHM, (int32)algorithm_d);
  
  // check known algorithms: invoke the proper lower-level method
  //
  if ((algorithm_d == AUTO)) {
    status = writeDataCommon(sof_a, pname_a);
  }

  // exit ungracefully
  //
  else {
    return Error::handle(name(), L"writeData",
			 ERR_UNKALG, __FILE__, __LINE__);
  }

  // put an end string if necessary
  //
  sof_a.writeLabelSuffix(pname_a);
  
  // exit gracefully
  //
  return status;
}

// method: writeDataCommon
//
// arguments:
//  Sof& sof: (input) sof file object
//  const String& pname: (input) parameter name
//
// return: a bool8 value indicating status
//
// this method writes data to a file for several algorithms that
// share the same parameters. currently, no additional information
// needs to be written.
//
bool8 MutualInformation::writeDataCommon(Sof& sof_a,
				     const String& pname_a) const {
  // write the implementation type
  //
  IMPL_MAP.writeElementData(sof_a, PARAM_IMPLEMENTATION,
			    (int32)implementation_d);

  // write the order
  //
  order_d.writeData(sof_a, PARAM_ORDER);

  // write the number of bins
  //
  num_bins_d.writeData(sof_a, PARAM_NUM_BINS);

  // write the debug level (from the base class)
  //
  debug_level_d.writeData(sof_a, PARAM_DBGL);

  // exit gracefully
  //
  return true;
}
