// file: $isip_ifc/class/algo/MutualInformation/mi_06.cc
// version: $Id: mi_06.cc 10436 2006-02-23 20:12:47Z pannuri $
//

// isip include files
//
#include "MutualInformation.h"

// method: getLeadingPad
//
// arguments: none 
//
// return: a int32 value indicating leading pad (in units of frames)
//
int32 MutualInformation::getLeadingPad() const {

  // in FRAME_INTERNAL mode, we only deal with the current frame
  //
  if (cmode_d == FRAME_INTERNAL) {
    return DEF_LEADING_PAD;
  }
  
  // in CROSS_FRAME mode, we need to compute the overlap.
  //
  else if (cmode_d == CROSS_FRAME) {

    // only algorithm_d == CROSS requires a leading pad. all other
    // algorithms only require access to previous data.
    //
    if (algorithm_d == CROSS) {
      float32 overlap = (float32)order_d / (frame_dur_d * sample_freq_d);
      return (int32)Integral::ceil(overlap);
    }
    else
      return DEF_LEADING_PAD;      
  }

  // else: exit ungracefully
  //
  else {
    return Error::handle(name(), L"getLeadingPad",
			 ERR_UNSUPM, __FILE__, __LINE__);
  }
}

// method: getTrailingPad
//
// arguments: none 
//
// return: a int32 value indicating the trailing pad (in units of frames)
//
int32 MutualInformation::getTrailingPad() const {

  // in FRAME-INTERNAL mode, we only deal with the current frame
  //
  if (cmode_d == FRAME_INTERNAL) {
    return DEF_TRAILING_PAD;
  }

  // in CROSS_FRAME mode, we need to compute the overlap.
  //
  else if (cmode_d == CROSS_FRAME) {

    // only two algorithms (AUTO and CONV) need a leading pad
    //
    if (algorithm_d == AUTO) {
      float32 overlap = (float32)order_d / (frame_dur_d*sample_freq_d);
      return (int32)Integral::ceil(overlap);
    }
    else
      return DEF_TRAILING_PAD;      
  }    

  // else: exit ungracefully
  //
  else {
    return Error::handle(name(), L"getTrailingPad",
			 ERR_UNSUPM, __FILE__, __LINE__);
  }
}
