// file: $isip/class/algo/Output/out_04.cc
// version: $Id: out_04.cc 9236 2003-06-23 18:12:22Z gao $
//

// isip include files
//
#include "Output.h"

// method: sofSize
//
// arguments: none 
//
// return: size of object
//
// this method returns the size of the object in the Sof file and is
// used for binary write
//
int32 Output::sofSize() const {

  // start with the space required for the algorithm name
  //
  int32 bytes = ALGO_MAP.elementSofSize();

  // add the space required for the implementation name
  //
  bytes += IMPL_MAP.elementSofSize();

  // add the space required for the data mode name
  //
  bytes += DMODE_MAP.elementSofSize();

  // add the space required for filename information
  //
  bytes += output_directory_d.sofSize();
  bytes += output_basename_d.sofSize();    
  bytes += output_extension_d.sofSize();  
  bytes += output_preserve_d.sofSize();
  bytes += output_suffix_d.sofSize();
  
  // add the space required for the audiofiles or featurefile
  //
  if (implementation_d == SAMPLED_DATA)
    bytes += audio_output_d.sofSize();
  else
    bytes += feature_output_d.sofSize();

  // add the space required for the debug level
  //
  bytes += debug_level_d.sofSize();

  // check the algorithm:
  //  at this point, there are no algorithm or implementation specific
  //  data to be accounted for
  //

  // return the size
  //
  return bytes;
}

// method: write
//
// arguments:
//  Sof& sof: (input) sof file object
//  int32 tag: (input) sof object instance tag
//  const String& name: (input) sof object instance name
//
// return: a bool8 value indicating status
//
// this method has the object write itself to an Sof file
//
bool8 Output::write(Sof& sof_a, int32 tag_a, const String& name_a) const {

  // declare local variable
  //
  int32 obj_size;
  
  // write the instance of the object into the Sof file
  //
  if (sof_a.isText()) {

    // set the size to be dynamic
    //
    obj_size = Sof::ANY_SIZE;
  }
  else {

    // set the size to be the size of the object written to the Sof file
    //
    obj_size = sofSize();
  }

  // write the object into the sof file's index
  //
  if (!sof_a.put(name_a, tag_a, obj_size)) {
    return false;
  }
  
  // write data and exit gracefully
  //
  return writeData(sof_a);
}

// method: writeData
//
// arguments:
//  Sof& sof: (input) sof file object
//  const String& pname: (input) parameter name
//
// return: a bool8 value indicating status
//
// this method branches on the algorithm name
//
bool8 Output::writeData(Sof& sof_a, const String& pname_a) const {
  
  // declare local variables
  //
  bool8 status = false;
  
  // write a start string if necessary
  //
  sof_a.writeLabelPrefix(pname_a);

  // write the algorithm name
  //
  ALGO_MAP.writeElementData(sof_a, PARAM_ALGORITHM, (int32)algorithm_d);
    
  // write the implementation type
  //
  IMPL_MAP.writeElementData(sof_a, PARAM_IMPLEMENTATION,
			    (int32)implementation_d);
  
  // write the output file directory
  //
  output_directory_d.writeData(sof_a, CommandLine::DEF_PARAM_DIRECTORY);  

  // write the file base name
  //
  output_basename_d.writeData(sof_a, PARAM_BASENAME);

  // write the file extension
  //
  output_extension_d.writeData(sof_a, CommandLine::DEF_PARAM_EXTENSION);  

  // write the directory preserve level
  //
  output_preserve_d.writeData(sof_a, CommandLine::DEF_PARAM_PRESERVE);

  // write the file suffix
  //
  output_suffix_d.writeData(sof_a, CommandLine::DEF_PARAM_SUFFIX);
  
  if (implementation_d == FEATURES) {
    feature_output_d.writeData(sof_a, PARAM_FEATURE_OUTPUT);
  }
  else {
    // write the mode type
    //
    DMODE_MAP.writeElementData(sof_a, PARAM_DMODE, (int32)dmode_d);
    
    audio_output_d.writeData(sof_a, PARAM_AUDIO_OUTPUT);
  }
  
  // write the debug level (from the base class)
  //
  debug_level_d.writeData(sof_a, PARAM_DBGL);  

  // check known algorithms: SUM
  //
  if (algorithm_d == DATA) {
    status = writeDataData(sof_a, pname_a);
  }

  // check unsupported algorithms
  //
  else {
    return Error::handle(name(), L"writeData",
			 ERR_UNKALG, __FILE__, __LINE__);
  }

  // put an end string if necessary
  //
  sof_a.writeLabelSuffix(pname_a);
  
  // exit gracefully
  //
  return status;
}

// method: writeDataData
//
// arguments:
//  Sof& sof: (input) sof file object
//  const String& pname: (input) parameter name
//
// return: a bool8 value indicating status
//
// this method writes data to a file for the DATA algorithm. note there
// is no additional data that needs to be written.
//
bool8 Output::writeDataData(Sof& sof_a, const String& pname_a) const {
  // exit gracefully
  //
  
  return true;
}
