// file: $isip_ifc/class/algo/ParticleFilter/pf_04.cc
// version: $Id: pf_04.cc 10352 2006-01-09 15:36:23Z patil $
//

// isip include files
//
#include "ParticleFilter.h"

// method: sofSize
//
// arguments: none 
//
// return: size of object
//
// this method returns the size of the object in the Sof file and is
// used for binary write
//
int32 ParticleFilter::sofSize() const {

  // start with the space required for the algorithm name
  //
  int32 bytes = ALGO_MAP.elementSofSize();

  // add the space required for the implementation name
  //
  bytes += IMPL_MAP.elementSofSize();

  // add the space required for the debug level
  //
  bytes += debug_level_d.sofSize();

  // add the space required for the initial state
  //
  bytes += state_mem_d.sofSize();

  // add the space required for the transition model
  //
  bytes += trn_d.sofSize();  

  // add the space required for the observation model
  //
  bytes += obs_d.sofSize();

//   // add the space required for the states 
//   //
//   bytes += states_d.sofSize();

  // add the space required for the observation 
  //
  bytes += particles_d.sofSize();
  
   // add the space required for the observation 
  //
  bytes += order_d.sofSize();
  
 // return the size
  //
  return bytes;
}

// method: write
//
// arguments:
//  Sof& sof: (input) sof file object
//  int32 tag: (input) sof object instance tag
//  const String& name: (input) sof object instance name
//
// return: a bool8 value indicating status
//
// this method has the object write itself to an Sof file
//
bool8 ParticleFilter::write(Sof& sof_a, int32 tag_a,
			    const String& name_a) const {

  // declare local variable
  //
  int32 obj_size;
  
  // write the instance of the object into the Sof file
  //
  if (sof_a.isText()) {

    // set the size to be dynamic
    //
    obj_size = Sof::ANY_SIZE;
  }
  else {

    // set the size to be the size of the object written to the Sof file
    //
    obj_size = sofSize();
  }

  // write the object into the sof file's index
  //
  if (!sof_a.put(name_a, tag_a, obj_size)) {
    return false;
  }

  // write data and exit gracefully
  //
  return writeData(sof_a);
}

// method: writeData
//
// arguments:
//  Sof& sof: (input) sof file object
//  const String& pname: (input) parameter name
//
// return: a bool8 value indicating status
//
// this method branches on the algorithm name
//
bool8 ParticleFilter::writeData(Sof& sof_a, const String& pname_a) const {
  
  // declare local variables
  //
  bool8 status = false;
  
  // write a start string if necessary
  //
  sof_a.writeLabelPrefix(pname_a);

  // write the algorithm name
  //
  ALGO_MAP.writeElementData(sof_a, PARAM_ALGORITHM, (int32)algorithm_d);
    
  // write the implementation type
  //
  IMPL_MAP.writeElementData(sof_a, PARAM_IMPLEMENTATION,
			    (int32)implementation_d);
  
  // write the debug level (from the base class)
  //
  debug_level_d.writeData(sof_a, PARAM_DBGL);

  // check known algorithms: CONVENTIONAL
  //
  if ((algorithm_d == CONVENTIONAL) && (implementation_d == AR_SCALAR)) {
    status = writeDataParticleFilter(sof_a, pname_a);
  }

  // check unsupported algorithms
  //
  else {
    return Error::handle(name(), L"writeData",
			 ERR_UNKALG, __FILE__, __LINE__);
  }

  // put an end string if necessary
  //
  sof_a.writeLabelSuffix(pname_a);
  
  // exit gracefully
  //
  return status;
}

// method: writeDataParticleFilter
//
// arguments:
//  Sof& sof: (input) sof file object
//  const String& pname: (input) parameter name
//
// return: a bool8 value indicating status
//
// this method writes data to a file for the CONVENTIONAL algorithm
//
bool8 ParticleFilter::writeDataParticleFilter(Sof& sof_a,
				       const String& pname_a) const {

  // write the order
  //
  order_d.writeData(sof_a, PARAM_ORDER);
  
  // write the order
  //
  particles_d.writeData(sof_a, PARAM_PARTICLES);
  
  // exit gracefully
  //
  return true;
}
