// file: $isip/class/algo/Statistics/stat_04.cc
// version: $Id: stat_04.cc 8165 2002-05-31 21:57:33Z picone $
//

// isip include files
//
#include "Statistics.h"
#include <Sof.h>
#include <String.h>

// method: sofSize
//
// arguments: none
//
// return: size of object
//
// this method returns the size of the object in the Sof file and is
// used for binary write
//
int32 Statistics::sofSize() const {
  
  // start with the space required for the algorithm name
  //
  int32 bytes = ALGO_MAP.elementSofSize();

  // add the space required for the mode name
  //
  bytes += DMODE_MAP.elementSofSize();

  // add the space required for the computation mode name
  //
  bytes += CMODE_MAP.elementSofSize();
  
  // add the space required for the implementation name
  //
  bytes += IMPL_MAP.elementSofSize();

  // add the space for the base class
  //
  bytes += debug_level_d.sofSize();

  // return the size
  //
  return bytes;
}

// method: write
//
// arguments:
//  Sof& sof: (input) sof file object
//  int32 tag: (input) sof object instance tag
//  const String& name: (input) sof object instance name
//
// return: a bool8 value indicating status
//
// this method has the object write itself to an Sof file
//
bool8 Statistics::write(Sof& sof_a,  int32 tag_a,
			  const String& name_a) const {

  // declare local variables
  //
  int32 obj_size = 0;

  // check the file type
  //
  if (sof_a.isText()) {
    obj_size = Sof::ANY_SIZE;
  }
  else {
    obj_size = sofSize();
  }
  
  // put the object into the sof file's index
  //
  if (!sof_a.put(name_a, tag_a, obj_size)) {
    return false;
  }

  // write data and exit gracefully
  //
  return writeData(sof_a);

}

// method: writeData
//
// arguments:
//  Sof& sof: (input) sof file object
//  const String& pname: (input) parameter name
//
// return: a bool8 value indicating status
//
// this method branches on the algorithm name
//
bool8 Statistics::writeData(Sof& sof_a, const String& pname_a) const {
  
  // declare local variables
  //
  bool8 status = false;
  
  // write a start string if necessary
  //
  sof_a.writeLabelPrefix(pname_a);

  // write the algorithm name
  //
  ALGO_MAP.writeElementData(sof_a, PARAM_ALGORITHM, (int32)algorithm_d);
    
  // check known algorithms: invoke the proper lower-level method
  //
  if ((algorithm_d == MINIMUM) ||
      (algorithm_d == MAXIMUM) ||
      (algorithm_d == MINIMUM_MAG) ||
      (algorithm_d == MAXIMUM_MAG) ||
      (algorithm_d == MEAN) ||
      (algorithm_d == MEDIAN) ||
      (algorithm_d == VARIANCE) ||
      (algorithm_d == STDEV) ||
      (algorithm_d == SKEW) ||
      (algorithm_d == KURTOSIS)) {
    status = writeDataCommon(sof_a, pname_a);
  }

  // exit ungracefully
  //
  else {
    return Error::handle(name(), L"writeData",
			 ERR_UNKALG, __FILE__, __LINE__);
  }

  // put an end string if necessary
  //
  sof_a.writeLabelSuffix(pname_a);
  
  // exit gracefully
  //
  return status;
}

// method: writeDataCommon
//
// arguments:
//  Sof& sof: (input) sof file object
//  const String& pname: (input) parameter name
//
// return: a bool8 value indicating status
//
// this method writes data to a file for several algorithms that
// share the same parameters
//
bool8 Statistics::writeDataCommon(Sof& sof_a, const String& pname_a) const {
  
  // write the implementation type
  //
  IMPL_MAP.writeElementData(sof_a, PARAM_IMPLEMENTATION, (int32)implementation_d);
  
  // write the compute mode
  //
  CMODE_MAP.writeElementData(sof_a, PARAM_CMODE, (int32)cmode_d);

  // write the data mode
  //
  DMODE_MAP.writeElementData(sof_a, PARAM_DMODE, (int32)dmode_d);

  // write the debug level (from the base class)
  //
  debug_level_d.writeData(sof_a, PARAM_DBGL);

  // exit gracefully
  //
  return true;
}
