// file: $isip/class/mmedia/Annotation/anno_04.cc
// version: $Id: anno_04.cc 8830 2002-11-24 16:43:55Z gao $
//

// isip include files
//
#include "Annotation.h"

// method: write
//
// arguments:
//  Sof& sof: (input) sof file object
//  int32 tag: (input) sof object instance tag
//  const String& name: (input) sof object instance name
//
// return: bool8 value indicating status
//
// this method has the object write itself to an Sof file
//
bool8 Annotation::write(Sof& sof_a, int32 tag_a,
		       const String& name_a) const {

  int32 obj_size = 0;
  
  // write the instance of the object into the Sof file
  //
  if (sof_a.isText()) {

    // set the size to by dynamic
    //
    obj_size = Sof::ANY_SIZE;
  }
  else {

    // the size index and the size of each element
    //
    obj_size = sofSize();
  }

  // write the object into the sof file's index
  //
  if (!sof_a.put(name_a, tag_a, obj_size)) {
    return false;
  }
  
  // write the primitive member data
  //
  return writeData(sof_a);
}

// method: writeData
//
// arguments:
//  Sof& sof: (input) sof file object
//  const String& pname: (input) parameter name
//
// return: bool8 value indicating status
//
// this method has the object write itself to an Sof file. it assumes
// that the Sof file is already positioned correctly.
//
bool8 Annotation::writeData(Sof& sof_a, const String& pname_a) const {
  
  // write a start string if necessary
  //
  sof_a.writeLabelPrefix(pname_a);

  // write the annotation id
  //
  id_d.writeData(sof_a, PARAM_ID);
  
  // write the annotation type
  //
  type_d.writeData(sof_a, PARAM_TYPE);
  
  // write the channel index
  //
  channel_index_d.writeData(sof_a, PARAM_CHANNEL_INDEX);
  
  // write the annotation features
  //  
  feature_map_d.writeData(sof_a, PARAM_FEATURES);
  
  // put an end string if necessary
  //
  sof_a.writeLabelSuffix(pname_a);
  
  // exit gracefully
  //
  return true;
}
