// file: $isip/class/mmedia/TranscriptionDatabase/trans_03.cc
// version: $Id: trans_03.cc 9421 2004-03-10 20:11:33Z parihar $
//

// isip include files
//
#include "TranscriptionDatabase.h"

// method: read
//
// arguments:
//  Sof& sof: (input) sof file object
//  int32 tag: (input) sof object instance tag
//  const String& name: (input) sof object instance name
//
// return: a bool8 value indicating status
//
// this method has the object read itself from an Sof file
//
bool8 TranscriptionDatabase::read(Sof& sof_a, int32 tag_a, const String& name_a) {

  // get the instance of the object from the Sof file
  //
  if (!sof_a.find(name_a, tag_a)) {
    return false;
  }

  // read the actual data from the sof file
  //
  if (!readData(sof_a)) {
    return false;
  }

  // exit gracefully
  //
  return true;
}

// method: readData
//
// arguments:
//  Sof& sof: (input) sof file object
//  const String& pname: (input) parameter name
//  int32 size: (input) size in bytes of object (or FULL_SIZE)
//  bool8 param: (input) is the parameter specified?
//  bool8 nested: (input) is this nested?
//
// return: a bool8 value indicating status
//
// this method has the object read itself from an Sof file. it assumes
// that the Sof file is already positioned correctly.
//
bool8 TranscriptionDatabase::readData(Sof& sof_a, const String& pname_a,
                         int32 size_a, bool8 param_a, bool8 nested_a) {

  // we need a parser
  //
  SofParser parser;  
  parser.setDebug(debug_level_d);
  
  // ignore implicit parameter setting
  //
  
  // are we nested?
  //
  if (nested_a) {
    parser.setNest();
  }
  
  // load the parse
  //
  if (!parser.load(sof_a, size_a)) {
    return Error::handle(name(), L"readData", Error::READ,
                         __FILE__, __LINE__, Error::WARNING);
  }    

  // read the database name
  //  
  if (parser.isPresent(sof_a, PARAM_NAME)) {
      if (!name_d.readData(sof_a, PARAM_NAME,
			   parser.getEntry(sof_a, PARAM_NAME))) {
	
	// return a warning message
	//
	return Error::handle(name(), L"readData", Error::READ,
			     __FILE__, __LINE__, Error::WARNING);
	
      }
  }
    
  // read the keys associated with the hash table
  //
  Vector<String> keys;
  if (parser.isPresent(sof_a, PARAM_KEYS)) {
    if (!keys.readData(sof_a, PARAM_KEYS,
			   parser.getEntry(sof_a, PARAM_KEYS),
			   false, false)) {
      
	// return a warning message
	//
	return Error::handle(name(), L"readData", Error::READ,
			     __FILE__, __LINE__, Error::WARNING);
	
      }
  }
  
  Vector<Long> values;

  values.setLength(keys.length());

  for (int32 i = 0; i < keys.length(); i++) {
    values(i).assign(i);
  }
  
  // reset the hash table to begin with
  //
  hash_index_d.clear(Integral::RESET);
  
  for (int32 i = 0; i < keys.length(); i++) {
    hash_index_d.insert(keys(i), &values(i));
  }

  // exit gracefully
  //
  return true;
}

// method: close
//
// arguments: none
//
// return: a bool8 value indicating status
//
// this method closes the database file
//
bool8 TranscriptionDatabase::close() {

  // close the database file
  //
  return database_sof_d.close();
}

// method: open
//
// arguments:
//  const Filename& filename: (input) database file
//
// return: a bool8 value indicating status
//
// this method opens the database file
//
bool8 TranscriptionDatabase::open(const Filename& filename_a) {

  // error checking
  //
  if (filename_a.length() < 1) {
    filename_a.debug(L"filename_a");
    return Error::handle(name(), L"readData - invalid filename", Error::ARG, __FILE__, __LINE__);
  }

  // open the Transcription database
  //
  if (!database_sof_d.open(filename_a)) {
    filename_a.debug(L"filename_a");
    return Error::handle(name(), L"error opening Transcription database file", Error::ARG, __FILE__, __LINE__);
  }

  // read the Transcription database
  //
  if (!read(database_sof_d, (int32)0)) {
    return Error::handle(name(), L"error loading transcription database", Error::ARG, __FILE__, __LINE__);
  }
  
  // exit gracefully
  //
  return true;
}
