// file: $isip/class/mmedia/TranscriptionDatabase/trans_04.cc
// version: $Id: trans_04.cc 8947 2002-12-29 02:19:02Z alphonso $
//

// isip include files
//
#include "TranscriptionDatabase.h"

// method: write
//
// arguments:
//  Sof& sof: (input) sof file object
//  int32 tag: (input) sof object instance tag
//  const String& name: (input) sof object instance name
//
// return: logical error status
//
// this method has the object read itself from an Sof file
//
bool8 TranscriptionDatabase::write(Sof& sof_a, int32 tag_a,
			     const String& name_a) const {

  int32 obj_size;

  if (sof_a.isText()) {
    obj_size = Sof::ANY_SIZE;
  }
  else {
    obj_size = sofSize();
  }
  
  // put the object into the sof file's index
  //
  if (!sof_a.put(name_a, tag_a, obj_size)) {
    return false;
  }

  // exit gracefully
  //
  return writeData(sof_a);
}

// method: writeData
//
// arguments:
//  Sof& sof: (input) sof file object
//  const String& pname: (input) parameter name
//
// return: logical error status
//
// this method writes the object to the Sof file. it assumes that the
// Sof file is already positioned correctly.
//
bool8 TranscriptionDatabase::writeData(Sof& sof_a, const String& pname_a) const {

  // write a start string if necessary
  //
  sof_a.writeLabelPrefix(pname_a);

  // write the database name
  //
  name_d.writeData(sof_a, PARAM_NAME);
  
  // get the keys associated with the hash table
  //
  Vector<String> keys;
  if (!hash_d.keys(keys)) {
    return Error::handle(name(), L"writeData", Error::ARG, __FILE__, __LINE__);
  }

  // write the keys associated with the hash table
  //
  keys.writeData(sof_a, PARAM_KEYS);
  
  // put an end string if necessary
  //
  sof_a.writeLabelSuffix(pname_a);
  
  // exit gracefully
  //
  return true;
}
