// file: $isip/class/pr/HiddenMarkovModel/hmm_04.cc
// version: $Id: hmm_04.cc 9449 2004-04-13 19:27:00Z gao $

// isip include files
//
#include "HiddenMarkovModel.h"
#include <Sof.h>

// method: write
//
// arguments:
//  Sof& sof: (input) sof file object
//  int32 tag: (input) sof object instance tag
//  const String& name: (input) sof object instance name
//
// return: logical error status
//
// this method has the object read itself from an Sof file
//
bool8 HiddenMarkovModel::write(Sof& sof_a, int32 tag_a,
			     const String& name_a) const {

  int32 obj_size;

  if (sof_a.isText()) {
    obj_size = Sof::ANY_SIZE;
  }
  else {
    obj_size = sofSize();
  }
  
  // put the object into the sof file's index
  //
  if (!sof_a.put(name_a, tag_a, obj_size)) {
    return false;
  }

  // exit gracefully
  //
  return writeData(sof_a);
}

// method: writeData
//
// arguments:
//  Sof& sof: (input) sof file object
//  const String& pname: (input) parameter name
//
// return: logical error status
//
// this method writes the object to the Sof file. it assumes that the
// Sof file is already positioned correctly.
//
bool8 HiddenMarkovModel::writeData(Sof& sof_a, const String& pname_a) const {
  
  // write a start string if necessary
  //
  sof_a.writeLabelPrefix(pname_a);

  // write the algorithm
  //
  ALGO_MAP.writeElementData(sof_a, PARAM_ALGORITHM, (int32)algorithm_d);

  // write the implementation
  //
  IMPL_MAP.writeElementData(sof_a, PARAM_IMPLEMENTATION, (int32)implementation_d);

  // put an end string if necessary
  //
  sof_a.writeLabelSuffix(pname_a);
  
  // exit gracefully
  //
  return true;
}

// method: sofSize
//
// arguments: none
//
// return: size of object
//
// this method returns the size of the object in the Sof file and is
// used for binary write
//
int32 HiddenMarkovModel::sofSize() const {

  // count up object by object
  //
  int32 size = 0;

  // accumulate the sofSize of all the data being written
  //
  size += ALGO_MAP.elementSofSize();

  size += IMPL_MAP.elementSofSize();

  // return the size
  //
  return size;
}
