// file: $isip/class/pr/LanguageModelBNF/lmbnf_04.cc
// version: $Id: lmbnf_04.cc 10496 2006-03-15 21:39:18Z may $

//isip include files
//
#include "LanguageModelBNF.h"

// method: write
//
// arguments:
//  Sof& sof: (input) sof file object
//  int32 tag: (input) sof object instance tag
//  const String& name: (input) sof object instance name
//
// return: bool8 value indicating status
//
// this method has the object write itself to an Sof file
//
bool8 LanguageModelBNF::write(Sof& sof_a, int32 tag_a,
				const String& name_a) const {
  int32 obj_size = 0;

  // figure out size
  //
  if (sof_a.isText()) {

    //dynamic for text
    //
    obj_size = Sof::ANY_SIZE;
  }
  else {
    obj_size = sofSize();
  }

  // write object into sof index
  //
  if (!sof_a.put(name_a, tag_a, obj_size)) {
    return false;
  }
  
  // exit gracefully
  //
  return writeData(sof_a);
}

// method: writeData
//
// arguments:
//  Sof& sof: (input) sof file object
//  const String& pname: (input) parameter name
//
// return: bool8 value indicating status
//
// this method has the object write itself to an Sof file.
//
bool8 LanguageModelBNF::writeData(Sof& sof_a, const String& pname_a) const {

  // write a start string if necessary
  //
  sof_a.writeLabelPrefix(pname_a);

  // write the rule model object
  //
  if (!rm_d.writeData(sof_a, PARAM_BNF)) {
    return Error::handle(name(), L"writeData",Error::IO, __FILE__, __LINE__);
  }

  // put an end string if necessary
  //
  sof_a.writeLabelSuffix(pname_a);

  // exit gracefully
  //
  return true;
}
