// file: $isip/class/pr/MaximumLikelihoodLinearRegression/mllr_03.cc
// version: $Id: mllr_03.cc 9457 2004-04-19 15:06:59Z gao $
//

// isip include files
//
#include "MaximumLikelihoodLinearRegression.h"

//------------------------------------------------------------------
// these methods have to be in the same file so they can use the same
// static parser pointer
//------------------------------------------------------------------

// declare a static sof parser pointer
//
static SofParser* parser_d = (SofParser*)NULL;

// method: setParser
//
// arguments:
//  SofParser* parser: (input) sof file object
//
// return: a bool8 value indicating status
//
// this method sets the parser from the parent object to be used in
// the next readData method.
//
bool8 MaximumLikelihoodLinearRegression::setParser(SofParser* parser_a) {
  
  // set the parser
  //
  parser_d = parser_a;

  // exit gracefully
  //
  return true;
}

// method: read
//
// arguments:
//  Sof& sof: (input) sof file object
//  int32 tag: (input) sof object instance tag
//  const String& name: (input) sof object instance name
//
// return: a bool8 value indicating status
//
// this method has the object read itself from an Sof file
//
bool8 MaximumLikelihoodLinearRegression::read(Sof& sof_a, int32 tag_a,
				   const String& name_a) {
  
  // get the instance of the object from the Sof file
  //
  if (!sof_a.find(name_a, tag_a)) {
    return false;
  }

  // read the actual data from the sof file
  //
  return readData(sof_a);
}

// method: readData
//
// arguments:
//  Sof& sof: (input) sof file object
//  const String& pname: (input) parameter name
//  int32 size: (input) size of the object
//  bool8 param: (input) is the parameter specified?
//  bool8 nested: (input) is this nested?
//
// return: a bool8 value indicating status
//
// this method has the object read itself from an Sof file. it assumes
// that the Sof file is already positioned correctly.
//
bool8 MaximumLikelihoodLinearRegression::readData(Sof& sof_a, const String& pname_a,
					 int32 size_a, bool8 param_a,
					 bool8 nested_a) {
  // declare local variables
  //
  bool8 status = true;

  // we need a parser
  // 
  if (parser_d == (SofParser*)NULL) {
    
    parser_d = new SofParser;
    
    if (nested_a) {
      parser_d->setNest();
    }
    
    // load the parse
    //
    if (!parser_d->load(sof_a, size_a)) {

      // delete the parser
      //
      delete parser_d;
      parser_d = (SofParser*)NULL;
      
      // return a warning message
      //
      return Error::handle(name(), L"readData", Error::READ,
                           __FILE__, __LINE__, Error::WARNING);
    }           
  }

  // set the debug level of the parser
  //
  parser_d->setDebug(debug_level_d.getIndex());
  
  // get the algorithm name
  //
  if (parser_d->isPresent(sof_a, PARAM_ALGORITHM)) {
    if (!ALGO_MAP.readElementData((int32&)algorithm_d, sof_a,
				  PARAM_ALGORITHM,
				  parser_d->getEntry(sof_a, PARAM_ALGORITHM))) {

      // delete the parser
      //
      delete parser_d;
      parser_d = (SofParser*)NULL;
      
      // return a warning message
      //
      return Error::handle(name(), L"readData", Error::READ,
			   __FILE__, __LINE__, Error::WARNING);
    }      
  }
  else {
    algorithm_d = DEF_ALGORITHM;
  }

  // get the implementation
  //
  if (parser_d->isPresent(sof_a, PARAM_IMPLEMENTATION)) {
    if (!IMPL_MAP.readElementData((int32&)implementation_d, sof_a,
				  PARAM_IMPLEMENTATION,
				  parser_d->getEntry(sof_a,
						     PARAM_IMPLEMENTATION))) {
      
      // delete the parser
      //
      delete parser_d;
      parser_d = (SofParser*)NULL;
      
      // return a warning message
      //
      return Error::handle(name(), L"readData", Error::READ,
			   __FILE__, __LINE__, Error::WARNING);
    }      
  }
  else {
    implementation_d = DEF_IMPLEMENTATION;
  }

  // read the adaptation supervision mode
  //
  if (parser_d->isPresent(sof_a, PARAM_SUPERVISION_MODE)) {
    if (!SUP_MODE_MAP.readElementData(
			      (int32&)supervision_mode_d, sof_a,
			      PARAM_SUPERVISION_MODE,
			      parser_d->getEntry(sof_a,
					      PARAM_SUPERVISION_MODE))) {
      return Error::handle(name(), L"readData", Error::READ,
			   __FILE__, __LINE__, Error::WARNING);
    }       
  }
  else {
    supervision_mode_d = DEF_SUPERVISION_MODE;
  }
  
  // read the adaptation sequence mode
  //
  if (parser_d->isPresent(sof_a, PARAM_SEQUENCE_MODE)) {
    if (!SEQ_MODE_MAP.readElementData(
				  (int32&)sequence_mode_d, sof_a,
				  PARAM_SEQUENCE_MODE,
			      parser_d->getEntry(sof_a,
					      PARAM_SEQUENCE_MODE))) {
      return Error::handle(name(), L"readData", Error::READ,
			   __FILE__, __LINE__, Error::WARNING);
    }       
  }
  else {
    sequence_mode_d = DEF_SEQUENCE_MODE;
  }

  // read the rdt object
  //
  if (parser_d->isPresent(sof_a, PARAM_RDT)) {
    if (!rdt_d.readData(sof_a, PARAM_RDT, parser_d->getEntry(sof_a, PARAM_RDT),
		       true, true)) {
      // delete the parser
      //
      delete parser_d;
      parser_d = (SofParser*)NULL;
      
      // return a warning message
      //
      return Error::handle(name(), L"readData", Error::READ,
			   __FILE__, __LINE__, Error::WARNING);
    }
  }
  else {
    // do nothing
    //
  }

  // read the statistical model object
  //
  if (parser_d->isPresent(sof_a, PARAM_SM_ADAPT)) {
    if (!sm_adapt_d.readData(sof_a, PARAM_SM_ADAPT,
			     parser_d->getEntry(sof_a, PARAM_SM_ADAPT),
			     true, true)) {
      // delete the parser
      //
      delete parser_d;
      parser_d = (SofParser*)NULL;
      
      // return a warning message
      //
      return Error::handle(name(), L"readData", Error::READ,
			   __FILE__, __LINE__, Error::WARNING);
    }
  }
  else {
    // do nothing
    //
  }  
  
  // get the debug level
  //
  if (parser_d->isPresent(sof_a, PARAM_DBGL)) {
    if (!debug_level_d.readData(sof_a, PARAM_DBGL,
				parser_d->getEntry(sof_a, PARAM_DBGL))) {
      
      // delete the parser
      //
      delete parser_d;
      parser_d = (SofParser*)NULL;
      
      // return a warning message
      //
      return Error::handle(name(), L"readData", Error::IO,
			   __FILE__, __LINE__, Error::WARNING);
    }
  }
  else {
    debug_level_d = Integral::DEF_DEBUG;
  }
  
  // check that all parameters are accounted for
  //
  if (!parser_d->checkParams(sof_a)) {

    // delete the parser
    //
    delete parser_d;
    parser_d = (SofParser*)NULL;

    // return a warning message
    //
    return Error::handle(name(), L"readData", Error::IO,
			 __FILE__, __LINE__, Error::WARNING);
  }

  // delete the parser
  //
  delete parser_d;
  parser_d = (SofParser*)NULL;

  // exit gracefully
  //
  return status;
}
