// file: $isip/class/pr/NeuralNetwork/svm_05.cc
//

// isip include files
//
#include "NeuralNetwork.h"
#include <stdlib.h>
#include <File.h>
#include <Console.h>

// method: init
//
// arguments:
//  int32 epochs: (input) number of epochs
//  int32 units: (input) number of hidden units per layer
//  int32 layers: (input) number of hidden layers
//  float32 bias: (input) network bias
//  float32 rate: (input) network learning rate
//  float32 momentum: (input) network momentum
//
// return: bool8 value indicating status
//
// this method loads the output target used train the neural network
//
bool8 NeuralNetwork::init(int32 epochs_a, int32 units_a, int32 layers_a,
			    float32 bias_a, float32 rate_a, float32 momentum_a) {

  // error checking
  //
  if (epochs_a < 1) {
    return Error::handle(name(), L"init", Error::ARG, __FILE__, __LINE__);
  }
  if (units_a < 1) {
    return Error::handle(name(), L"init", Error::ARG, __FILE__, __LINE__);
  }    
  if (layers_a != 1) {
    return Error::handle(name(), L"init", Error::ARG, __FILE__, __LINE__);
  }
  
  // set the number of epochs
  //
  epochs_d = epochs_a;
  
  // set the number of hidden units and hidden layers
  //
  hidden_layers_d.setLength(layers_a);
  for (int i=0; i < layers_a; i++) {
    hidden_layers_d(i).setLength(units_a + 1);
  }

  // set the network bias
  //
  bias_d = bias_a;

  // set the network learning rate
  //
  eta_d = rate_a;

  // set the network momentum
  //
  momentum_d = momentum_a;

  // exit gracefully
  //
  return true;  
}

// method: loadFeatures
//
// arguments:
//  String& file: (input) training feature vectors path
//
// return: bool8 value indicating status
//
// this method loads the feature vectors used train the neural network
//
bool8 NeuralNetwork::loadFeatures(String& file_a) {

  // declare local variables
  //
  File fp;
  String line;
  String substring;
  String substring1;  
  
  int32 pos = 0;
  int index = 0;
  int dimension = 0;  
  float32 value1 = 0.0;
  
  // open the file for reading
  //
  if (!fp.open(file_a)) {
    return Error::handle(name(), L"loadFeatures", Error::ARG,
			 __FILE__, __LINE__);
  }

  // read the contents of the file
  //
  while(fp.get(line)) {

    // remove whitespaces from the line
    //
    line.trim();

    // ignore all comment tags
    //
    pos = line.firstStr(DEF_COMMENT_TAG);
    if (pos != 0) {

      // set the feature vector values
      //
      dimension = 0;    
      while (line.tokenize(substring, pos)) {
	substring.get(value1);	
	features_d.setLength(index+1);
	features_d(index).setLength(dimension+1);
	features_d(index)(dimension++) = value1;
      }
      index++;
    }
  }

  // close the file descriptor
  //
  fp.close();

  // exit gracefully
  //
  return true;  
}

// method: loadOutputs
//
// arguments:
//  String& file: (input) output target path
//
// return: bool8 value indicating status
//
// this method loads the output target used train the neural network
//
bool8 NeuralNetwork::loadOutputs(String& file_a) {

  // declare local variables
  //
  File fp;
  String line;
  String substring;
  String substring1;  
  
  int32 pos = 0;
  int index = 0;
  int dimension = 0;  
  float32 value1 = 0.0;
  
  // open the file for reading
  //
  if (!fp.open(file_a)) {
    return Error::handle(name(), L"loadOutputs", Error::ARG,
			 __FILE__, __LINE__);
  }

  // read the contents of the file
  //
  while(fp.get(line)) {

    // remove whitespaces from the line
    //
    line.trim();

    // ignore all comment tags
    //
    pos = line.firstStr(DEF_COMMENT_TAG);
    if (pos != 0) {

      // set the feature vector values
      //
      dimension = 0;    
      while (line.tokenize(substring, pos)) {
	substring.get(value1);	
	outputs_d.setLength(index+1);
	outputs_d(index).setLength(dimension+1);
	outputs_d(index)(dimension++) = value1;
      }
      index++;
    }
  }

  // close the file descriptor
  //
  fp.close();

  // exit gracefully
  //
  return true;  
}

// method: loadTestFeatures
//
// arguments:
//  String& file: (input) test feature vectors path
//
// return: bool8 value indicating status
//
// this method loads the feature vectors used test the neural network
//
bool8 NeuralNetwork::loadTestFeatures(String& file_a) {

  // declare local variables
  //
  File fp;
  String line;
  String substring;
  String substring1;  
  
  int32 pos = 0;
  int index = 0;
  int dimension = 0;  
  float32 value1 = 0.0;
  
  // open the file for reading
  //
  if (!fp.open(file_a)) {
    return Error::handle(name(), L"loadFeatures", Error::ARG,
			 __FILE__, __LINE__);
  }

  // read the contents of the file
  //
  while(fp.get(line)) {

    // remove whitespaces from the line
    //
    line.trim();

    // ignore all comment tags
    //
    pos = line.firstStr(DEF_COMMENT_TAG);
    if (pos != 0) {

      // set the feature vector values
      //
      dimension = 0;    
      while (line.tokenize(substring, pos)) {
	substring.get(value1);	
	test_features_d.setLength(index+1);
	test_features_d(index).setLength(dimension+1);
	test_features_d(index)(dimension++) = value1;
      }
      index++;
    }
  }

  // close the file descriptor
  //
  fp.close();

  // exit gracefully
  //
  return true;  
}

// method: loadTestOutputs
//
// arguments:
//  String& file: (input) output target path
//
// return: bool8 value indicating status
//
// this method loads the output target used test the neural network
//
bool8 NeuralNetwork::loadTestOutputs(String& file_a) {

  // declare local variables
  //
  File fp;
  String line;
  String substring;
  String substring1;  
  
  int32 pos = 0;
  int index = 0;
  int dimension = 0;  
  float32 value1 = 0.0;
  
  // open the file for reading
  //
  if (!fp.open(file_a)) {
    return Error::handle(name(), L"loadOutputs", Error::ARG,
			 __FILE__, __LINE__);
  }

  // read the contents of the file
  //
  while(fp.get(line)) {

    // remove whitespaces from the line
    //
    line.trim();

    // ignore all comment tags
    //
    pos = line.firstStr(DEF_COMMENT_TAG);
    if (pos != 0) {

      // set the feature vector values
      //
      dimension = 0;    
      while (line.tokenize(substring, pos)) {
	substring.get(value1);	
	test_outputs_d.setLength(index+1);
	test_outputs_d(index).setLength(dimension+1);
	test_outputs_d(index)(dimension++) = value1;
      }
      index++;
    }
  }

  // close the file descriptor
  //
  fp.close();

  // exit gracefully
  //
  return true;  
}

// method: writeModel
//
// arguments:
//  String& file: (input) output file
//
// return: bool8 value indicating status
//
// this method write the model to file
//
bool8 NeuralNetwork::writeModel(String& file_a) {

  // open the file descriptor
  //
  Sof tmp_file;
  tmp_file.open(file_a, File::WRITE_ONLY);
  
  // write the neural network weights
  //
  w1_d.write(tmp_file, (int32)0);
  h1_d.write(tmp_file, (int32)1);    
  w2_d.write(tmp_file, (int32)2);  
    
  // close the file descriptor
  //
  tmp_file.close();
     
  // exit gracefully
  //
  return true;
}

// method: test
//
// arguments:
//  String& model: (input) trained model
//  String& test: (input) test features
//  String& output: (input) test outputs
//
// return: bool8 value indicating status
//
// this method loads the testing data and generated the statistics
//
bool8 NeuralNetwork::test(String& model_a, String& test_a,
			    String& output_a) {

  // declare local variables
  //
  int32 N = 0;
  int32 A = 0;
  int32 B = 0;
  int32 C = 0;  
  int32 misclassified = 0;
  
  float32 accum = 0;
  float32 reference = 0;
  float32 hypothesis = 0;
  float32 classification = 0;
  
  // load the trained models
  //
  Sof tmp_file;
  tmp_file.open(model_a);

  // read the neural network weights
  //
  w1_d.read(tmp_file, (int32)0);
  h1_d.read(tmp_file, (int32)1);    
  w2_d.read(tmp_file, (int32)2);

  // close the file descriptor
  //
  tmp_file.close();
  
  // load the testing data
  //
  loadTestFeatures(test_a);
  loadTestOutputs(output_a);

  // dimensions checking
  //
  if (test_features_d.length() != features_d.length()) {
    return Error::handle(name(), L"test", Error::ARG, __FILE__, __LINE__);
  }
  if (test_outputs_d.length() != outputs_d.length()) {
    return Error::handle(name(), L"test", Error::ARG, __FILE__, __LINE__);
  }  
  
  // determine the dimensions of the test data
  //
  N = test_features_d.length();
  A = test_features_d(0).length() + 1;
  B = hidden_layers_d(0).length();
  C = test_outputs_d(0).length();

  // classify the test set
  //
  fprintf(stdout, "\n\nClassifying the test examples...\n");
  fflush(stdout);
      
  // outer loop - iterates over each observation
  //
  for (int obser=0; obser < N; obser++) {

    // display the observation we are on
    //
    fprintf(stdout, "classifying observation %d...\n", obser);
    fflush(stdout);
    
    // assign the activation levels to the inputs for the current observation
    //
    input_layer_d(0) = 1;
    for (int j=1; j < A; j++) {
      input_layer_d(j) = test_features_d(obser)(j - 1);
    }

    // propogate the activations form the units in the input layer
    // to the units in the hidden layer
    //
    hidden_layers_d(0)(0) = 1;
    for (int j=1; j < B; j++) {
      accum = 0;
      for (int i=0; i < A; i++) {
	accum += w1_d(i, j) * input_layer_d(i);
      }
      hidden_layers_d(0)(j) = 1 / (1 + exp(-accum));
    }
      
    // propogate the activations form the units in the hidden layer
    // to the units in the output layer
    // test_outputs_d(obser)(j)
    for (int j=0; j < C; j++) {
      accum = 0;
      for (int i=0; i < B; i++) {
	accum += w2_d(i, j) * hidden_layers_d(0)(i);
      }
      output_layer_d(j) = 1 / (1 + exp(-accum));

      // retrieve the reference and hypothesis scores
      //
      reference = (float32)test_outputs_d(obser)(j);
      hypothesis = (float32)output_layer_d(j);

      // determine the classification error for the example
      //
      if (((reference >= 0.5) && (hypothesis < 0.5)) ||
	  ((reference < 0.5) && (hypothesis >= 0.5))) {
	misclassified++;
      }

      // display the reference and hypothesis for the example
      //
      fprintf(stdout, "reference: %f\n", (float32)test_outputs_d(obser)(j));
      fprintf(stdout, "hypothesis: %f\n", (float32)output_layer_d(j));      
      fflush(stdout);
    }
    
  } // end outer loop

  // display the classification statistics
  //
  classification = ((float32)misclassified/(float32)N) * (float32)100;
  fprintf(stdout, "--------------------------------------------------\n");  
  fprintf(stdout, "Total number of observations: %ld\n", N);
  fprintf(stdout, "Observations correctly classified: %ld\n", N-misclassified);
  fprintf(stdout, "Observations incorrectly classified: %ld\n", misclassified);
  fprintf(stdout, "Classification error: %f percent\n", classification);
  fprintf(stdout, "--------------------------------------------------\n");    
  fflush(stdout);
      
  // exit gracefully
  //
  return true;
}

// method: train
//
// arguments:
//  String& train: (input) training features
//  String& output: (input) training outputs
//  String& output: (input) trained models output
//
// return: bool8 value indicating status
//
// this method loads the training data and start learning
//
bool8 NeuralNetwork::train(String& train_a, String& output_a,
			     String& model_a) {

  // declare local variables
  //
  int32 A = 0;
  int32 B = 0;
  int32 C = 0;
  int32 N = 0;
  int32 misclassified = 0;
  
  float32 error = 0;
  float32 accum = 0;
  float32 reference = 0;
  float32 hypothesis = 0;
  float32 average_squared_error = 0;  
  
  // initialize the random number generator
  //
  srand48(time(NULL));
  
  // load the training data
  //
  loadFeatures(train_a);
  loadOutputs(output_a);

  // set up the neural network dimensions
  //
  N = features_d.length();
  A = features_d(0).length() + 1;
  B = hidden_layers_d(0).length();
  C = outputs_d(0).length();
  
  // set up the neural network
  //
  input_layer_d.setLength(A);
  output_layer_d.setLength(C);    
  
  // initialize the network weights between the input layer and hidden layer
  //
  w1_d.setDimensions(A, B);
  
  for (int i=0; i < A; i++) {
    for (int j=0; j < B; j++) {
      w1_d.setValue(i, j, drand48());
    }
  }
  
  // initialize the network weights between the hidden layer and output layer
  //  
  w2_d.setDimensions(B, C);

  for (int i=0; i < B; i++) {
    for (int j=0; j < C; j++) {
      w2_d.setValue(i, j, drand48());
    }
  }
  
  // outer loop -loop over the number of epochs
  //
  for (int epoch=0; epoch < epochs_d; epoch++) {

    // display the epoch we are on
    //
    fprintf(stdout, "starting epoch %d...\n", epoch);
    fflush(stdout);

    // inner loop - iterates over each observation
    //
    error = 0;
    misclassified = 0;
    for (int obser=0; obser < N; obser++) {

      // assign the activation levels to the inputs for the current observation
      //
      input_layer_d(0) = bias_d;
      for (int j=1; j < A; j++) {
	input_layer_d(j) = features_d(obser)(j - 1);
      }

      // propogate the activations form the units in the input layer
      // to the units in the hidden layer
      //
      hidden_layers_d(0)(0) = bias_d;
      for (int j=1; j < B; j++) {
	accum = 0;
	for (int i=0; i < A; i++) {
	  accum += w1_d(i, j) * input_layer_d(i);
	}
	hidden_layers_d(0)(j) = 1 / (1 + exp(-accum));
      }
      
      // propogate the activations form the units in the hidden layer
      // to the units in the output layer
      //
      for (int j=0; j < C; j++) {
	accum = 0;
	for (int i=0; i < B; i++) {
	  accum += w2_d(i, j) * hidden_layers_d(0)(i);
	}
	output_layer_d(j) = 1 / (1 + exp(-accum));

	// compute the sum squared error
	//
	error += pow((float64)(output_layer_d(j) -
			      outputs_d(obser)(j)), (float64)2);

	// determine if the netowrk has misclassified the example
	//
	reference = (float32)outputs_d(obser)(j);
	hypothesis = (float32)output_layer_d(j);
	if (((reference >= 0.5) && (hypothesis < 0.5)) ||
	    ((reference < 0.5) && (hypothesis >= 0.5))) {
	  misclassified++;
	}	  
      }
      
      // compute the errors of the units in the output layer
      //
      VectorFloat delta2(C);
      for (int j=0; j < C; j++) {
	delta2(j) = output_layer_d(j) * (1 - output_layer_d(j)) *
	  (outputs_d(obser)(j) - output_layer_d(j));
      }

      // compute the errors of the units in the hidden layer
      //
      VectorFloat delta1(B);
      for (int j=1; j < B; j++) {
	accum = 0;
	for (int i=0; i < C; i++) {
	  accum += delta2(i) * w2_d(j, i);
	}
	delta1(j) = hidden_layers_d(0)(j) *
	  (1 - hidden_layers_d(0)(j)) * accum;
      }

      // adjust the weights between the hidden layer and the output layer
      //
      MatrixFloat deltaW2(B, C);
      for (int i=0; i < B; i++) {
	for (int j=0; j < C; j++) {
	  deltaW2.setValue(i, j, (eta_d * delta2(j) * hidden_layers_d(0)(i)));
	}
      }
      w2_d.add(deltaW2);
      
      // adjust the weights between the hidden layer and the output layer
      //
      MatrixFloat deltaW1(A, B);
      for (int i=0; i < A; i++) {
	for (int j=1; j < B; j++) {
	  deltaW1.setValue(i, j, (eta_d * delta1(j) * input_layer_d(i)));
	}
      }
      w1_d.add(deltaW1);
      
    } // end of inner loop

    // display the average squared error
    //
    average_squared_error = error / (float32)N;    
    fprintf(stdout, "average squared error: %f\n", average_squared_error);
    fflush(stdout);

    // determine if the neural network has converged
    //
    if (misclassified == 0) {

      fprintf(stdout, "neural network has converged...\n");
      fflush(stdout);

      // write the network weights
      //
      writeModel(model_a);
      return true;
    }
    
  } // end of outer loop
  
  // write the network weights
  //
  writeModel(model_a);
  
  // exit gracefully
  //
  return true; 
}

