// file: $isip/class/pr/PhoneticDecisionTreeNode/pdtnod_03.cc
// version: $Id: pdtnod_03.cc 8688 2002-09-11 20:04:08Z parihar $
//

// isip include files
//
#include "PhoneticDecisionTreeNode.h"

// method: read
//
// arguments:
//  Sof& sof: (input) sof file object
//  int32 tag: (input) sof object instance tag
//  const String& name: (input) sof object instance name
//
// return: a bool8 value indicating status
//
// this method has the object read itself from an Sof file
//
bool8 PhoneticDecisionTreeNode::read(Sof& sof_a, int32 tag_a,
				       const String& name_a) {

  // read the instance of the object from the Sof file
  //
  if (!sof_a.find(name_a, tag_a)) {
    return false;
  }
  
  // read the actual data from the sof file
  //
  if (!readData(sof_a)) {
    return false;
  }
  
  // exit gracefully
  //
  return true;  
}

// method: readData
//
// arguments:
//  Sof& sof: (input) sof file object
//  const String& pname: (input) parameter name
//  int32 size: (input) size in bytes of object (or full_size)
//  bool8 param: (input) is the parameter name in the file?
//  bool8 nested: (input) are we nested?
//
// return: a bool8 value indicating status
//
// this method has the object read itself from an Sof file. it assumes
// that the Sof file is already positioned correctly.
//
bool8 PhoneticDecisionTreeNode::readData(Sof& sof_a, const String& pname_a,
					   int32 size_a, bool8 param_a,
					   bool8 nested_a) {

  Integral::DEBUG debug_level = Integral::NONE;

  SofParser parser;

  parser.setDebug(debug_level);
    
  // ignore implicit parameter setting
  //
  
  // are we nested?
  //
  if (nested_a) {
    parser.setNest();
  }
  
  // load the parse
  //
  if (!parser.load(sof_a, size_a)) {
    return Error::handle(name(), L"readData", Error::READ,
			 __FILE__, __LINE__, Error::WARNING);
  }	    

  // get the datapoints
  //
  if (!datapoints_d.readData(sof_a, PARAM_DATAPOINTS,
			     parser.getEntry(sof_a, PARAM_DATAPOINTS),
			     false, false)) {
    return Error::handle(name(), L"readData", Error::READ,
			 __FILE__, __LINE__, Error::WARNING);
  }	    

  // get the best attribute
  //
  if (!best_attribute_d.readData(sof_a, PARAM_BEST_ATTRIBUTE,
				 parser.getEntry(sof_a, PARAM_BEST_ATTRIBUTE),
				 true, true)) {
    return Error::handle(name(), L"readData", Error::READ,
			 __FILE__, __LINE__, Error::WARNING);
  }

  // get the typical statistical-model index
  //
  if (!typical_index_d.readData(sof_a, PARAM_TYPICAL_INDEX,
				parser.getEntry(sof_a, PARAM_TYPICAL_INDEX),
				true, true)) {
    return Error::handle(name(), L"readData", Error::READ,
			 __FILE__, __LINE__, Error::WARNING);
  }

  // get the actual index
  //
  if (!actual_index_d.readData(sof_a, PARAM_ACTUAL_INDEX,
				parser.getEntry(sof_a, PARAM_ACTUAL_INDEX),
				true, true)) {
    return Error::handle(name(), L"readData", Error::READ,
			 __FILE__, __LINE__, Error::WARNING);
  }

  // get the typical statistical-model
  //
  if (!typical_stat_model_d.readData(sof_a, PARAM_TYPICAL_STAT_MODEL,
				     parser.getEntry(sof_a, PARAM_TYPICAL_STAT_MODEL),
				     true, true)) {
    return Error::handle(name(), L"readData", Error::READ,
			 __FILE__, __LINE__, Error::WARNING);
  }

  // get the flag that denotes whether a node exists or it is merged
  // with some other node
  //
  if (!flag_exists_d.readData(sof_a, PARAM_FLAG_EXISTS,
			      parser.getEntry(sof_a, PARAM_FLAG_EXISTS),
			      true, true)) {
    return Error::handle(name(), L"readData", Error::READ,
			 __FILE__, __LINE__, Error::WARNING);
  }

  // exit gracefully
  //
  return true;
}
