// file: $isip/class/pr/ProductionRule/prdrl_02.cc
// version: $Id: prdrl_02.cc 10292 2005-11-09 18:06:13Z wholland $

// ISIP include files
//
#include "ProductionRule.h"

// method: diagnose
//
// arguments:
//  Integral::DEBUG level: (input) debug level for diagnostics
//
// return: a bool8 value indicating status
//
bool8 ProductionRule::diagnose(Integral::DEBUG level_a) {
  
  //---------------------------------------------------------------------------
  //
  // 0. preliminaries
  //
  //---------------------------------------------------------------------------
  
  // output the class name
  //
  if (level_a > Integral::NONE) {
    SysString output(L"diagnosing class ");
    output.concat(CLASS_NAME);
    output.concat(L": ");
    Console::put(output);
    Console::increaseIndention();
  }
  
  //--------------------------------------------------------------------------
  //
  // 1. required public methods
  //
  //--------------------------------------------------------------------------
  
  // set indention
  //
  if (level_a > Integral::NONE) {
    Console::put(L"testing required public methods...\n");
    Console::increaseIndention();
  }
  
  // test constructor
  //
  ProductionRule prdrl;
  
  // test debug methods
  //
  prdrl.setDebug(level_a);
  
  if (level_a > Integral::BRIEF) {
    Console::put(L"testing debug method...");
    prdrl.debug(L"debug");
  }

  prdrl.setRuleName(L"fujamabob");
  
  // test clear method
  //
  prdrl.clear(Integral::RESET);
  
  if (level_a > Integral::BRIEF) {
    Console::put(L"testing clear method...");
  }
  
  // if rule name is clear, it worked
  //
  if(!prdrl.rule_name_d.eq(L"")) {
    return Error::handle(name(), L"clear", Error::TEST, __FILE__, __LINE__);
  }
  
  // test eq method
  //
  ProductionRule prdrl2;
  
  if (level_a > Integral::BRIEF) {
    Console::put(L"testing eq method...");
  }
  
  if(!prdrl2.eq(prdrl)) {
    return Error::handle(name(), L"eq", Error::TEST, __FILE__, __LINE__);
  }
  
  // test assign
  //
  prdrl2.setRuleName(L"fujamabob");
  prdrl.assign(prdrl2);
  
  if (level_a > Integral::BRIEF) {
    Console::put(L"testing assign method...");
  }
  
  if(!prdrl.rule_name_d.eq(L"fujamabob")) {
    return Error::handle(name(), L"assign", Error::TEST, __FILE__, __LINE__);
  } 
  
  // reset indention
  //
  if (level_a > Integral::NONE) {
    Console::decreaseIndention();
  }
  
  //--------------------------------------------------------------------------
  //
  // 2. class-specific public methods:
  //     
  //
  //--------------------------------------------------------------------------

  // set indention
  //
  if (level_a > Integral::NONE) {
    Console::put(L"testing class specific public methods...\n");
    Console::increaseIndention();
  }

  if (level_a > Integral::BRIEF) {
    Console::put(L"testing append method...");
  }
  
  prdrl.clear(Integral::RESET);
  prdrl.setRuleName(L"B");
  prdrl.append(ProductionRuleTokenType::TERMINAL, L"a");
  prdrl.append(ProductionRuleTokenType::CONCATENATION, L"", .42);
  prdrl.append(ProductionRuleTokenType::OPEN_PAREN);
  prdrl.append(ProductionRuleTokenType::TERMINAL, L"b");
  prdrl.append(ProductionRuleTokenType::ALTERNATION);
  prdrl.append(ProductionRuleTokenType::NON_TERMINAL, L"C");
  prdrl.append(ProductionRuleTokenType::CLOSE_PAREN);
  prdrl.append(ProductionRuleTokenType::KLEENE_STAR);

  if (level_a > Integral::BRIEF) {
    Console::put(L"testing gotoFirst method...");
  }  
  if (!prdrl.gotoFirst()) {
    return Error::handle(name(), L"gotoFirst", Error::TEST, __FILE__, __LINE__);
  }
  
  if (level_a > Integral::BRIEF) {
    Console::put(L"testing gotoNext method...");
  }  
  if (!prdrl.gotoNext()) {
    return Error::handle(name(), L"gotoNext", Error::TEST, __FILE__, __LINE__);
  }
  
  if (level_a > Integral::BRIEF) {
    Console::put(L"testing gotoPrev method...");
  }  
  if (!prdrl.gotoPrev()) {
    return Error::handle(name(), L"gotoPrev", Error::TEST, __FILE__, __LINE__);
  }

  if (level_a > Integral::BRIEF) {
    Console::put(L"testing getType method...");
  }  
  if (prdrl.getType()!=ProductionRuleTokenType::TERMINAL) {
    return Error::handle(name(), L"getType", Error::TEST, __FILE__, __LINE__);
  }

  if (level_a > Integral::BRIEF) {
    Console::put(L"testing getValue method...");
  }  
  if (!prdrl.getValue().eq(L"a")) {
    return Error::handle(name(), L"getValue", Error::TEST, __FILE__, __LINE__);
  }

  prdrl.gotoNext();
  if (level_a > Integral::BRIEF) {
    Console::put(L"testing getWeight method...");
  }  
  if (!prdrl.getWeight().eq(.42)) {
    return Error::handle(name(), L"getWeight", Error::TEST, __FILE__, __LINE__);
  }

  if (level_a > Integral::BRIEF) {
    Console::put(L"testing insert method...");
  }
  prdrl.insert(ProductionRuleTokenType::TERMINAL, L"f");
  prdrl.insert(ProductionRuleTokenType::CONCATENATION, L"", .75);
  prdrl.gotoPrev();
  prdrl.gotoPrev();
  
  if (!prdrl.getWeight().eq(.42) || !prdrl.gotoNext() ||
      !prdrl.getValue().eq(L"f") || !prdrl.gotoNext() ||
      !prdrl.getWeight().eq(.75)) {
    return Error::handle(name(), L"insert", Error::TEST, __FILE__, __LINE__);
  }

  if (level_a > Integral::BRIEF) {
    Console::put(L"testing getRule method...");
  }
  if (!prdrl.getRule().eq(L"B -> a f (b|C)*")) {
    Console::increaseIndention();
    Console::put((SysString)prdrl.getRule());
    Console::decreaseIndention();
    return Error::handle(name(), L"getRule", Error::TEST, __FILE__, __LINE__);
  }

  if (level_a > Integral::BRIEF) {
    Console::put(L"testing getRuleName method...");
  }
  if (!prdrl.getRuleName().eq(L"B")) {
    return Error::handle(name(), L"getRuleName", Error::TEST, __FILE__, __LINE__);
  }
  
  
  if (level_a > Integral::NONE) {
    Console::decreaseIndention();
  }

  //--------------------------------------------------------------------------
  //
  // 3. Test file i/o
  //
  //--------------------------------------------------------------------------    
  
  // set indention
  //
  if (level_a > Integral::NONE) {
    Console::put(L"testing file i/o methods...\n");
    Console::increaseIndention();
  }
  
  // test file i/o methods
  Filename out_file(L"testout1.sof");
  SysString temp_file1(L"testout1.sof");
  File::registerTemp(temp_file1);
  
  Sof sof;

  if (level_a > Integral::BRIEF) {
    Console::put(L"testing ascii write...");
  }  
  // test text
  //
  if(!sof.open(out_file, File::WRITE_ONLY)) {
    return Error::handle(out_file, L"open", Error::TEST,
			 __FILE__, __LINE__);
  }

  prdrl.write(sof, 0);
  sof.close();

  if (level_a > Integral::BRIEF) {
    Console::put(L"testing ascii read...");
  }  
  
  if(!sof.open(out_file, File::READ_ONLY)) {
    return Error::handle(out_file, L"open", Error::TEST,
			 __FILE__, __LINE__);
  }

  prdrl2.read(sof, 0);
  sof.close();

  if (!prdrl.eq(prdrl2)) {
    return Error::handle(name(), L"open", Error::TEST,
			 __FILE__, __LINE__);
  }

  // test binary
  //
  prdrl2.clear(Integral::RESET);

  out_file=L"testout2.sof";
  SysString temp_file2(L"testout2.sof");
  File::registerTemp(temp_file2);
  
  if (level_a > Integral::BRIEF) {
    Console::put(L"testing binary write...");
  }    
  if(!sof.open(out_file, File::WRITE_ONLY, File::BINARY)) {
    return Error::handle(out_file, L"open", Error::TEST,
			 __FILE__, __LINE__);
  }

  prdrl.write(sof, 0);
  sof.close();

  if (level_a > Integral::BRIEF) {
    Console::put(L"testing binary read...");
  }    
  if(!sof.open(out_file, File::READ_ONLY)) {
    return Error::handle(out_file, L"open", Error::TEST,
			 __FILE__, __LINE__);
  }

  prdrl2.read(sof, 0);
  sof.close();

  if (!prdrl.eq(prdrl2)) {
    return Error::handle(name(), L"open", Error::TEST,
			 __FILE__, __LINE__);
  }
  
  

  if (level_a > Integral::NONE) {
    Console::decreaseIndention();
  }
  
  
  //--------------------------------------------------------------------------
  //
  // 4. Clean up.
  //
  //--------------------------------------------------------------------------    

  File::cleanTemps();
  
  // reset indention
  //
  if (level_a > Integral::NONE) {
    Console::decreaseIndention();
  }
  
  if (level_a > Integral::NONE) {
    SysString output(L"diagnostics passed for class ");
    output.concat(name());
    output.concat(L"\n");
    Console::put(output);
  }
  
  // indicate success
  //
  return true;
}
