// file: $isip/class/pr/ProductionRule/prdrl_04.cc
// version: $Id: prdrl_04.cc 10372 2006-01-19 15:34:47Z may $

//isip include files
//
#include "ProductionRule.h"

// method: write
//
// arguments:
//  Sof& sof: (input) sof file object
//  int32 tag: (input) sof object instance tag
//  const String& name: (input) sof object instance name
//
// return: bool8 value indicating status
//
// this method has the object write itself to an Sof file
//
bool8 ProductionRule::write(Sof& sof_a, int32 tag_a,
			     const String& name_a) const {
  int32 obj_size = 0;

  // figure out size
  //
  if (sof_a.isText()) {

    //dynamic for text
    //
    obj_size = Sof::ANY_SIZE;
  }
  else {
    obj_size = sofSize();
  }

  // write object into sof index
  //
  if (!sof_a.put(name_a, tag_a, obj_size)) {
    return false;
  }
  
  // exit gracefully
  //
  return writeData(sof_a);
}

// method: writeData
//
// arguments:
//  Sof& sof: (input) sof file object
//  const String& pname: (input) parameter name
//
// return: bool8 value indicating status
//
// this method has the object write itself to an Sof file.
//
bool8 ProductionRule::writeData(Sof& sof_a, const String& pname_a) const {

  // write a start string if necessary
  //
  sof_a.writeLabelPrefix(pname_a);

  // write the rule type
  //
  TYPE_MAP.writeElementData(sof_a, PARAM_RULE_TYPE, (ushort)rule_type_d);

  if (!rule_name_d.writeData(sof_a, PARAM_RULE_NAME)) {
    return Error::handle(name(), L"writeData",Error::IO, __FILE__, __LINE__);
  }

  if (!rule_d.writeData(sof_a, PARAM_RULE)) {
    return Error::handle(name(), L"writeData",Error::IO, __FILE__, __LINE__);
  }

  // put an end string if necessary
  //
  sof_a.writeLabelSuffix(pname_a);
  
  return true;
}
