// file: $isip/class/pr/RegressionDecisionTree/rdt_04.cc
// version: $Id: rdt_04.cc 9457 2004-04-19 15:06:59Z gao $
//

// isip include files
//
#include "RegressionDecisionTree.h"

// method: sofSize
//
// arguments: none 
//
// return: size of object
//
// this method returns the size of the object in the Sof file and is
// used for binary write
//
int32 RegressionDecisionTree::sofSize() const {

  // start with the space required for the algorithm name
  //
  int32 bytes = ALGO_MAP.elementSofSize();

  // add the space required for the implementation name
  //
  bytes += IMPL_MAP.elementSofSize();

  // add the space required for the stopmode
  //
  //  bytes += STOPMODE_MAP.elementSofSize();

  // add the space required for the runmode
  //
  //  bytes += RUNMODE_MAP.elementSofSize();

  // add the space required for the speech flag
  //
  bytes += speech_flag_d.sofSize();

  // add the space required for the number of terminals
  //
  bytes += num_terminals_d.sofSize();

  // add the space required for the perturb depth
  //
  bytes += perturb_depth_d.sofSize();

  // add the space required for the occ threshold
  //
  bytes += occ_threshold_d.sofSize();

  // add the space required for the components threshold
  //
  bytes += components_threshold_d.sofSize();

  // add the space required for the mixture offset
  //
  bytes += mixture_offset_d.sofSize();

  // add the space required for the map from statistical to
  // transformation matrix
  //
  bytes += map_stat_to_trans_d.sofSize();  

  // add the space required for the base BiGraph
  //
  bytes += BiGraph<RegressionDecisionTreeNode>::sofSize();
  
  // add the space required for the debug level
  //
  bytes += debug_level_d.sofSize();

  // add the space for other parameters defined in the
  // DecisionTreeBase class
  //
  //  bytes += attributes_d.sofSize();

  // return the size
  //
  return bytes;
}

// method: write
//
// arguments:
//  Sof& sof: (input) sof file object
//  int32 tag: (input) sof object instance tag
//  const String& name: (input) sof object instance name
//
// return: a bool8 value indicating status
//
// this method has the object write itself to an Sof file
//
bool8 RegressionDecisionTree::write(Sof& sof_a, int32 tag_a,
				      const String& name_a) const {

  // declare local variable
  //
  int32 obj_size;
  
  // write the instance of the object into the Sof file
  //
  if (sof_a.isText()) {

    // set the size to be dynamic
    //
    obj_size = Sof::ANY_SIZE;
  }
  else {

    // set the size to be the size of the object written to the Sof file
    //
    obj_size = sofSize();
  }

  // write the object into the sof file's index
  //
  if (!sof_a.put(name_a, tag_a, obj_size)) {
    return false;
  }
  
  // write data and exit gracefully
  //
  return writeData(sof_a);
}

// method: writeData
//
// arguments:
//  Sof& sof: (input) sof file object
//  const String& pname: (input) parameter name
//
// return: a bool8 value indicating status
//
// this method branches on the algorithm name
//
bool8 RegressionDecisionTree::writeData(Sof& sof_a,
					  const String& pname_a) const {
  
  // declare local variables
  //
  bool8 status = false;
  
  // write a start string if necessary
  //
  sof_a.writeLabelPrefix(pname_a);

  // write the stopmode (from the base class)
  //
  //  STOPMODE_MAP.writeElementData(sof_a, PARAM_STOPMODE, (int32)stopmode_d);
    
  // write the runmode (from the base class)
  //
  //  RUNMODE_MAP.writeElementData(sof_a, PARAM_RUNMODE, (int32)runmode_d);

  // write the algorithm name
  //
  ALGO_MAP.writeElementData(sof_a, PARAM_ALGORITHM, (int32)algorithm_d);
    
  // write the implementation type
  //
  IMPL_MAP.writeElementData(sof_a, PARAM_IMPLEMENTATION,
			    (int32)implementation_d);
  
  // write the speech flag
  //
  speech_flag_d.writeData(sof_a, PARAM_SPEECH_FLAG);

  // write the number of terminals
  //
  num_terminals_d.writeData(sof_a, PARAM_NUM_TERMINALS);

  // write the perturb depth
  //
  perturb_depth_d.writeData(sof_a, PARAM_PERTURB_DEPTH);
  
  // write the occ threshold
  //
  occ_threshold_d.writeData(sof_a, PARAM_OCC_THRESHOLD);
  
  // write the components threshold
  //
  components_threshold_d.writeData(sof_a, PARAM_COMPONENTS_THRESHOLD);

  // write the mixture offset
  //
  mixture_offset_d.writeData(sof_a, PARAM_MIXTURE_OFFSET);
  
  // write the map from index from statistical model to node index of
  // transformation matrix
  //
  map_stat_to_trans_d.writeData(sof_a, PARAM_MAP_STAT_TO_TRANS);
  
  // write the BiGraph (base class)
  //
  if (!BiGraph<RegressionDecisionTreeNode>::writeData(sof_a, PARAM_BDT)) {
    return Error::handle(name(), L"writeData", Error::ARG,
			 __FILE__, __LINE__);
  }
  
  // write the debug level (from the base class)
  //
  debug_level_d.writeData(sof_a, PARAM_DBGL);
 
  // put an end string if necessary
  //
  sof_a.writeLabelSuffix(pname_a);
  
  // exit gracefully
  //
  return status;
}
