// file: $isip_ifc/class/pr/RelevanceVectorMachine/rvm_03.cc
// version: $Id: rvm_03.cc 10424 2006-02-10 20:37:45Z raghavan $
//

// isip include files
//
#include "RelevanceVectorMachine.h"

// method: read
//
// arguments:
//  Sof& sof: (input) sof file object
//  int32 tag: (input) sof object instance tag
//  const String& name: (input) sof object instance name
//
// return: a bool8 value indicating status
//
// this method has the object read itself from an Sof file according
// to the specified name and tag
//
bool8 RelevanceVectorMachine::read(Sof& sof_a, int32 tag_a,
				     const String& name_a) {

  // get the instance of the object from the Sof file
  //
  if (!sof_a.find(name_a, tag_a)) {
    return false;
  }

  // read the actual data from the sof file
  //
  return readData(sof_a);
}

// method: readData
//
// arguments:
//  Sof& sof: (input) sof file object
//  const String& pname: (input) parameter name
//  int32 size: (input) size in bytes of object (or FULL_OBJECT)
//  bool8 param: (input) is the parameter name in the file?
//  bool8 nested: (input) are we nested?
//
// return: a bool8 value indicating status
//
// this method has the object read itself from an Sof file. it assumes
// that the Sof file is already positioned correctly
//
bool8 RelevanceVectorMachine::readData(Sof& sof_a, const String& pname_a,
			 int32 size_a, bool8 param_a, bool8 nested_a) {

  Integral::DEBUG debug_level = Integral::NONE;

  SofParser parser;

  parser.setDebug(debug_level);
    
  // ignore implicit parameter setting
  //
  // are we nested?
  //
  if (nested_a) {
    parser.setNest();
  }

  // load the parse
  //
  if (!parser.load(sof_a, size_a)) {
    return Error::handle(name(), L"readData", Error::READ,
			 __FILE__, __LINE__, Error::WARNING);
  }
  
  // read the algorithm
  //
  if (parser.isPresent(sof_a, PARAM_ALGORITHM)) {
    if (!ALGO_MAP.readElementData((int32&)algorithm_d, sof_a, PARAM_ALGORITHM,
				  parser.getEntry(sof_a, PARAM_ALGORITHM))) {
      return Error::handle(name(), L"readData", Error::READ,
			   __FILE__, __LINE__, Error::WARNING);
    }       
  }
  else {
    algorithm_d = DEF_ALGORITHM;
  }
  
  // read the implementation
  //
  if (parser.isPresent(sof_a, PARAM_IMPLEMENTATION)) {
    if (!IMPL_MAP.readElementData((int32&)implementation_d, sof_a,
				  PARAM_IMPLEMENTATION,
			      parser.getEntry(sof_a, PARAM_IMPLEMENTATION))) {
      return Error::handle(name(), L"readData", Error::READ,
			   __FILE__, __LINE__, Error::WARNING);
    }       
  }
  else {
    implementation_d = DEF_IMPLEMENTATION;
  }
  
  // read the bias
  //
  if (parser.isPresent(sof_a, PARAM_BIAS)) {
    if (!bias_d.readData(sof_a, PARAM_BIAS,
			 parser.getEntry(sof_a, PARAM_BIAS))) {
      
      // return a warning message
      //
      return Error::handle(name(), L"readData", Error::READ,
			   __FILE__, __LINE__, Error::WARNING);
      
    }
  }
  
  // read the weights
  //
  if (parser.isPresent(sof_a, PARAM_WEIGHTS)) {
      if (!weights_d.readData(sof_a, PARAM_WEIGHTS,
			      parser.getEntry(sof_a, PARAM_WEIGHTS),
			      false, false)) {
      
	// return a warning message
	//
	return Error::handle(name(), L"readData", Error::READ,
			     __FILE__, __LINE__, Error::WARNING);
      }
  }

  // read the hyperparameters
  //
  if (parser.isPresent(sof_a, PARAM_INV_HESSIAN)) {
    if (!inv_hessian_d.readData(sof_a, PARAM_INV_HESSIAN,
			      parser.getEntry(sof_a, PARAM_INV_HESSIAN),
			      true, true)) {
      
	// return a warning message
	//
	return Error::handle(name(), L"readData", Error::READ,
			     __FILE__, __LINE__, Error::WARNING);
      }
  }
  
  // read the targets
  //
  if (parser.isPresent(sof_a, PARAM_TARGETS)) {
      if (!targets_d.readData(sof_a, PARAM_TARGETS,
			      parser.getEntry(sof_a, PARAM_TARGETS),
			      false, false)) {
      
	// return a warning message
	//
	return Error::handle(name(), L"readData", Error::READ,
			     __FILE__, __LINE__, Error::WARNING);
	
      }
  }

  // read the relevance vectors
  //
  if (parser.isPresent(sof_a, PARAM_VECTORS)) {
      if (!vectors_d.readData(sof_a, PARAM_VECTORS,
			      parser.getEntry(sof_a, PARAM_VECTORS),
			      false, false)) {
      
	// return a warning message
	//
	return Error::handle(name(), L"readData", Error::READ,
			     __FILE__, __LINE__, Error::WARNING);
	
      }
  }

  // read the polynomial degree
  //
  if (parser.isPresent(sof_a, PARAM_POLYNOMIAL_DEGREE)) {
    if (!polynomial_degree_d.readData(sof_a, PARAM_POLYNOMIAL_DEGREE,
		      parser.getEntry(sof_a, PARAM_POLYNOMIAL_DEGREE))) {
      
      // return a warning message
      //
      return Error::handle(name(), L"readData", Error::READ,
			   __FILE__, __LINE__, Error::WARNING);
      
    }
  }    

  // read the rbf gamma
  //
  if (parser.isPresent(sof_a, PARAM_RBF_GAMMA)) {
    if (!rbf_gamma_d.readData(sof_a, PARAM_RBF_GAMMA,
		      parser.getEntry(sof_a, PARAM_RBF_GAMMA))) {
      
      // return a warning message
      //
      return Error::handle(name(), L"readData", Error::READ,
			   __FILE__, __LINE__, Error::WARNING);
      
    }
  }    

  // read the sigmoid kappa
  //
  if (parser.isPresent(sof_a, PARAM_SIGMOID_KAPPA)) {
    if (!sigmoid_kappa_d.readData(sof_a, PARAM_SIGMOID_KAPPA,
		      parser.getEntry(sof_a, PARAM_SIGMOID_KAPPA))) {
      
      // return a warning message
      //
      return Error::handle(name(), L"readData", Error::READ,
			   __FILE__, __LINE__, Error::WARNING);
      
    }
  }    

  // read the sigmoid delta
  //
  if (parser.isPresent(sof_a, PARAM_SIGMOID_DELTA)) {
    if (!sigmoid_delta_d.readData(sof_a, PARAM_SIGMOID_DELTA,
		      parser.getEntry(sof_a, PARAM_SIGMOID_DELTA))) {
      
      // return a warning message
      //
      return Error::handle(name(), L"readData", Error::READ,
			   __FILE__, __LINE__, Error::WARNING);
      
    }
  }
  
  // read the kernel
  //
  if (parser.isPresent(sof_a, PARAM_KERNEL)) {
    if (!kernel_d.readData(sof_a, PARAM_KERNEL,
			   parser.getEntry(sof_a, PARAM_KERNEL),
			   true, true)) {
      
      // return a warning message
      //
      return Error::handle(name(), L"readData", Error::READ,
			   __FILE__, __LINE__, Error::WARNING);
      
    }
  }
  
  // read the audio database file
  //
  if (parser.isPresent(sof_a, PARAM_AUDIO_DB)) {
    if (!audio_db_file_d.readData(sof_a, PARAM_AUDIO_DB,
			       parser.getEntry(sof_a,
					       PARAM_AUDIO_DB))) {
      return Error::handle(name(), L"readData", Error::READ,
			   __FILE__, __LINE__, Error::WARNING);
    }       
  }

  // read the output type
  //
  if (parser.isPresent(sof_a, PARAM_OUTPUT_TYPE)) {
    if (!TYPE_MAP.readElementData((int32&)output_type_d, sof_a,
					 PARAM_OUTPUT_TYPE,
				  parser.getEntry(sof_a, PARAM_OUTPUT_TYPE))) {
      return Error::handle(name(), L"readData", Error::READ,
			   __FILE__, __LINE__, Error::WARNING);
    }       
  }

  // read the output file name
  //
  if (parser.isPresent(sof_a, PARAM_OUTPUT_FILE)) {
    if (!output_file_d.readData(sof_a, PARAM_OUTPUT_FILE,
				parser.getEntry(sof_a,
						PARAM_OUTPUT_FILE))) {
      return Error::handle(name(), L"readData", Error::READ,
			   __FILE__, __LINE__, Error::WARNING);
    }       
  }
  
  // check that all parameters are accounted for
  //
  if (!parser.checkParams(sof_a)) {
    
    // return a warning message
    //
    return Error::handle(name(), L"readData", Error::IO,
			 __FILE__, __LINE__, Error::WARNING);
  }
  
  // exit gracefully
  //
  return true;
}
