// file: $isip/class/pr/StatisticalModelAdaptation/smadp_02.cc
// version: $Id: smadp_02.cc 9453 2004-04-13 19:28:26Z gao $
//

// isip include files
//
#include "StatisticalModelAdaptation.h"
#include <Console.h>

// method: diagnose
//
// arguments:
//  Integral::DEBUG level: (input) debug level for diagnostics
//
// return: logical error status
//
// this is the diagnostics method
//
bool8 StatisticalModelAdaptation::diagnose(Integral::DEBUG level_a) {
  
  //----------------------------------------------------------------------
  //
  // 0. preliminaries
  //
  //----------------------------------------------------------------------

  // output the class name
  //
  if (level_a > Integral::NONE) {
    SysString output(L"diagnosing class ");
    output.concat(CLASS_NAME);
    output.concat(L": ");
    Console::put(output);
    Console::increaseIndention();
  }

  //---------------------------------------------------------------------
  //
  // 1. required public methods
  //
  //---------------------------------------------------------------------


  // set indentation
  //
  if (level_a > Integral::NONE) {
    Console::put(L"testing required public methods...\n");
    Console::increaseIndention();
  }

  // test destructor/constructor(s) and memory management
  //
  StatisticalModelAdaptation rdt0;
  StatisticalModelAdaptation rdt1(rdt0);
  
  if (!rdt1.eq(rdt0)) {
    return Error::handle(name(), L"copy constructor", Error::TEST,
			 __FILE__, __LINE__);
  }
  
  // test large allocation construction and deletion
  //
  if (level_a == Integral::ALL) {
    
    Console::put(L"\ntesting large chunk memory allocation and deletion:\n");
    
    // set the memory to a strange block size so we can hopefully catch any
    // frame overrun errors
    //
    StatisticalModelAdaptation::setGrowSize((int32)500);
    
    StatisticalModelAdaptation* rdt = new StatisticalModelAdaptation();

    for (int32 j = 1; j <= 100; j++) {
      StatisticalModelAdaptation** rdts = new StatisticalModelAdaptation*[j * 100];
      
      // create the objects
      //
      for (int32 i = 0; i < j * 100; i++) {
	rdts[i] = new StatisticalModelAdaptation();
      }
      
      // delete objects
      //
      for (int32 i = (j * 100) - 1; i >= 0; i--) {
	delete rdts[i];
      }
      
      delete [] rdts;
    }
    
    delete rdt;
  }

  // reset indentation
  //
  if (level_a > Integral::NONE) {
    Console::decreaseIndention();
  }

  //---------------------------------------------------------------------
  //
  // 2. class-specific public methods
  //
  //---------------------------------------------------------------------

  // set indentation
  //
  if (level_a > Integral::NONE) {
    Console::put(L"testing class-specific public methods...\n");
    Console::increaseIndention(); 
  }

  // reset indentation
  //
  if (level_a > Integral::NONE) {
    Console::decreaseIndention();
  }
  
  //---------------------------------------------------------------------
  //
  // 3. print completion message
  //
  //---------------------------------------------------------------------

  // reset indentation
  //
  if (level_a > Integral::NONE) {
    Console::decreaseIndention();
  }
  
  if (level_a > Integral::NONE) {
    SysString output(L"diagnostics passed for class");
    output.concat(name());
    output.concat(L"\n");
    Console::put(output);
  }
  
  // exit gracefully
  //
  return true;
}
