// file: $isip/class/pr/SupportVectorMachine/svm_04.cc
//

// isip include files
//
#include "SupportVectorMachine.h"

// method: sofSize
//
// arguments: none
//
// return: size of object
//
// this method returns the size of the object in the Sof file and is
// used for binary write
//
int32 SupportVectorMachine::sofSize() const {

  // return the space required for this object
  //
  int32 bytes = tol_d.sofSize() + eps_d.sofSize() + penalty_d.sofSize() +
    polynomial_degree_d.sofSize() + rbf_gamma_d.sofSize() +
    sigmoid_kappa_d.sofSize() + sigmoid_delta_d.sofSize();
  
  // return the size
  //
  return bytes;
}

// method: write
//
// arguments:
//  Sof& sof: (input) sof file object
//  int32 tag: (input) sof object instance tag
//  const String& name: (input) sof object instance name
//
// return: a bool8 value indicating status
//
// this method has the object write itself to an Sof file
//
bool8 SupportVectorMachine::write(Sof& sof_a, int32 tag_a,
				      const String& name_a) const {

  // declare local variable
  //
  int32 obj_size;
  
  // write the instance of the object into the Sof file
  //
  if (sof_a.isText()) {

    // set the size to by dynamic
    //
    obj_size = Sof::ANY_SIZE;
  }
  else {

    // set the size to be the size of the object written to the Sof file
    //
    obj_size = sofSize();
  }

  // write the object into the sof file's index
  //
  if (!sof_a.put(name_a, tag_a, obj_size)) {
    return false;
  }
  
  // write data and exit gracefully
  //
  return writeData(sof_a);
}

// method: writeData
//
// arguments:
//  Sof& sof: (input) sof file object
//  const String& pname: (input) parameter name
//
// return: a bool8 value indicating status
//
// this method has the object write itself to an Sof file. it assumes
// that the Sof file is already positioned correctly
//
bool8 SupportVectorMachine::writeData(Sof& sof_a,
					  const String& pname_a) const {

  // write a start string if necessary
  //
  sof_a.writeLabelPrefix(pname_a);

  // write the algorithm
  //
  ALGO_MAP.writeElementData(sof_a, PARAM_ALGORITHM, (int32)algorithm_d);

  // write the implementation
  //
  IMPL_MAP.writeElementData(sof_a, PARAM_IMPLEMENTATION, (int32)implementation_d);
  
  // write the penalty
  //
  penalty_d.writeData(sof_a, PARAM_PENALTY);
  
  // write the tolerance
  //
  tol_d.writeData(sof_a, PARAM_TOLERANCE);
  
  // write the epsilon
  //  
  eps_d.writeData(sof_a, PARAM_EPSILON);

  // write the polynomial degree
  //
  polynomial_degree_d.writeData(sof_a, PARAM_POLYNOMIAL_DEGREE);

  // write the rbf gamma
  //  
  rbf_gamma_d.writeData(sof_a, PARAM_RBF_GAMMA);

  // write the sigmoid kappa
  //  
  sigmoid_kappa_d.writeData(sof_a, PARAM_SIGMOID_KAPPA);

  // write the sigmoid delta
  //    
  sigmoid_delta_d.writeData(sof_a, PARAM_SIGMOID_DELTA);
  
  // put an end string if necessary
  //
  sof_a.writeLabelSuffix(pname_a);
  
  // exit gracefully
  //
  return true;
}
