// file: $isip/class/search/HistoryPool/HistoryPool.h
//

// make sure definitions are only made once
//
#ifndef ISIP_HISTORY_POOL
#define ISIP_HISTORY_POOL

// isip include files
//
#ifndef ISIP_HISTORY
#include <History.h>
#endif

#ifndef ISIP_CONTEXT
#include <Context.h>
#endif

#ifndef ISIP_HASH_TABLE
#include <HashTable.h>
#endif

// HistoryPool: A class to store and manage a pool of histories
//
class HistoryPool {

  //---------------------------------------------------------------------------
  //
  // public constants
  //
  //---------------------------------------------------------------------------
public:

  // define the class name
  //
  static const String CLASS_NAME;

  //----------------------------------------
  //
  // i/o related constants
  //
  //----------------------------------------  

  static const String DEF_PARAM;

  //----------------------------------------
  //
  // other important constants
  //
  //----------------------------------------  
  
  //----------------------------------------
  //
  // default values and arguments
  //
  //----------------------------------------  

  // default values
  //
  static const int32 DEF_CAPACITY = 20000;
  
  //---------------------------------------
  //
  // error codes
  //
  //---------------------------------------
  
  //---------------------------------------------------------------------------
  //
  // protected data
  //
  //---------------------------------------------------------------------------
protected:
  
  // define the history pool
  //
  HashTable<History, History> pool_d;
  
  // define a static debug level
  //
  static Integral::DEBUG debug_level_d;
  
  // define a static memory manager
  //
  static MemoryManager mgr_d;

  //---------------------------------------------------------------------------
  //
  // required public methods
  //
  //---------------------------------------------------------------------------
public:

  // method: name
  //
  static const String& name() {
    return CLASS_NAME;
  }

  // method: diagnose
  //
  static bool8 diagnose(Integral::DEBUG debug_level);

  // method: debug
  //
  bool8 debug(const unichar* message) const;

  // method: setDebug
  //
  static bool8 setDebug(Integral::DEBUG debug_level) {
    debug_level_d = debug_level;
    return true;
  }
  
  // method: destructor
  //
  ~HistoryPool() {

    if (debug_level_d >= Integral::ALL) {     
      fprintf(stdout, "Destructor of history_pool: %p\n", this);
      fflush(stdout);
    }

    // free allocated memory
    //
    clear();
  }

  // constructor(s)
  //
  HistoryPool();
  HistoryPool(const HistoryPool& copy_node);
  
  // assign methods
  //
  bool8 assign(const HistoryPool& copy_node);

  // method: sofSize
  //
  int32 sofSize() const {
    return Error::handle(name(), L"sofSize", Error::ARG, __FILE__, __LINE__);
  }
  
  // method: read
  //
  bool8 read(Sof& sof, int32 tag, const String& cname = CLASS_NAME) {
    return Error::handle(name(), L"read", Error::ARG, __FILE__, __LINE__);
  }

  // method: write
  //
  bool8 write(Sof& sof, int32 tag, const String& cname = CLASS_NAME) const {
    return Error::handle(name(), L"write", Error::ARG, __FILE__, __LINE__);
  }

  // method: readData
  //
  bool8 readData(Sof& sof, const String& pname = DEF_PARAM,
		   int32 size = SofParser::FULL_OBJECT, bool8 param = true,
                   bool8 nested = false) {
    return Error::handle(name(), L"readData", Error::ARG, __FILE__, __LINE__);
  }

  // method: writeData
  //
  bool8 writeData(Sof& sof, const String& pname = DEF_PARAM) const {
    return Error::handle(name(), L"writeData", Error::ARG, __FILE__, __LINE__);
  }

  // equality method
  //
  bool8 eq(const HistoryPool& compare_node) const;
  
  // method: new
  //
  static void* operator new(size_t size) {
    return mgr_d.get();
  }

  // method: new[]
  //
  static void* operator new[](size_t size) {
    return mgr_d.getBlock(size);
  }

  // method: delete
  //
  static void operator delete(void* ptr) {
    mgr_d.release(ptr);
  }

  // method: delete[]
  //
  static void operator delete[](void* ptr) {
    mgr_d.releaseBlock(ptr);
  }

  // method: setGrowSize
  //
  static bool8 setGrowSize(int32 grow_size) {
    return mgr_d.setGrow(grow_size);
  }

  // clear methods
  //
  bool8 clear(Integral::CMODE ctype = Integral::DEF_CMODE);

  //---------------------------------------------------------------------------
  //
  // class-specific public methods
  //
  //---------------------------------------------------------------------------

  // method: length
  //
  int32 length() const {
    return pool_d.getNumItems();
  }
  
  // allocate methods
  //
  History* initAndAllocate();    
  History* popAndAllocate(History* obj, Context*& val);
  History* pushAndAllocate(History* obj, Context* val);    

  //---------------------------------------------------------------------------
  //
  // private methods
  //
  //---------------------------------------------------------------------------
private:

};

// end of include file
//
#endif
