// file: $isip/class/sp/FrontEnd/FrontEndUndefined.h
// version: $Id: FrontEndUndefined.h 9236 2003-06-23 18:12:22Z gao $
//

// make sure definitions are only made once
//
#ifndef ISIP_FRONTEND_UNDEFINED
#define ISIP_FRONTEND_UNDEFINED

#ifndef ISIP_FRONTEND_BASE
#include <FrontEndBase.h>
#endif

// FrontEndUndefined: a class that is used to define an undefined front end
// object. usually, this class is used for initialization purposes.
//
class FrontEndUndefined : public FrontEndBase {

  //---------------------------------------------------------------------------
  //
  // public constants
  //
  //---------------------------------------------------------------------------
public:

  // define the class name
  //
  static const String CLASS_NAME;

  //----------------------------------------
  //
  // default values and arguments
  //
  //----------------------------------------  

  // define the default value(s) of the class data
  //
  
  //----------------------------------------
  //
  // error codes
  //
  //----------------------------------------  

  static const int32 ERR = (int32)80260;    
  
  //---------------------------------------------------------------------------
  //
  // protected data
  //
  //---------------------------------------------------------------------------

protected:

  //---------------------------------------------------------------------------
  //
  // required public methods
  //
  //---------------------------------------------------------------------------
public:
  
  // static methods:
  //  the diagnose method is not needed
  //
  static const String& name() {
    return CLASS_NAME;
  }
  
  // debug methods
  //  setDebug is inherited from base class
  //
  inline bool8 debug(const unichar* msg) const {
    return Error::handle(name(), L"debug",
			 Error::VIRTUAL_PTR, __FILE__, __LINE__);
  }

  // method: destructor
  //
  ~FrontEndUndefined() { }

  // method: default constructor

  FrontEndUndefined() { }

  // assign methods
  //
  
  // i/o methods
  //
  int32 sofSize() const {
    return Error::handle(name(), L"sofSize",
			 Error::VIRTUAL_PTR, __FILE__, __LINE__);
  }
  
  bool8 read(Sof& sof, int32 tag, const String& name = CLASS_NAME) {
    return Error::handle(CLASS_NAME, L"read",
			 Error::VIRTUAL_PTR, __FILE__, __LINE__);
  }
  
  bool8 write(Sof& sof, int32 tag, const String& name = CLASS_NAME) const {
    return Error::handle(CLASS_NAME, L"write",
			 Error::VIRTUAL_PTR, __FILE__, __LINE__);
  }
  
  bool8 readData(Sof& sof, const String& pname,
		   int32 size, bool8 param, bool8 nested) {
    return Error::handle(CLASS_NAME, L"readData",
			 Error::VIRTUAL_PTR, __FILE__, __LINE__);
  }

  bool8 writeData(Sof& sof,
		    const String& pname = String::getEmptyString()) const {
    return Error::handle(CLASS_NAME, L"writeData",
			 Error::VIRTUAL_PTR, __FILE__, __LINE__);
  }
  
  // equality methods
  //
  bool8 eq(const FrontEndUndefined& arg_a) const {
    return Error::handle(CLASS_NAME, L"eq",
			 Error::VIRTUAL_PTR, __FILE__, __LINE__);
  }

  // method: clear
  //
  bool8 clear(Integral::CMODE cmode = Integral::DEF_CMODE) {
    return Error::handle(CLASS_NAME, L"clear",
			 Error::VIRTUAL_PTR, __FILE__, __LINE__);
  }
  
  //---------------------------------------------------------------------------
  //
  // class-specific public methods:
  //  set methods
  //
  //---------------------------------------------------------------------------

  // method: setNumChannels
  //
  inline bool8 setNumChannels(int32 arg) {
    return Error::handle(CLASS_NAME, L"setNumChannels",
			 Error::VIRTUAL_PTR, __FILE__, __LINE__);
  }
  
  // method: setCoefName
  //
  inline bool8 setCoefName(const String& name) {
    return Error::handle(CLASS_NAME, L"setCoefName",
			 Error::VIRTUAL_PTR, __FILE__, __LINE__);
  }

  // method: setOutputMode
  //
  inline bool8 setOutputMode(DATA_MODE arg) {
    return Error::handle(CLASS_NAME, L"setOutputMode",
			 Error::VIRTUAL_PTR, __FILE__, __LINE__);
  }

  // method: setOutputBasename
  //
  inline bool8 setOutputBasename(String& basename) {
    return Error::handle(CLASS_NAME, L"setOutpuBas",
			 Error::VIRTUAL_PTR, __FILE__, __LINE__);
  }

  // method: setOutputExtension
  //
  inline bool8 setOutputExtension(String& extension) {
    return Error::handle(CLASS_NAME, L"setOutpuExt",
			 Error::VIRTUAL_PTR, __FILE__, __LINE__);
  }
  
  // method: setOutputSuffix
  //
  inline bool8 setOutputSuffix(String& suffix) {
    return Error::handle(CLASS_NAME, L"setOutpuSuf",
			 Error::VIRTUAL_PTR, __FILE__, __LINE__);
  }  

  // method: setOutputDirectory
  //
  inline bool8 setOutputDirectory(const String& new_dir) {
    return Error::handle(CLASS_NAME, L"setOutpuDir",
			 Error::VIRTUAL_PTR, __FILE__, __LINE__);
  }

  // method: setOutputPreserve
  //
  inline bool8 setOutputPreserve(int32 new_preserve_level) {
    return Error::handle(CLASS_NAME, L"setOutputPreserve",
			 Error::VIRTUAL_PTR, __FILE__, __LINE__);
  }
  
  // method: setOutputType
  //
  inline bool8 setOutputType(File::TYPE type) {
     return Error::handle(CLASS_NAME, L"setOutpuType",
			 Error::VIRTUAL_PTR, __FILE__, __LINE__);
  }
  
  
  //---------------------------------------------------------------------------
  //
  // class-specific public methods:
  //  get methods
  //
  //---------------------------------------------------------------------------

  // method: getFrameDuration
  //
  inline float32 getFrameDuration() const {
    return Error::handle(CLASS_NAME, L"getFrameDuration", Error::VIRTUAL_PTR,
			 __FILE__, __LINE__);
  }

  // method: getNumFrames
  //
  inline int32 getNumFrames() {
    return Error::handle(CLASS_NAME, L"getNumFrames", Error::VIRTUAL_PTR,
			 __FILE__, __LINE__);
  }  

  // method: getNumFeatures
  //
  inline int32 getNumFeatures() {
    return Error::handle(CLASS_NAME, L"getNumFeatures", Error::VIRTUAL_PTR,
			 __FILE__, __LINE__);

  }
  
  // method: getNumChannels
  //
  inline int32 getNumChannels() const {
    return Error::handle(CLASS_NAME, L"getNumChannels",
			 Error::VIRTUAL_PTR, __FILE__, __LINE__);
  }
  
  // method: getInputFlag
  //
  inline bool8 getInputFlag() const {
    return Error::handle(CLASS_NAME, L"getInputFlag",
			 Error::VIRTUAL_PTR, __FILE__, __LINE__);
  }
      
  // method: getOutputMode
  //
  inline DATA_MODE getOutputMode() const {
    return (DATA_MODE)Error::handle(CLASS_NAME, L"getOutputMode",
				    Error::VIRTUAL_PTR, __FILE__, __LINE__);
  }

  // method: getOutputBasename
  //
  inline const bool8 getOutputBasename(String& basename) const {
    Error::handle(CLASS_NAME, L"getOutputBasename",
		  Error::VIRTUAL_PTR, __FILE__, __LINE__);
    return true;
  }

  // method: getOutputExtension
  //
  inline const bool8 getOutputExtension(String& extension) const {
    Error::handle(CLASS_NAME, L"getOutputExtension",
		  Error::VIRTUAL_PTR, __FILE__, __LINE__);
    return true;
  }

  // method: getOutputSuffix
  //
  inline const bool8 getOutputSuffix(String& suffix) const {
    Error::handle(CLASS_NAME, L"getOutputSuffix",
		  Error::VIRTUAL_PTR, __FILE__, __LINE__);
    return true;
  }  

  // method: getOutputType
  //
  inline const File::TYPE& getOutputType() const {
    static File::TYPE type;
    Error::handle(CLASS_NAME, L"getOutputType",
		  Error::VIRTUAL_PTR, __FILE__, __LINE__);
    return type;
  }
  
  // method: getID
  //
  const String& getID() const {
    static String str;
    Error::handle(CLASS_NAME, L"getID",
		  Error::VIRTUAL_PTR, __FILE__, __LINE__);
    return str;
  }
  
  // method: isNameInput
  //
 inline bool8 isNameInput(const String& name) const {
    return Error::handle(CLASS_NAME, L"isNameInput",
			 Error::VIRTUAL_PTR, __FILE__, __LINE__);
  }
  
  //---------------------------------------------------------------------------
  //
  // class-specific public methods:
  //  core FrontEndBase interface contract methods
  //
  //---------------------------------------------------------------------------

  // method: assign
  //
  bool8 assign(const FrontEndBase& arg) {
    return Error::handle(CLASS_NAME, L"assign",
			 Error::VIRTUAL_PTR, __FILE__, __LINE__);
  }

  // method: eq
  //
  bool8 eq(const FrontEndBase& arg) const {
    return Error::handle(CLASS_NAME, L"eq",
			 Error::VIRTUAL_PTR, __FILE__, __LINE__);
  }

  // method: className
  //
  inline const String& className() const {
    Error::handle(CLASS_NAME, L"className",
		  Error::VIRTUAL_PTR, __FILE__, __LINE__);
    return String::getEmptyString();
  }

  // method: getVector
  //
  inline bool8 getVector(VectorFloat& coeffs, int32 channel,
			   int32 frame_index) {
    return Error::handle(CLASS_NAME, L"getVector", Error::VIRTUAL_PTR,
			 __FILE__, __LINE__);
  }
  
  // method: open
  //
  inline bool8 open(const Filename& input) {
    return Error::handle(CLASS_NAME, L"open",
			 Error::VIRTUAL_PTR, __FILE__, __LINE__);
  }

  // method: close
  //
  inline bool8 close() {
    return Error::handle(CLASS_NAME, L"close",
			 Error::VIRTUAL_PTR, __FILE__, __LINE__);
  }

  // method: init
  //
 inline bool8 init(float32 start_time, float32 stop_time, int channel) {
    return Error::handle(CLASS_NAME, L"init",
			 Error::VIRTUAL_PTR, __FILE__, __LINE__);
  }
  
  // method: run
  //
  inline bool8 run(const Filename& input) {
    return Error::handle(CLASS_NAME, L"run",
			 Error::VIRTUAL_PTR, __FILE__, __LINE__);
  }

  //---------------------------------------------------------------------------
  //
  // class-specific public methods:
  //  other related methods
  //
  //---------------------------------------------------------------------------

  // method: readComponents
  //
  inline bool8 readComponents(Sdb& all_recipes) {
    return Error::handle(CLASS_NAME, L"readComponents",
			 Error::VIRTUAL_PTR, __FILE__, __LINE__);
  }

  //---------------------------------------------------------------------------
  //
  // private methods
  //
  //---------------------------------------------------------------------------
private:
};

// end of include file
// 
#endif
