// file: $isip/class/sp/FrontEnd/fe_04.cc
// version: $Id: fe_04.cc 8371 2002-07-14 15:39:01Z picone $
//

// isip include files
//
#include "FrontEnd.h"

// method: write
//
// arguments:
//  Sof& sof: (input) sof file object
//  int32 tag: (input) sof object instance tag
//  const String& name: (input) sof object instance name
//
// return: a bool8 value indicating status
//
// this method has the object write itself to an Sof file
//
bool8 FrontEnd::write(Sof& sof_a, int32 tag_a, const String& name_a) const {

  // declare local variables
  //
  int32 obj_size = 0;
  
  // write the instance of the object into the Sof file
  //
  if (sof_a.isText()) {

    // set the size to by dynamic
    //
    obj_size = Sof::ANY_SIZE;
  }
  else {

    // the size index and the size of each element
    //
    obj_size = sofSize();
  }

  // write the object into the sof file's index
  //
  if (!sof_a.put(name_a, tag_a, obj_size)) {
    return false;
  }
  
  // exit gracefully
  //
  return writeData(sof_a);
}

// method: writeData
//
// arguments:
//  Sof& sof: (input) sof file object
//  const String& pname: (input) parameter name
//
// return: a bool8 value indicating status
//
// this method has the object writes itself from an Sof file. it assumes
// that the Sof file is already positioned correctly.
//
bool8 FrontEnd::writeData(Sof& sof_a, const String& pname_a) const {

  // write a start string if necessary
  //
  sof_a.writeLabelPrefix(pname_a);

  // write the number of coefficients
  //
  TYPES type = getType();
  TYPES_MAP.writeElementData(sof_a, PARAM_TYPES, (ushort)type);

  // write the data
  //
  if (!virtual_fe_d->writeData(sof_a, String::getEmptyString())) {
    return false;
  }

  // put an end string if necessary
  //
  sof_a.writeLabelSuffix(pname_a);
  
  // exit gracefully
  //
  return true;
}

// method: sofSize
//
// arguments: none 
//
// return: size of the object
//
// this method returns the size of the object
//
int32 FrontEnd::sofSize() const {

  // start with enough space for the type of algorithm
  //
  int32 bytes = TYPES_MAP.elementSofSize();

  // add the size of the algorithm itself
  //
  bytes += virtual_fe_d->sofSize();

  // return the size
  //
  return bytes;
}
