// file: $isip/class/sp/FtrBuffer/ftrb_02.cc
// version: $Id:
//

// isip include files
//
#include "FtrBuffer.h"
#include <Console.h>

// method: diagnose
//
// arguments:
//  Integral::DEBUG level: (input) debug level for diagnostics
//
// return: a bool8 value indicating status
//
bool8 FtrBuffer::diagnose(Integral::DEBUG level_a) {

  //---------------------------------------------------------------------
  //
  // 0. preliminaries
  //
  //---------------------------------------------------------------------

  // output the class name
  //
  if (level_a > Integral::NONE) {
    String output(L"diagnosing class ");
    output.concat(CLASS_NAME);
    output.concat(L": ");
    Console::put(output);
    Console::increaseIndention();
  } 

  //--------------------------------------------------------------------
  //
  // 1. required public methods
  //
  //--------------------------------------------------------------------

  // set indentation
  //
  if (level_a > Integral::NONE) {
    Console::put(L"testing required public methods...\n");
    Console::increaseIndention();
  }

  // test destructor/constructor(s) and memory management
  //
  FtrBuffer ftrb0;
  FtrBuffer ftrb1(ftrb0);
  FtrBuffer ftrb2;
  
  if (!ftrb1.eq(ftrb0)) {
    return Error::handle(name(), L"copy constructor", Error::TEST,
			 __FILE__, __LINE__);
  }
  
  if (!ftrb2.eq(ftrb0)) {
    return Error::handle(name(), L"copy constructor", Error::TEST,
			 __FILE__, __LINE__);
  }

  // test large allocation construction and deletion
  //
  if (level_a == Integral::ALL) {
    
    Console::put(L"\ntesting large chunk memory allocation and deletion:\n");
    
    // set the memory to a strange block size so we can hopefully catch any
    // frame overrun errors
    //
    FtrBuffer::setGrowSize((int32)500);
    
    FtrBuffer* pft = new FtrBuffer();

    for (int32 j = 1; j <= 100; j++) {
      FtrBuffer** pfts = new FtrBuffer*[j * 100];
      
      // create the objects
      //
      for (int32 i = 0; i < j * 100; i++) {
	pfts[i] = new FtrBuffer();
      }
      
      // delete objects
      //
      for (int32 i = (j * 100) - 1; i >= 0; i--) {
	delete pfts[i];
      }
      
      delete [] pfts;
    }
    
    delete pft;
  }

  // test the i/o methods
  //
  FtrBuffer ftrb3;
  FtrBuffer ftrb4;
  FtrBuffer ftrb5;

  // we need binary and text sof files
  //
  String tmp_filename0;
  Integral::makeTemp(tmp_filename0);
  String tmp_filename1;
  Integral::makeTemp(tmp_filename1);

  // open files in write mode
  //
  Sof tmp_file0;
  tmp_file0.open(tmp_filename0, File::WRITE_ONLY, File::TEXT);
  Sof tmp_file1;
  tmp_file1.open(tmp_filename1, File::WRITE_ONLY, File::BINARY);

  ftrb3.write(tmp_file0, (int32)0);
  ftrb3.write(tmp_file1, (int32)0);

  // close the files
  //
  tmp_file0.close();
  tmp_file1.close();

  // open the files in read mode
  //
  tmp_file0.open(tmp_filename0);
  tmp_file1.open(tmp_filename1);

  // read the value back
  //
  ftrb4.read(tmp_file0, (int32)0);

  if (!ftrb4.eq(ftrb3)) {
    return Error::handle(name(), L"read", Error::TEST, __FILE__, __LINE__);
  }

  // close and delete the temporary files
  //
  tmp_file0.close();
  tmp_file1.close();

  File::remove(tmp_filename0);
  File::remove(tmp_filename1);

  // reset indentation
  //
  if (level_a > Integral::NONE) {
    Console::decreaseIndention();
  }
  
  //--------------------------------------------------------------------------
  //
  // 2. class-specific public methods:
  //     set and get methods
  //
  //--------------------------------------------------------------------------

  // set indentation
  //
  if (level_a > Integral::NONE) {
    Console::put(L"testing class-specific public methods: set/get methods...\n");
    Console::increaseIndention();
  }

  // declare local variables
  //
  FtrBuffer fb;

  // set and get the frame index
  //
  int32 test_index = 5;
  fb.setFrameIndex(test_index);
  if (fb.getFrameIndex() != test_index) {
      return Error::handle(name(), L"setFrameIndex", Error::TEST,
			   __FILE__, __LINE__);
  }

  // set and get the last index
  //
  test_index = 10;
  fb.setLastIndex(test_index);
  if (fb.getLastIndex() != test_index) {
      return Error::handle(name(), L"setLastIndex", Error::TEST,
			   __FILE__, __LINE__);
  }

  // set and get the left over samples
  //
  test_index = 15;
  fb.setLeftoverSamps(test_index);
  if (fb.getLeftoverSamps() != test_index) {
      return Error::handle(name(), L"setLeftoverSamps", Error::TEST,
			   __FILE__, __LINE__);
  }

  // reset indentation
  //
  if (level_a > Integral::NONE) {
    Console::decreaseIndention();
  }
  
  // --------------------------------------------------------------------
  //
  // 3. print completion message
  //
  // --------------------------------------------------------------------

  // reset indentation
  //
  if (level_a > Integral::NONE) {
    Console::decreaseIndention();
  }
  
  if (level_a > Integral::NONE) {
    String output(L"diagnostics passed for class ");
    output.concat(name());
    output.concat(L"\n");
    Console::put(output);
  }
  
  // exit gracefully
  //
  return true;
}
