// file: $isip/class/sp/FtrBuffer/ftrb_03.cc
// version: $Id: ftrb_03.cc 8359 2002-07-13 02:06:30Z picone $
//

// isip include files
//
#include "FtrBuffer.h"

// method: read
//
// arguments:
//  Sof& sof: (input) sof file object
//  int32 tag: (input) sof object instance tag
//  const String& name: (input) sof object instance name
//
// return: a bool8 value indicating status
//
// this method has the object read itself from an Sof file according
// to the specified name and tag
//
bool8 FtrBuffer::read(Sof& sof_a, int32 tag_a, const String& name_a) {

  // read the instance of the object from the Sof file
  //
  if (!sof_a.find(name_a, tag_a)) {
    return false;
  }
  
  // read the actual data from the sof file
  //
  return readData(sof_a);
}

// method: readData
//
// arguments:
//  Sof& sof: (input) sof file object
//  const String& pname: (input) parameter name
//  int32 size: (input) size in bytes of object (or FULL_OBJECT)
//  bool8 param: (input) is the parameter name in the file?
//  bool8 nested: (input) are we nested?
//
// return: a bool8 value indicating status
//
// this method has the object read itself from an Sof file. it assumes
// that the Sof file is already positioned correctly
//
bool8 FtrBuffer::readData(Sof& sof_a, const String& pname_a, int32 size_a,
			   bool8 param_a, bool8 nested_a) {

  // declare local variable
  //
  SofParser parser;

  // set the debug level of the parser
  //
  parser.setDebug(debug_level_d);
  
  // are we nested?
  //
  if (nested_a) {
    parser.setNest();
  }
    
  // load the parse
  //
  if (!parser.load(sof_a, size_a)) {

    // return a warning message
    //
    return Error::handle(name(), L"readData", Error::READ,
			 __FILE__, __LINE__, Error::WARNING);
  }          

  // get the hash table
  //
  if (parser.isPresent(sof_a, PARAM_HASH)) {
    if (!hash_d.readData(sof_a, PARAM_HASH,
			 parser.getEntry(sof_a, PARAM_HASH), false, true)) {

      // return a warning message
      //
      return Error::handle(name(), L"readData", Error::READ,
			   __FILE__, __LINE__, Error::WARNING);
    }      
  }
  else {
    hash_d.clear();
  }

  // get the length
  //
  if (parser.isPresent(sof_a, PARAM_LENGTH)) {
    if (!length_d.readData(sof_a, PARAM_LENGTH,
			   parser.getEntry(sof_a, PARAM_LENGTH),
			   false, true)) {

      // return a warning message
      //
      return Error::handle(name(), L"readData", Error::READ,
			   __FILE__, __LINE__, Error::WARNING);
    }      
  }
  else {
    length_d.clear();
  }

  // get the frame index
  //
  if (parser.isPresent(sof_a, PARAM_FRAME_INDEX)) {
    if (!frame_index_d.readData(sof_a, PARAM_FRAME_INDEX,
				parser.getEntry(sof_a,
						   PARAM_FRAME_INDEX))) {

      // return a warning message
      //
      return Error::handle(name(), L"readData", Error::READ,
			   __FILE__, __LINE__, Error::WARNING);
    }      
  }
  else {
    frame_index_d = 0;
  }

  // get the last index
  //
  if (parser.isPresent(sof_a, PARAM_LAST_INDEX)) {
    if (!last_index_d.readData(sof_a, PARAM_LAST_INDEX,
			       parser.getEntry(sof_a, PARAM_LAST_INDEX))) {

      // return a warning message
      //
      return Error::handle(name(), L"readData", Error::READ,
			   __FILE__, __LINE__, Error::WARNING);
    }      
  }
  else {
    last_index_d = 0;
  }

  // get the coefficient name
  //
  if (parser.isPresent(sof_a, PARAM_COEF_NAME)) {
    if (!coef_name_d.readData(sof_a, PARAM_COEF_NAME,
			      parser.getEntry(sof_a, PARAM_COEF_NAME))) {

      // return a warning message
      //
      return Error::handle(name(), L"readData", Error::READ,
			   __FILE__, __LINE__, Error::WARNING);
    }      
  }
  else {
    coef_name_d.clear();
  }

  // get the the number of samples in the last frame
  //
  if (parser.isPresent(sof_a, PARAM_LEFTOVER)) {
    if (!leftover_samps_d.readData(sof_a, PARAM_LEFTOVER,
				   parser.getEntry(sof_a,
						      PARAM_LEFTOVER))) {

      // return a warning message
      //
      return Error::handle(name(), L"readData", Error::READ,
			   __FILE__, __LINE__, Error::WARNING);
    }      
  }
  else {
    leftover_samps_d = 0;
  }

  // check that all parameters are accounted for
  //
  if (!parser.checkParams(sof_a)) {

    // return a warning message
    //
    return Error::handle(name(), L"readData", Error::IO,
			 __FILE__, __LINE__,
                         Error::WARNING);
  }

  // exit gracefully
  //
  return true;
}
