// file: $isip/class/sp/FtrBuffer/ftrb_04.cc
// version: $Id:
//

// isip include files
//
#include "FtrBuffer.h"

// method: sofSize
//
// arguments: none 
//
// return: size of object
//
// this method returns the size of the object in the Sof file and is
// used for binary write
//
int32 FtrBuffer::sofSize() const {

  // start with the space required for the hash table
  //
  int32 bytes = hash_d.sofSize();

  // add the space required for the length
  //
  bytes += length_d.sofSize();

  // add the space required for the frame index
  //
  bytes += frame_index_d.sofSize();

  // add the space required for the last index
  //
  bytes += last_index_d.sofSize();

  // add the space required for the coefficient name
  //
  bytes += coef_name_d.sofSize();

  // add the space required for the number of samples in the last frame
  //
  bytes += leftover_samps_d.sofSize();

  // return the size
  //
  return bytes;
}

// method: write
//
// arguments:
//  Sof& sof: (input) sof file object
//  int32 tag: (input) sof object instance tag
//  const String& name: (input) sof object instance name
//
// return: a bool8 value indicating status
//
// this method has the object write itself to an Sof file
//
bool8 FtrBuffer::write(Sof& sof_a, int32 tag_a, const String& name_a) const {

  // declare local variable
  //
  int32 obj_size;
  
  // write the instance of the object into the Sof file
  //
  if (sof_a.isText()) {

    // set the size to be dynamic
    //
    obj_size = Sof::ANY_SIZE;
  }
  else {

    // set the size to be the size of the object written to the Sof file
    //
    obj_size = sofSize();
  }

  // write the object into the sof file's index
  //
  if (!sof_a.put(name_a, tag_a, obj_size)) {
    return false;
  }
  
  // write data and exit gracefully
  //
  return writeData(sof_a);
}

// method: writeData
//
// arguments:
//  Sof& sof: (input) sof file object
//  const String& pname: (input) parameter name
//
// return: a bool8 value indicating status
//
// this method has the object writes itself to an Sof file. it assumes
// that the Sof file is already positioned correctly
//
bool8 FtrBuffer::writeData(Sof& sof_a, const String& pname_a) const {
  
  // write a start string if necessary
  //
  sof_a.writeLabelPrefix(pname_a);

  // write the hash table
  //
  hash_d.writeData(sof_a, PARAM_HASH);
    
  // write the length
  //
  length_d.writeData(sof_a, PARAM_LENGTH);
    
  // write the frame index
  //
  frame_index_d.writeData(sof_a, PARAM_FRAME_INDEX);
    
  // write the last index
  //
  last_index_d.writeData(sof_a, PARAM_LAST_INDEX);
    
  // write the coefficient name
  //
  coef_name_d.writeData(sof_a, PARAM_COEF_NAME);
    
  // write the number of samples in the last frame
  //
  leftover_samps_d.writeData(sof_a, PARAM_LEFTOVER);
    
  // put an end string if necessary
  //
  sof_a.writeLabelSuffix(pname_a);
  
  // exit gracefully
  //
  return true;
}
