// file: $isip_ifc/class/sp/SignalDetector/sigd_08.cc
// version: $Id: sigd_08.cc 10210 2005-08-12 20:24:51Z stanley $
//

// isip include files
//

#include "SignalDetector.h"

// method: computeEnergy
//
// arguments:
//  VectorFloat x_pre_off: (input) signal buffer
//  int32 channel_index: (input) channel index
//
// return: a float64 containing the energy value
//
// note that this method debiases the data
//
float32 SignalDetector::computeEnergy(VectorFloat x_pre_off_a,
				    int32 channel_index_a) {
  
  
  float32 avg = 0, sum = 0;
  
  // xn_d has the value of the sample before the current sample. 
  // In the older code we had a global variable to hold this value so that
  // when we have the next fram we still remember the last sample from the
  // previous frame.
  //
  float32 xn, xn_d = 0;
  for (int32 i = 0; i < frame_sample_duration_d; i++) {
    xn = ((float32)x_pre_off_a(i))* DEF_SAMPLE_SCALE_FACTOR;
    x_pre_off_a(i) = (Float)(xn - (preemphasis_d * xn_d));
    xn_d = xn;
  }
  
  for (int32 i = (window_sample_duration_d - 1); i>(window_sample_duration_d - frame_sample_duration_d - 1); i--) {
    egy_data_window_d(channel_index_a)(i) = x_pre_off_a(i-(window_sample_duration_d - frame_sample_duration_d));
  }
  
  // compute the average and the sum
  //
  for (int32 i = 0; i < window_sample_duration_d; i++) {
    avg += egy_data_window_d(channel_index_a)(i);
    sum += egy_data_window_d(channel_index_a)(i) * egy_data_window_d(channel_index_a)(i);
  }
  
  // calculate the energy
  //
  float32 win_dur_scale_factor_d = 1.0/ (float32) window_sample_duration_d;
  float32 egy = win_dur_scale_factor_d * (sum - win_dur_scale_factor_d*avg*avg);
  
  // Shift the signal
  //
  VectorFloat temp_d(window_sample_duration_d);
  for(int32 i=0;i<(window_sample_duration_d - frame_sample_duration_d);i++) {
    
    temp_d(i) = egy_data_window_d (channel_index_a)(window_sample_duration_d - (window_sample_duration_d - frame_sample_duration_d) + i);
  }
  egy_data_window_d(channel_index_a).assign(temp_d);
  
  if (egy < Integral::DB_POW_MIN_VALUE) {
    egy = Integral::DB_POW_MIN_VALUE;
  }
  return (float32)(Integral::DB_POW_SCALE_FACTOR * log10(egy));
}


// method: computeZeroCrossingRate
//
// arguments:
//  VectorFloat x_pre_off: (input) signal buffer
//  int32 channel_index: (input) channel index
// return: a int32 value containing the zerocrossing rate value
//
// note that this method debiases the data
//

int32 SignalDetector::computeZeroCrossingRate(VectorFloat x_pre_off_a, int32 channel_index_a) {
  
  float32 xn, xn_d = 0;
  int32 num_crossings = 0;
  for (int32 i = 0; i < frame_sample_duration_d; i++) {
    
    xn = ((float32)x_pre_off_a(i))* DEF_SAMPLE_SCALE_FACTOR;
    x_pre_off_a(i) = (Float)(xn - (preemphasis_d * xn_d));
    xn_d = xn;
  }
  
  
  for (int32 i = (window_sample_duration_d - 1); i > (window_sample_duration_d - frame_sample_duration_d - 1); i--) {
    
    zc_data_window_d(channel_index_a)(i) = x_pre_off_a(i - (window_sample_duration_d - frame_sample_duration_d));
    
  }
  
  // compute the number of crossings
  //
  for (int32 i = 0; i < window_sample_duration_d; i++) {
    
    if ((zc_data_window_d(channel_index_a)((window_sample_duration_d - 1) - i) > (Float)((int32)zc_positive_thresh_d*DEF_SAMPLE_SCALE_FACTOR)) &&
	((zc_data_window_d(channel_index_a)(((window_sample_duration_d-1) -i) - 1))  < (Float)((int32)zc_negative_thresh_d*DEF_SAMPLE_SCALE_FACTOR))) 
      num_crossings++; 
    if ((zc_data_window_d(channel_index_a)((window_sample_duration_d -1)- i) < (Float)((int32)zc_negative_thresh_d*DEF_SAMPLE_SCALE_FACTOR)) &&
	((zc_data_window_d(channel_index_a)(((window_sample_duration_d - 1) -i) - 1)) > (Float)((int32)zc_positive_thresh_d*DEF_SAMPLE_SCALE_FACTOR)))
      num_crossings++;
  }
  
  
  // shift the signal
  //
  VectorFloat temp_d(window_sample_duration_d);
  for(int32 i=0;i<(window_sample_duration_d - frame_sample_duration_d);i++) {
    
    temp_d(i) = zc_data_window_d (channel_index_a)(window_sample_duration_d - (window_sample_duration_d - frame_sample_duration_d) + i);
  }
  zc_data_window_d(channel_index_a).assign(temp_d);
  
  return (num_crossings);
  
}
