// file: $isip_ifc/class/stat/Histogram/hist_04.cc
// version: $Id: hist_04.cc 10393 2006-01-25 05:20:55Z srinivas $
//

// isip include files
//
#include "Histogram.h"

// method: write
//
// arguments:
//  Sof& sof: (input) sof file object
//  int32 tag: (input) sof object instance tag
//  const String& name: (input) sof object instance name
//
// return: bool8 value indicating status
//
// this method has the object write itself to an Sof file
//
bool8 Histogram::write(Sof& sof_a, int32 tag_a,
			 const String& name_a) const {

  int32 obj_size = 0;
  
  // write the instance of the object into the Sof file
  //
  if (sof_a.isText()) {

    // set the size to by dynamic
    //
    obj_size = Sof::ANY_SIZE;
  }
  else {

    // the size index and the size of each element
    //
    obj_size = sofSize();
  }

  // write the object into the sof file's index
  //
  if (!sof_a.put(name_a, tag_a, obj_size)) {
    return false;
  }
  
  // exit gracefully
  //
  return writeData(sof_a);
}

// method: writeData
//
// arguments:
//  Sof& sof: (input) sof file object
//  const String& pname: (input) parameter name
//
// return: bool8 value indicating status
//
// this method has the object write itself to an Sof file. it assumes
// that the Sof file is already positioned correctly.
//
bool8 Histogram::writeData(Sof& sof_a, const String& pname_a) const {
  
  // write a start string if necessary
  //
  sof_a.writeLabelPrefix(pname_a);

  // write the scale
  //
  SCALE_MAP.writeElementData(sof_a, PARAM_SCALE, (int32)scale_d);

  // write the bin mode
  //
  MODE_MAP.writeElementData(sof_a, PARAM_MODE, (int32)mode_d);

  // write the dimension
  //
  dim_d.writeData(sof_a, PARAM_DIM);
  
  // write the vector of bins for each dimension
  //
  bins_d.writeData(sof_a, PARAM_BINS);

  // write the vector of minimums
  //
  min_d.writeData(sof_a, PARAM_MIN);

  // write the vector of maximums
  //
  max_d.writeData(sof_a, PARAM_MAX);

  // write the vector of number of bins
  //
  num_bins_d.writeData(sof_a, PARAM_NUM_BINS);
  
  // write the counts of all multi-dimensional bins
  //
  counts_d.writeData(sof_a, PARAM_COUNTS);

  // put an end string if necessary
  //
  sof_a.writeLabelSuffix(pname_a);
  
  // exit gracefully
  //
  return true;
}
