// file: $isip/class/stat/NGramModel/ngrm_04.cc
// version: $Id: ngrm_04.cc 8501 2002-07-30 05:03:25Z zheng $
//

// isip include files
//
#include "NGramModel.h"

// method: sofSize
//
// arguments: none
//
// return: a int32 value indicating the size of object
//
// this method return  the size of the object
//
int32 NGramModel::sofSize() const {
  
  // calculate the space of data type and algorithm
  //
  int32 bytes = order_d.sofSize();

  Vector<Long> keys;
  Vector<NGramNode> values;

  if (!gram_hash_d.keys(keys)) {
    return Error::handle(name(), L"sofSize", Error::ARG, __FILE__, __LINE__);
  } 
  bytes += keys.sofSize();
  
  if (!gram_hash_d.values(values)) {
    return Error::handle(name(), L"sofSize", Error::ARG, __FILE__, __LINE__);
  }
  bytes += values.sofSize();
  
  // return the size
  //
  return bytes;
}

// method: write
//
// arguments:
//  Sof& sof: (input) sof file object
//  int32 tag: (input) sof object instance tag
//  const String& name: (input) sof object instance name
//
// return: a bool8 value indicating status
//
// this method has the object write itself, with specified name and
// tag, to an Sof file
//
bool8 NGramModel::write(Sof& sof_a, int32 tag_a,
			  const String& name_a) const {

  // declare local variables
  //  
  int32 obj_size;

  // write the instance of the object into the Sof file
  //  
  if (sof_a.isText()) {
    obj_size = Sof::ANY_SIZE;
  }
  else {
    obj_size = sofSize();
  }
  
  // put the object into the sof file's index
  //
  if (!sof_a.put(name_a, tag_a, obj_size)) {
    return false;
  }

  // exit gracefully
  //
  return writeData(sof_a);
}

// method: writeData
//
// arguments:
//  Sof& sof: (input) sof file object
//  const String& pname: (input) textual parameter name
//
// return: logical error status
//
// this method has the object read itself from an Sof file. it assumes
// that the Sof file is already positioned correctly.
//
bool8 NGramModel::writeData(Sof& sof_a, const String& pname_a) const {

  // write a start string if necessary
  //
  sof_a.writeLabelPrefix(pname_a);
    
  // write the configuration
  //  
  order_d.writeData(sof_a, PARAM_ORDER);

  // get the keys associated with the hash table
  //
  Vector<Long> keys;
  if (!gram_hash_d.keys(keys)) {
    return Error::handle(name(), L"readData", Error::ARG, __FILE__, __LINE__);
  }
  
  // write the keys associated with the hash table
  //
  keys.writeData(sof_a, PARAM_KEYS);
  
  // get the values associated with the hash table
  //
  Vector<NGramNode> values;
  if (!gram_hash_d.values(values)) {
    return Error::handle(name(), L"readData", Error::ARG, __FILE__, __LINE__);
  }
  
  // write the values associated with the hash table
  //
  values.writeData(sof_a, PARAM_VALUES);

  // put an end string if necessary
  //
  sof_a.writeLabelSuffix(pname_a);
  
  // exit gracefully
  //
  return true;
}
