// file: $isip_ifc/class/stat/RelevanceVectorModel/rvm_03.cc
// version: $Id: rvm_03.cc 10423 2006-02-09 00:52:30Z raghavan $
//

// isip include files
//
#include "RelevanceVectorModel.h"

// method: read
//
// arguments:
//  Sof& sof: (input) sof file object
//  int32 tag: (input) sof object instance tag
//  const String& name: (input) sof object instance name
//
// return: a bool8 value indicating status
//
// this method has the object read itself from an Sof file
//
bool8 RelevanceVectorModel::read(Sof& sof_a, int32 tag_a,
				 const String& name_a) {

  // read the instance of the object from the Sof file
  //
  if (!sof_a.find(name_a, tag_a)) {
    return false;
  }
  
  // read the actual data from the sof file
  //
  if (!readData(sof_a)) {
    return false;
  }
  
  // exit gracefully
  //
  return true;  
}

// method: readData
//
// arguments:
//  Sof& sof: (input) sof file object
//  const String& pname: (input) parameter name
//  int32 size: (input) size in bytes of object (or full_size)
//  bool8 param: (input) is the parameter name in the file?
//  bool8 nested: (input) are we nested?
//
// return: a bool8 value indicating status
//
// this method has the object read itself from an Sof file. it assumes
// that the Sof file is already positioned correctly.
//
bool8 RelevanceVectorModel::readData(Sof& sof_a, const String& pname_a,
				int32 size_a, bool8 param_a,
				bool8 nested_a) {

  Integral::DEBUG debug_level = Integral::NONE;

  SofParser parser;

  parser.setDebug(debug_level);
    
  // are we nested?
  //
  if (nested_a) {
    parser.setNest();
  }
  
  // load the parser
  //
  if (!parser.load(sof_a, size_a)) {
    return Error::handle(name(), L"readData", Error::READ,
			 __FILE__, __LINE__, Error::WARNING);
  }
  
  // get the algorithm
  //
  if (parser.isPresent(sof_a, PARAM_ALGORITHM)) {
    if (!ALGO_MAP.readElementData((int32&)algorithm_d, sof_a, PARAM_ALGORITHM,
				  parser.getEntry(sof_a, PARAM_ALGORITHM))) {

      // return a warning message
      //
      return Error::handle(name(), L"readData", Error::READ,
			   __FILE__, __LINE__, Error::WARNING);
    }
  }
  else {
    algorithm_d = DEF_ALGORITHM;
  }
  
  // get the implementation
  //
  if (parser.isPresent(sof_a, PARAM_IMPLEMENTATION)) {
    if (!IMPL_MAP.readElementData((int32&)implementation_d, sof_a,
				  PARAM_IMPLEMENTATION,
				  parser.getEntry(sof_a,
						  PARAM_IMPLEMENTATION))) {

      // return a warning message
      //
      return Error::handle(name(), L"readData", Error::READ,
			   __FILE__, __LINE__, Error::WARNING);
    }
  }
  else {
    implementation_d = DEF_IMPLEMENTATION;
  }
  
  // read the bias
  //
  if (parser.isPresent(sof_a, PARAM_BIAS)) {
      if (!bias_d.readData(sof_a, PARAM_BIAS,
			   parser.getEntry(sof_a, PARAM_BIAS))) {
      
	// return a warning message
	//
	return Error::handle(name(), L"readData", Error::READ,
			     __FILE__, __LINE__, Error::WARNING);
	
      }
  }
  else {
    return Error::handle(name(), L"readData", Error::ARG, __FILE__, __LINE__);
  }
  
  // read the weights
  //
  if (parser.isPresent(sof_a, PARAM_WEIGHTS)) {
      if (!weights_d.readData(sof_a, PARAM_WEIGHTS,
			      parser.getEntry(sof_a, PARAM_WEIGHTS),
			      false, false)) {
      
	// return a warning message
	//
	return Error::handle(name(), L"readData", Error::READ,
			     __FILE__, __LINE__, Error::WARNING);
      }
  }
  else {
    return Error::handle(name(), L"readData", Error::ARG, __FILE__, __LINE__);
  }
  
  // read the hyperparameters
  //
  if (parser.isPresent(sof_a, PARAM_INV_HESSIAN)) {
      if (!inv_hessian_d.readData(sof_a, PARAM_INV_HESSIAN,
			      parser.getEntry(sof_a, PARAM_INV_HESSIAN),
			      true, true)) {
      
	// return a warning message
	//
	return Error::handle(name(), L"readData", Error::READ,
			     __FILE__, __LINE__, Error::WARNING);
      }
  }
  else {
    return Error::handle(name(), L"readData", Error::ARG, __FILE__, __LINE__);
  }
  
  // read the targets
  //
  if (parser.isPresent(sof_a, PARAM_TARGETS)) {
      if (!targets_d.readData(sof_a, PARAM_TARGETS,
			      parser.getEntry(sof_a, PARAM_TARGETS),
			      false, false)) {
      
	// return a warning message
	//
	return Error::handle(name(), L"readData", Error::READ,
			     __FILE__, __LINE__, Error::WARNING);
	
      }
  }
  else {
    return Error::handle(name(), L"readData", Error::ARG, __FILE__, __LINE__);
  }
  
  // read the relevance vectors
  //
  if (parser.isPresent(sof_a, PARAM_VECTORS)) {
      if (!vectors_d.readData(sof_a, PARAM_VECTORS,
			      parser.getEntry(sof_a, PARAM_VECTORS),
			      false, false)) {
      
	// return a warning message
	//
	return Error::handle(name(), L"readData", Error::READ,
			     __FILE__, __LINE__, Error::WARNING);
	
      }
  }
  else {
    return Error::handle(name(), L"readData", Error::ARG, __FILE__, __LINE__);
  }
  
  // get the kernels
  //
  if (!kernels_d.readData(sof_a, PARAM_KERNELS,
			  parser.getEntry(sof_a, PARAM_KERNELS),
			  false, false)) {
    return Error::handle(name(), L"readData", Error::READ,
			 __FILE__, __LINE__, Error::WARNING);
  }
  
  // call initialization method and exit gracefully
  //
  return init();
}
