// file: $isip_ifc/class/stat/RelevanceVectorModel/rvm_04.cc
// version: $Id: rvm_04.cc 10423 2006-02-09 00:52:30Z raghavan $
//

// isip include files
//
#include "RelevanceVectorModel.h"

// method: sofSize
//
// arguments: none
//
// return: size of object
//
// this method returns the size of the object in the Sof file and is
// used for binary write
//
int32 RelevanceVectorModel::sofSize() const {

  // start with the space required for the algorithm name
  //
  int32 bytes = ALGO_MAP.elementSofSize();
  bytes += IMPL_MAP.elementSofSize();
    
  // add the space required for data memebers
  //
  bytes +=  weights_d.sofSize() + targets_d.sofSize() +
    vectors_d.sofSize() + bias_d.sofSize() + kernels_d.sofSize() +
    inv_hessian_d.sofSize();

  // return the size
  //
  return bytes;
}

// method: write
//
// arguments:
//  Sof& sof: (input) sof file object
//  int32 tag: (input) sof object instance tag
//  const String& name: (input) sof object instance name
//
// return: bool8 value indicating status
//
// this method has the object write itself to an Sof file
//
bool8 RelevanceVectorModel::write(Sof& sof_a, int32 tag_a,
			     const String& name_a) const {

  int32 obj_size = 0;
  
  // write the instance of the object into the Sof file
  //
  if (sof_a.isText()) {

    // set the size to by dynamic
    //
    obj_size = Sof::ANY_SIZE;
  }
  else {

    // the size index and the size of each element
    //
    obj_size = sofSize();
  }

  // write the object into the sof file's index
  //
  if (!sof_a.put(name_a, tag_a, obj_size)) {
    return false;
  }
  
  // exit gracefully
  //
  return writeData(sof_a);
}

// method: writeData
//
// arguments:
//  Sof& sof: (input) sof file object
//  const String& pname: (input) parameter name
//
// return: bool8 value indicating status
//
// this method has the object write itself to an Sof file. it assumes
// that the Sof file is already positioned correctly.
//
bool8 RelevanceVectorModel::writeData(Sof& sof_a,
				      const String& pname_a) const {
  
  // write a start string if necessary
  //
  sof_a.writeLabelPrefix(pname_a);

  // write the algorithm and implementation name
  //
  ALGO_MAP.writeElementData(sof_a, PARAM_ALGORITHM, (int32)algorithm_d);
  IMPL_MAP.writeElementData(sof_a,
			    PARAM_IMPLEMENTATION, (int32)implementation_d);

  // write the bias
  //
  bias_d.writeData(sof_a, PARAM_BIAS);
  
  // write the weights
  //
  weights_d.writeData(sof_a, PARAM_WEIGHTS);
  
  // write the hyperparameters
  //
  inv_hessian_d.writeData(sof_a, PARAM_INV_HESSIAN);
  
  // write the targets
  //
  targets_d.writeData(sof_a, PARAM_TARGETS);
  
  // write the relevance vectors
  //
  vectors_d.writeData(sof_a, PARAM_VECTORS);
  
  // write the kernel
  //
  kernels_d.writeData(sof_a, PARAM_KERNELS);

  // put an end string if necessary
  //
  sof_a.writeLabelSuffix(pname_a);
  
  // exit gracefully
  //
  return true;
}
