// file: $isip/class/stat/UniformModel/uni_03.cc
// version: $Id: uni_03.cc 6073 2000-12-23 00:31:01Z picone $
//

// isip include files
//
#include "UniformModel.h"

// method: read
//
// arguments:
//  Sof& sof: (input) sof file object
//  int32 tag: (input) sof object instance tag
//  const String& name: (input) sof object instance name
//
// return: logical error status
//
// this method has the object read itself from an Sof file
//
bool8 UniformModel::read(Sof& sof_a, int32 tag_a, const String& name_a) {

  // read the instance of the object from the Sof file
  //
  if (!sof_a.find(name_a, tag_a)) {
    return false;
  }
  
  // read the actual data from the sof file
  //
  if (!readData(sof_a)) {
    return false;
  }
  
  // exit gracefully
  //
  return true;  
}

// method: readData
//
// arguments:
//  Sof& sof: (input) sof file object
//  const String& pname: (input) parameter name
//  int32 size: (input) size in bytes of object (or full_size)
//  bool8 param: (input) is the parameter name in the file?
//  bool8 nested: (input) are we nested?
//
// return: logical error status
//
// this method has the object read itself from an Sof file. it assumes
// that the Sof file is already positioned correctly.
//
bool8 UniformModel::readData(Sof& sof_a, const String& pname_a,
			       int32 size_a, bool8 param_a,
			       bool8 nested_a) {

  // build a parser
  //
  Integral::DEBUG debug_level = Integral::NONE;
  SofParser parser;
  parser.setDebug(debug_level);
    
  // ignore implicit parameter setting
  //
  
  // are we nested?
  //
  if (nested_a) {
    parser.setNest();
  }
  
  // load the parse
  //
  if (!parser.load(sof_a, size_a)) {
    return Error::handle(name(), L"readData", Error::READ,
			 __FILE__, __LINE__, Error::WARNING);
  }    

  // get the vector of mins
  //
  if (!min_d.readData(sof_a, PARAM_MIN,
		      parser.getEntry(sof_a, PARAM_MIN), false, false)) {
    return Error::handle(name(), L"readData", Error::READ,
			 __FILE__, __LINE__, Error::WARNING);
  }    
  
  // get the vector of maxs
  //
  if (!max_d.readData(sof_a, PARAM_MAX,
		      parser.getEntry(sof_a, PARAM_MAX), false, false)) {
    return Error::handle(name(), L"readData", Error::READ,
			 __FILE__, __LINE__, Error::WARNING);
  }    
  
  // set the initialization flag
  //
  is_valid_d = false;

  // exit gracefully
  //
  return true;
}
