// file:$ISIP_IFC/util/demo/isip_comm_client_dialog_navigation/DialogNavigationServer.java
// version: $Id: DialogNavigationServer.java 10572 2006-06-26 21:54:34Z rei5 $

// import necessary java libraries
//
import java.net.*;
import java.io.*;
import javax.swing.text.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.util.*;
import javax.swing.border.*;
import javax.sound.sampled.*;
import java.text.DecimalFormat;
import javax.swing.JOptionPane;
import javax.swing.JFrame;


// import galaxy libraries
//
import galaxy.lang.*;
import galaxy.server.*;
import galaxy.server.ui.ServerUI;

/**
 * This class is responsible for the user interface of the
 * DialogNavigation Demo.
 */
public class DialogNavigationServer extends galaxy.server.Server  {
    
    //-------------------------------------------------------------------------
    //
    // public constants
    //
    //-------------------------------------------------------------------------
    
    /**
     *  audio recorder server volume index
     */
    public static final int AudioRecorderServer_VOLUME_INDEX = 0;
    
    /**
     *  audio recorder server energy index
     */
    public static final int AudioRecorderServer_ENERGY_INDEX = 1;
    
    /**
     * Clause name for Galaxy frame
     */
    public static String MAIN = "main"; 
    
    /**
     * The variable that keeps the count of data frames that were sent.
     */
    public static int sent_frame_number_d = 0 ;
    
    /**
     * The flag for acknowledgement for audio ready signal.
     */
    public static int audio_ready_ack_d = 0;
    
    /**
     *  audio recorder UI frame
     */
    public static DialogNavigationUIFrame speechFrame_d;
    
    /**
     * Finite state for this server -- INITIALIZATION. 
     */
    public static final int INITIALIZATION = 1000;
    
    /**
     * Finite state for this server -- AUDIO_READY. 
     */
    public static final int AUDIO_READY = 2000;
    
    /**
     * Finite state for this server -- AUDIO_READY_ACK. 
     */
    public static final int AUDIO_READY_ACK = 3000;
    
    /**
     * Finite state for this server -- RAW_DATA_TRANSFER. 
     */
    public static final int RAW_DATA_TRANSFER = 4000;
    
    /**
     * Finite state for this server -- END_OF_UTTERANCE. 
     */
    public static final int END_OF_UTTERANCE = 5000;
    
    /**
     * Action state for this server -- IDLE.
     */
    public static final int IDLE = 100;
    
    /**
     * Action state for this server -- PLAYBACK.
     */
    public static final int PLAYBACK = 200;
    
    /**
     * Mode for playback.
     * PLAYBACK_SIGNAL_ONLY only the endpointed signal is played back
     */
    public static final int PLAYBACK_SIGNAL_ONLY = 250;
    
    /**
     * Mode for playback.
     * PLAYBACK_ALL playback the whole audio data
     */
    public static final int PLAYBACK_ALL = 230;
       
    /**
     * Action state for this server -- RECORDING.
     */
    public static final int RECORDING = 300;
    
    /**
     * The default audio format.
     */
    public static final AudioFormat SPEECH = new AudioFormat(
							     AudioFormat.Encoding.PCM_SIGNED, 16000.0F, 
							     16, 1, 2, 16000.0F, 
							     false);

    /**
     * Keeps track of the number of audio_ready acks got.
     */
    public static int audio_ready_ack_count = 0;
    
    /**
     * Keeps track of the number of end_of_utterance acks got.
     */
    public static int utterance_end_ack_count = 0;

    /**
     * The flag used to find whether the end points were detected or not.
     */
    public static int end_points = 0;

    /**
     * A float array that holds the audio data.
     */
    public float[][] audio_data; 
    
    /**
     * A JFrame object for the error window which pops up whenever the
     * audio format is not supported .
     */
    public JFrame frame;
    
    /**
     * this object is used to pop up a window which gets the database parameters
     * from the user
     */
    public DatabasePanel database_parameters;
    
    /**
     * the configdata object having audio parameters
     */
    ConfigData audio;
    
    /**
     * the configdata object having signal detector parameters
     */
    ConfigData detector;

    //-----------------------------------------------------------------------
    //
    // protected data
    //
    //-----------------------------------------------------------------------
    
    
    // button counter
    //
    protected int btn_press_counter_d = 0; 
     
    // data frame number index.
    //
    protected int data_frame_number_d = 0;
    
    // the state of the dialog navigation
    //
    protected int finite_state_d;
    
    // the action state of the dialog navigation(recording or playback or idle)
    //
    protected int action_state_d;

    // the flag that gets set from the user presses the stop button
    //
    protected int stop_d; 
    
    // the audiointerface object
    //
    protected AudioInterface audioInterface ;
    
    // the utterance id
    //
    protected static int utterance_id_d = 0;
    
    // flag to check whether an indentation has been set or not before
    // printing to the text field
    //
    protected boolean indent_d =  false;
    
    // recording string
    //
    protected static String recording = "Recording ...";
  
    // an audioformat object
    //
    protected AudioFormat format_d;
    
    // an array of float values containing the audio samples
    //
    protected float[] samples_d;
    
    // the channel to be displayed
    //
    protected int channel_display;
    
    // playback mode:
    // mode 1: signal_only
    // mode 2: all (default)
    protected int playback_mode_d;

    // the flag that indicates whether recording has taken place
    // ot not
    //
    protected boolean record_event;

    // the buffer size to be transferred from the AudioInterface
    //
    protected int buffer_size;
    
    // the string that holds the unicode to print the line in input
    // window
    //
    protected String unicode_line;

    // the number of unicode blocks needed to print the line
    //
    protected int unicode_count = 79;

    // the start time
    //
    protected Float start_index;
    
    // the stop time
    //
    protected Float stop_index;

    //-------------------------------------------------------------------------
    //
    // class_specific public methods
    //
    //-------------------------------------------------------------------------
    
    public DialogNavigationServer(DialogNavigationMainServer mainServer, 
				Socket socket) throws IOException {
	
	// call the constructor for super class
	//
	super(mainServer, socket);
	speechFrame_d = mainServer.getDialogNavigationUIFrame();

	// error checking for null pointers
	//
	if (speechFrame_d == null) {
	    System.out.println("speechFrame_d is null! Exiting!\n");
	    return;
	}

	if (speechFrame_d.speech_plot_d == null) {
	    System.out.println("speechFrame_d.speech_plot_d is null! " +
			       "Exiting!\n");
	    return;
	}

	// initialize the graphic display
	//
	float percentage_1 = (float) 0.5;
	float percentage_2 = (float) 0.5;
	float percentage_3 = (float) 0;
		
	// set the sample rate
	//
	speechFrame_d.speech_plot_d.setSampleRate(
                      (int)(SPEECH.getSampleRate()));

	// set the space the energy, waveform and spectrogram monitor
	// occupy
	//
	speechFrame_d.speech_plot_d.setConfiguration(
		      percentage_1,percentage_2,percentage_3);
	javax.swing.SwingUtilities.invokeLater(new Runnable() {
		public void run() {
		    speechFrame_d.speech_plot_d.run();
		}
	    });
	
	// action listener for the stop button
	//
	speechFrame_d.stop_button_d.addActionListener(new ActionListener() {
		
		public void actionPerformed(ActionEvent e) {
		    
		     
		    if((action_state_d == RECORDING)|| (action_state_d == PLAYBACK)) {
		    if (audioInterface.getMode() == audioInterface.READ) {
			System.out.println("Closing the line");
		    }
		    else if (
                        audioInterface.getMode() == audioInterface.WRITE) {
			audioInterface.stopPlayBack();
			
			// stop the line
			//
			audioInterface.stop();
			
			// close the line
			//
			audioInterface.close();  
			 }
		    
		    action_state_d = IDLE;
		    
		    // set the stop flag to 1
		    //
		    stop_d = 1; 
		}
		    else {
			JFrame frame = new JFrame("Warning"); 
			String warningMessage = new 
			    String("You can press stop only when recording or playback is in progress.");
			JOptionPane.showMessageDialog(frame,warningMessage);
		    }
		}
		    
	    });

	// action listener for the play_back button
	//
	speechFrame_d.playback_button_d.
	    addActionListener(new ActionListener() {
		
		public void actionPerformed(ActionEvent e) {

		      
		    if(!record_event) {
			JFrame frame = new JFrame("Warning"); 
			String warningMessage = new 
			    String("Need to record audio data before you playback.");
			JOptionPane.showMessageDialog(frame,warningMessage);
		    }
		    
		    else {
			
			// disable the push to talk and the quit button
			// until this utterrance end 
			//
			speechFrame_d.record_button_d.setEnabled(false);
			speechFrame_d.playback_button_d.setEnabled(false);
			speechFrame_d.quit_button_d.setEnabled(false);
			action_state_d = PLAYBACK;
			
			// switch to write mode
			//
			audioInterface.mode_d = audioInterface.WRITE;
			
			// create an SourceDataLine object and open the line
			//
			audioInterface.open();
			
			// start the line
			//
			audioInterface.start();
			
			AudioSignal signal_write = new AudioSignal();    
			
			// set the audioFormat to the signal object
			//
			signal_write.setFormat(audioInterface.getAudioFormat());
			
		      	// get the audioConfig object
			//
			audio = speechFrame_d.getAudioData();
			
			// make sure the object is not null. If null
			// set the default values
			//
			if (audio == null) {
			    
			    // set the data to the signal object
			    //
			    signal_write.setData(audio_data);   
			    playback_mode_d = PLAYBACK_ALL;
			}
			
			// if the object is not null then extract the
			// audio parameters from the audioConfig
			// object ad set to the AudioFormat object
			//
			else {
			    audioInterface.setAudioConfig(audio);			    
			    
			    // loop all values
			    //
			    for (int i = 0;
				 i < audio.getAssociation().size();
				 i++) {
				
				// check whether the name tag equals
				// ChannelDisplayed and if so get the
				// value
				//
				if (audio.getName().get(i)
				    .equals("PlaybackMode")) {
				    String playbackmode = 
					((String)(audio.getValue().get(i)));
				   				    
				    // if the parsed string is equal to "SIGNAL_ONLY"
				    // set the palyback_mode_d to signal_only
				    //
				    if(playbackmode.equals("SIGNAL_ONLY")) {
					
					
					// set the decimal format
					//
					DecimalFormat df = new DecimalFormat("0.###");
					
					// round off the values to third significant decimal point
					//
					Float  start = Float.valueOf(df.format((double)
									       (start_index.floatValue())));
					Float  stop = Float.valueOf(df.format((double)
									      (stop_index.floatValue())));
					       				
					// get the start and the stop sample number
					//
					int start_sample = (int)(start_index.floatValue() * 
								 (float)format_d.getSampleRate());
					int stop_sample = (int)(stop_index.floatValue() * 
								(float)format_d.getSampleRate());
				
					int f = 0;
					int num_channels = audioInterface.getNumOfChannels();
						
					// initialise the float array
					//
					float[][] data_endpointed = new float [num_channels][stop_sample - start_sample + 1];
	
					// put each channel in a separate float array and set
					// it to the frame that is to be sent
					//
					for(int k = 0; k < num_channels; k++) {
					    for( int j = start_sample; j < stop_sample; j++) {
						data_endpointed[k][f] = audio_data[k][j];
						f++;
					    }
					    f = 0; 
					} 
					
					// set the data to the signal object
					//
					signal_write.setData(data_endpointed);   
					playback_mode_d = PLAYBACK_SIGNAL_ONLY;
				    }
				    
				    // else set it to palyback_all
				    //
				    else {
					
					// set the data to the signal object
					//
					signal_write.setData(audio_data);   
					playback_mode_d = PLAYBACK_ALL;  
				    }			       
				}
			    }
			}
			
			// pass the signal object to the startPlayback
			// function of the audioInterface for playback
			//
			audioInterface.startPlayBack(signal_write);
			System.out.println("Playback has started");
			
			// disable the push to talk and the quit button
			// until this utterrance end 
			//
			speechFrame_d.record_button_d.setEnabled(true);
			speechFrame_d.playback_button_d.setEnabled(true);
			speechFrame_d.quit_button_d.setEnabled(true);
			
		    }
		}
	    });
	
	// action listener for the record button
	//
	speechFrame_d.record_button_d.addActionListener(new ActionListener() {
		
		public void actionPerformed(ActionEvent e) {
		    String prog = null;
		    
		    try {
			
			speechFrame_d.setText(" ",DialogNavigationUIFrame.CLEAR);
			speechFrame_d.setTextDialog(" ",DialogNavigationUIFrame.CLEAR);
			// set the flag to true
			//
			record_event = true;
			
			// initialise the audiointerface object
			//
			audioInterface = new AudioInterface();
			
			// reset the display
			//
			speechFrame_d.speech_plot_d.reset();
			
			// set the mode to realtime
			//
			speechFrame_d.speech_plot_d.setMode(true);
			
			// set the flags
			//
			stop_d = 0;
			end_points = 0;
			audio_ready_ack_count = 0;
			utterance_end_ack_count = 0;
			action_state_d = IDLE;
			if (prog == null) prog = "main";
			start_index =  new Float(0.0);
			stop_index = new Float(0.0);
			unicode_line = new String();
			
			// create a new frame and send the audio_ready signal
			//
			GFrame fr = new Clause(prog);
					
			// get the audioConfig object
			//
			audio = speechFrame_d.getAudioData();
			
			// make sure the object is not null. If null
			// set the default values
			//
			if (audio == null) {
			    audioInterface.setAudioFormat(SPEECH);
			    channel_display = 1;
			    buffer_size = 16000;
			    
			}
			
			// if the object is not null then extract the
			// audio parameters from the audioConfig
			// object ad set to the AudioFormat object
			//
			else {
			    audioInterface.setAudioConfig(audio);
			    			    
			    // loop all values
			    //
			    for (int i = 0; 
				 i < audio.getAssociation().size(); i++) {
				
				// check whether the name tag equals
				// ChannelDisplayed and if so get the
				// value
				//
				if (audio.getName().get(i).equals(
						    "ChannelDisplayed")) {
				    channel_display = (int)(Float.parseFloat(
					   (String)(audio.getValue().get(i))));
				    System.out.println("channel_display" + 
						       channel_display);
				}
				
				// check whether the name tag equals
				// BufferSize and if so get the value
				//
				else if (audio.getName().get(i).equals(
							 "BufferSize")) {
				      float buffer_size_in_secs = (float)
					  (Float.parseFloat
					   ((String)(audio.getValue().get(i))));
				      
				      buffer_size = (int) (buffer_size_in_secs *
							   audioInterface.getSampleRate());
				     System.out.println("buffer_size" +
							buffer_size); 
				}
				
				// check whether the name tag equal
				//
				else if (audio.getName().get(i).equals(
							 "FileFormat")) {
				    fr.setProperty(":file_format",
					  (String)(audio.getValue().get(i)));
				}
				    
			    }
			}
				
			// get the audioConfig object
			//
			detector = speechFrame_d.getDetectorData();
			
			// make sure the object is not null. If null
			// set the default values
			//
			if (detector != null) {
			    
			    // loop all values
			    //
			    for (int i = 0; 
				 i < detector.getAssociation().size(); i++) {
	    
				// set the channel to be processed
				//
				if (detector.getName().get(i).equals(
						       "channel_d")) {
				    
				    fr.setProperty(":channel_d",
				      (Float.parseFloat(
				      (String)(detector.getValue().get(i)))));
				}
				
				// set the frame duration
				//
				else if (detector.getName().get(i).equals(
							"frame_duration_d")) {
				    
				    fr.setProperty(":frame_duration_d",
				      (Float.parseFloat(
				      (String)(detector.getValue().get(i)))));
				}
				
				// set the window duration
				//
				else if (detector.getName().get(i).equals(
						       "window_duration_d")) {
				    
				    fr.setProperty(":window_duration_d",
				      (Float.parseFloat(
				      (String)(detector.getValue().get(i)))));
				}
				
					
				// set the frame duration
				//
				else if (detector.getName().get(i).equals(
						       "nominal_signal_d")) {
				    
				    fr.setProperty(":nominal_signal_d",
				      (Float.parseFloat(
				      (String)(detector.getValue().get(i)))));
				}

		
				// set the frame duration
				//
				else if (detector.getName().get(i).equals("signal_adaptation_delta_d")) {
				    
				    fr.setProperty(":signal_adaptation_delta_d",(Float.parseFloat
										 ((String)
										  (detector.getValue()
										   .get(i)))));
				}
		
				// set the frame duration
				//
				else if (detector.getName().get(i).equals("signal_adaptation_constant_d")) {
				    
				    fr.setProperty(":signal_adaptation_constant_d",(Float.parseFloat
										    ((String)
										     (detector.getValue()
										      .get(i)))));
				}
		
				// set the frame duration
				//
				else if (detector.getName().get(i).equals("nominal_noise_d")) {
				    
				    fr.setProperty(":nominal_noise_d",(Float.parseFloat
								       ((String)
									(detector.getValue()
									 .get(i)))));
				}
		
				// set the frame duration
				//
				else if (detector.getName().get(i).equals("noise_adaptation_delta_d")) {
				    
				    fr.setProperty(":noise_adaptation_delta_d",(Float.parseFloat
										((String)
										 (detector.getValue()
										  .get(i)))));
				}
		
				// set the frame duration
				//
				else if (detector.getName().get(i).equals("noise_adaptation_constant_d")) {
				    
				    fr.setProperty(":noise_adaptation_constant_d",(Float.parseFloat
										   ((String)
										    (detector.getValue()
										     .get(i)))));
				}
		
				// set the frame duration
				//
				else if (detector.getName().get(i).equals("noise_floor_d")) {
				    
				    fr.setProperty(":noise_floor_d",(Float.parseFloat
								     ((String)
								      (detector.getValue()
								       .get(i)))));
				}
		
				// set the frame duration
				//
				else if (detector.getName().get(i).equals("utterance_delta_d")) {
				    
				    fr.setProperty(":utterance_delta_d",(Float.parseFloat
									 ((String)
									  (detector.getValue()
									   .get(i)))));
				}
		
				// set the frame duration
				//
				else if (detector.getName().get(i).equals("min_utt_dur_d")) {
				    
				    fr.setProperty(":min_utt_dur_d",(Float.parseFloat
								     ((String)
								      (detector.getValue()
								       .get(i)))));
				}
		
				// set the frame duration
				//
				else if (detector.getName().get(i).equals("min_utt_sep_d")) {
				    
				    fr.setProperty(":min_utt_sep_d",(Float.parseFloat
								     ((String)
								      (detector.getValue()
								       .get(i)))));
				}
		
				// set the frame duration
				//
				else if (detector.getName().get(i).equals("max_utt_dur_d")) {
				    
				    fr.setProperty(":max_utt_dur_d",(Float.parseFloat
								     ((String)
								      (detector.getValue()
								       .get(i)))));
				}
				
				// set the frame duration
				//
				else if (detector.getName().get(i).equals("zc_utt_thresh_d")) {
				    
				    fr.setProperty(":zc_utt_thresh_d",(Float.parseFloat
								       ((String)
									(detector.getValue()
									 .get(i)))));
				}
				
				// set the frame duration
				//
				else if (detector.getName().get(i).equals("preemph_coef_d")) {
				    
				    fr.setProperty(":preemph_coef_d",(Float.parseFloat
								      ((String)
								       (detector.getValue()
									.get(i)))));
				}
				
				// set the frame duration
				//
				else if (detector.getName().get(i).equals("pad_time_d")) {
				    
				    fr.setProperty(":pad_time_d",(Float.parseFloat
								  ((String)
								   (detector.getValue()
								    .get(i)))));
				}
			    }
			}


			// create the neccesary keys and their
			// corresponding values
			//
			fr.setProperty(":program",prog);
			fr.setProperty(":audio_ready", 1);
			fr.setProperty(":utterance_id", utterance_id_d);
			fr.setProperty(":source_name","dialog_navigation"); 
			fr.setProperty(":sample_rate", audioInterface.getSampleRate());
			fr.setProperty(":num_channels",audioInterface.getNumOfChannels());
			fr.setProperty(":sample_bytes",audioInterface.getSampleSize());
			fr.setProperty(":client_name" ,"IBM");
			
			// counter to keep track of the number of
			// button clicks
			//
			btn_press_counter_d = btn_press_counter_d + 1;
			System.out.println("\n");
			
			// print the number of button presses 
			//
			System.out.println("This is the "+ btn_press_counter_d +" button press");
			System.out.println("");  
			System.out.println("Sending the audio ready signal to the data recorder server");
			
			// set the button_press_counter info in the
			// frame
			//
			fr.setProperty(":button_press_counter",btn_press_counter_d); 
			send(fr);
			
			// once the audio_ready signal is sent the
			// state is set to audio_redy_ack
			//
			finite_state_d = AUDIO_READY_ACK;
			
		    } 
		    
		    catch (Exception ex) {
			System.out.println("Caught exception in serverOp: " +ex.toString());
		    }
		    
		    // set the recording to true and set the display
		    // to realtime mode
		    //
		    speechFrame_d.setRecording(true);
		    speechFrame_d.speech_plot_d.setMode(true);
		    
		    // disable the record and the quit button
		    // until this utterrance end 
		    //
		    speechFrame_d.record_button_d.setEnabled(false);
		    speechFrame_d.quit_button_d.setEnabled(false);
		    
		    // initialize the graphic display
		    //
		    // These are the percentages of the three graphical
		    // displays. Energy, Waveform, and Spectro respectively
		    //
		    float percentage_1 = (float) 0.5;
		    float percentage_2 = (float) 0.5;
		    float percentage_3 = (float) 0;
		    
		    // set the sampling rate
		    //
		    speechFrame_d.speech_plot_d.setSampleRate((int)(SPEECH.getSampleRate()));
		    
		    // set the data to the speech plot to null
		    //
		    speechFrame_d.speech_plot_d.setData(null);
		    javax.swing.SwingUtilities.invokeLater(new Runnable() {
		        public void run() {
		    	
		    	speechFrame_d.speech_plot_d.run();
		        }
		    });
		}
	    });
	
	// action listener for the "Input String" box
	//
	speechFrame_d.input_field_d.addActionListener(new ActionListener() {

		public void actionPerformed(ActionEvent e) {
		    try {
			GFrame fr = new Clause("main");
			
			String text = speechFrame_d.input_field_d.getText();
			speechFrame_d.setTextDebug(
					"LOCAL: Got input text as [" 
					+ text + "]\n\n",
					DialogNavigationUIFrame.APPEND);
			if (!text.equals("")) {
			    fr.setProperty(":nbest_list", "garbage_string");
			    fr.setProperty(":input_string", text);
			    send(fr);
			}			
		    }
		    catch (Exception ex) {
			speechFrame_d.setText("When sending text caught" +
					      "this exception:",
				      DialogNavigationUIFrame.APPEND);

			speechFrame_d.setText("Trace: " + ex.toString(),
					      DialogNavigationUIFrame.APPEND);
		    }
		}
	    });
	System.out.println("Exiting constructor\n");
    }
    
    /**
     * This function checks for different properties of the frame it
     * receives. One of its uses is that it gets the process id and
     * server name of all the other servers by querying through the
     * Hub.  
     *
     * @param frame_a a Galaxy frame from Hub.
     *
     * @return a Galaxy frame to Hub.
     */
    public GFrame enableInput(GFrame frame_a) {
	
	// debug window
	//
	speechFrame_d.setTextDebug("IN: Retrieved 'enableInput' Frame from "+
				   "hub ------\n" +
				   frame_a.toFormattedString() +
				   "\n-----------------------------------\n\n",
				   DialogNavigationUIFrame.APPEND);
	//
	// end debug window


	speechFrame_d.setText("It works",
	            	  DialogNavigationUIFrame.APPEND);

	String prog = null;
	
	// enable the record and the quit buttons 
	//
	speechFrame_d.record_button_d.setEnabled(true);
	speechFrame_d.quit_button_d.setEnabled(true);
	
	// set the recording to false
	//
	speechFrame_d.setRecording(false);
	prog = frame_a.getString(":input_string");
	
	if (prog != null) {
            //speechFrame_d.rows_d.setForeground(Color.blue);
	    System.out.println("[enableInput :input_string]: " + prog);
	    //speechFrame_d.setTextDebug("[System]: " + prog,  
	    //			  DialogNavigationUIFrame.APPEND);
	}
	
	// get the file name and display in the main windows
	//
	prog = frame_a.getString(":filename");
	
	if (prog != null) {
	    
	    //speechFrame_d.rows_d.setForeground(Color.blue);
	    System.out.println("[enableInput :filename]: " + prog);
	    //speechFrame_d.setText("File: " + prog + "/n",
	    //          	  DialogNavigationUIFrame.APPEND);
	}

	try {
	    Environment env = getCurrentEnvironment();
       	} 
	catch (Exception ex) {
	    speechFrame_d.setTextDebug("LOCAL: Caught exception in " +
				       "serverOpEnableInput: " +
				       ex.toString(),
				       DialogNavigationUIFrame.APPEND);
	    System.out.println("Caught exception in serverOpEnableInput: " + 
			       ex.toString());
	}

	return frame_a;
    }
    
    /**
     * It is used to reset the whole interface once the end of
     * utterance is sent. This method enables the record and the
     * quit button which are diabled during recording.
     */    
    public void enableButton() {
	
	// enable the record and quit buttons
	//
	speechFrame_d.record_button_d.setEnabled(true);
	speechFrame_d.quit_button_d.setEnabled(true);
	
	// set the recording to false
	//
	speechFrame_d.setRecording(false);
	
	// reset the stop flag to zero
	//
	stop_d = 0;
    }
   
    /**
     * Create Galaxy acknowledgement frame
     *
     * @param source_name_a source name for the signal.
     * @param destination_name_a destination name for the signal.
     * @param message_name_a a message intended to send.
     *
     * @return a Galaxy frame for acknowledgement a signal.
     */
    public GFrame acknowledgementSignal (String source_name_a, 
					 String destination_name_a,
					 String message_name_a) {
	
	// construct a new acknowledgement message
	//
	String prog = MAIN;

	// create a new frame
	//
	GFrame ack_frame = new Clause(prog);

	// set properties
	//
	ack_frame.setProperty(":source_name", source_name_a);
	ack_frame.setProperty(":" + destination_name_a + "_ack",
			      message_name_a);
	ack_frame.setProperty(":button_press_counter", 
			      btn_press_counter_d);
    
	// return the ack frame
	//
    	return ack_frame;
    }

  

    /**
     * Send Galaxy frame to hub.
     *
     * @param frame_a a Galaxy frame to be sent.
     *
     * @return a boolean value indicating status.
     */
    public boolean send(GFrame frame_a) {
	
	// get environment and send the frame
	//
	try {
	    Environment env = getCurrentEnvironment();
	    env.writeFrame(frame_a);
	}
	catch (Exception ex) { 
	    speechFrame_d.setTextDebug("send() caught an exception-------\n" +
				       ex.toString() + "\n",
				       DialogNavigationUIFrame.APPEND);
	    System.out.println("Couldn't send the frame." + ex.toString());
	}	
        // exit gracefully
        //
	return true;
    }
    
    /**
     * This method gets the data from the Audio Interface as a signal
     * object sends the data to the server side.
     *
     * @return a boolean value indicating status.
     */
    public boolean sendData() {

	// set some default value
	//
	int status_indicator;
	AudioSignal signal = new AudioSignal();
	float[][] data;
	
	// make sure there is enough data to get from the
	// audioInterface's circularbuffer 
	//
	if ((status_indicator = audioInterface.
	     read(signal, buffer_size)) != -1) {
	    System.out.println("Status_indicator value[" 
			       + status_indicator + "]");
	    
	    // if there is no data available then sleep for sometime
	    // ad try again
	    //
	    while(status_indicator == 0) {
		try {
		    Thread.sleep(10);
		}
		catch (Exception e) { }
		status_indicator = audioInterface.read(signal,buffer_size);
	    }
	    
	    // make sure we got data ehiel reading the circular buffer
	    //
	    if (status_indicator != 0) { 
		
		// get the data from the signal object
		//
		data = signal.getData();
		
		// get the format 
		//
		format_d = signal.getFormat();
		int channel_len = data.length;
		int data_len = data[0].length;
		int audio_channel_len = audio_data.length;
		int audio_data_len = audio_data[0].length;
		
		if (channel_len != audio_channel_len) {
		    System.out.println("The Number of channels differ in " +
				       "the sent and already stored data.");
		}
		System.out.println("This is data length" + 
				   audio_data[0].length);

		// initialise a float array
		//
		float[][] temp_audio_data = new 
		    float [audio_channel_len][data_len+audio_data_len];
		
		// copy the data in the audio_data array to the temporary array
		//
		if(audio_data_len != 0) {
		    for (int i = 0; i < audio_channel_len; i++) {
			for(int j = 0; j < audio_data_len; j++) {
			    temp_audio_data[i][j]  = audio_data[i][j];
			}
		    }
		}
		
		// add the data got from the read to the temporary
		// array
		//
		for(int i = 0; i < channel_len; i++) {
		    for(int j = audio_data_len; j < (audio_data_len + data_len)
			    ; j++) {
			temp_audio_data[i][j]= data[i][j - audio_data_len];
		    }
		}
		
		// copy back the temporary array to the audio_data
		// object
		//
		audio_data = temp_audio_data;
		System.out.println("the float buffer length of audio data" + 
				   audio_data.length);
		System.out.println("the float buffer length of audio data" + 
				   audio_data[0].length);
		
		// keep count of the number of data frames sent
		//
		data_frame_number_d += 1;
		
		// local variable
		//	    
		String prog = MAIN;
		
		// create a new Galaxy frame
		//
		GFrame data_frame = new Clause(prog);
		
		// set the properties for this frame
		//
		data_frame.setProperty(":raw_data", "raw_data");
		int num_channels = audioInterface.getNumOfChannels();
		int count = 0;
		
		// put each channel in a separate float array and set
		// it to the frame that is to be sent
		//
		while(count < num_channels) {
		    float[] channel_data = new float[data_len];
		    for( int i=0;i< data_len;i++) {
			channel_data[i] = data[count][i];
		    }
		    data_frame.setProperty(":channel"+count,new Float32(channel_data));
		    count++;
		}     
		
		// set the data frame number and source name info
		//
		data_frame.setProperty(":frame_index", data_frame_number_d);
		data_frame.setProperty(":source_name", "dialog_navigation");
		
		// send the data frame
		//
		send(data_frame);
		System.out.println(data_frame_number_d + " frame has been sent.");		
		
		// set the mode to real time and also set the sample
		// rate
		//
		speechFrame_d.speech_plot_d.setMode(true);
		//speechFrame_d.speech_plot_d.
		//  setSampleRate((int)(audioInterface.getSampleRate()));
		
		float[] data_sent = new float[data_len];
		
		// send the user defined channel to be diplayed on the
		// display monitor
		//
		for(int i=0;i<data_len;i++) {
		    data_sent[i] = data[0][i];
		    data_sent[i] = data[channel_display - 1][i];
		    //data_sent[i] = data[0][i];
		}

		System.out.println("the float buffer length sent to display" + 
				   data_sent.length);
		speechFrame_d.speech_plot_d.setData(data_sent);
		
		javax.swing.SwingUtilities.invokeLater(new Runnable() {
			public void run() {
			    speechFrame_d.speech_plot_d.run();
			}
		    });
	    }
	}
	
	else {
	    
	    return false;
	}
	
        // exit gracefully
        //
        return true;	
    }    
    

    //-------------------------------------------------------------------------
    //
    // class-specific public methods that can be called directly from
    // the hub script:
    //
    //-------------------------------------------------------------------------
    
     /**
     * This method is used by the AudioRecorderServer to get the Hub's
     * process id and it's name. It also sends a frame with process_id
     * to query other process's id through the hub which in turn
     * distributes it to other servers.
     *
     * @param frame_a a Galaxy frame from Hub.
     */
    public void serverOpReinitialize(GFrame frame_a) {
	
	// extract the program from the incoming key
	//	
	String prog = frame_a.getString(":program");
	
	if (prog == null) prog = "main";
	
	// create a new frame and set to hub to query the process id
	// from other servers
	//
	GFrame fr = new Clause(prog);
	
	// set the key and the values
	//
	fr.setProperty(":int",1);
	fr.setProperty(":program", prog);
	
	// send the frame
	//
	try {
	    getCurrentEnvironment().writeFrame(fr);
	} 
	catch (Exception ex) {
	    System.out.println("Caught exception while writing to Hub");
	}
	
	// display welcome information in the main user interface window
	//
	speechFrame_d.setText("\n" + "Welcome to Dialog Navigation Demo.\n\n", 
			      DialogNavigationUIFrame.APPEND);
	//debug window
	//
	speechFrame_d.setTextDebug("OUT: Writing frame to hub in response " +
				   "to request for PID--------\n" +
				   fr.toFormattedString() +
				   "\n-----------------------------------\n\n",
				   DialogNavigationUIFrame.APPEND);
	
	
	GFrame fr_database = new Clause(prog);
	fr_database.setProperty(":database_query", 1);
	
	try {
	    getCurrentEnvironment().writeFrame(fr_database);
	} 
	catch (Exception ex) {
	    System.out.println("Caught exception while writing to Hub");
	}
	
	// end debug window
	//
	finite_state_d = INITIALIZATION;
    }
    
    
     /**
     * Returns back the frame.
     */
    public GFrame serverOpEchoFrame(GFrame frame) {
	return frame;
    }
    
  /**
     * This method receives all the frames that are sent from other
     * servers through the hub to the dialog navigation
     * server. Depending on the state when the frame was received
     * different actions are taken.
     *
     * @param frame_a a Galaxy frame received from Hub.
     *
     * @return a GFrame return to Hub.
     */    
    public GFrame serverOpReceiveSignal (GFrame frame_a) {
	
	//debug
	//
	speechFrame_d.setTextDebug("IN: Entered serverOpReceiveSignal while " +
				   "being in state " + finite_state_d + 
				   " and getting frame [" + 
				   frame_a.toFormattedString()
				   + "]\n",
				   DialogNavigationUIFrame.APPEND);
	//end debug

	// check whether the current state is initialization
 	//	
	if (finite_state_d == INITIALIZATION) {
	    enableInput(frame_a);

	    // set the flag to zero
	    //
	    stop_d = 0;
	}
	
	// set the current state to audio_ready_ack
	//
	else if (finite_state_d == AUDIO_READY_ACK) {
	    
	    // get the string stored in the "dialog_navigation_ack" key
	    //
	    String check = (String)frame_a.getProperty(":dialog_navigation_ack");
	    
	    // Put in so that we won't get a null pointer error
	    //
	    if (check == null) {
		check = "";
	    }
    
	    // get the filename and pat where the audio data is stored
	    // on the server side and print the path along with the
	    // filename on the output window in the client side
	    //
	    String prog = (String)frame_a.getProperty(":filename");
	    
	    int code_count = unicode_count;
	    while (code_count > 0) {

		unicode_line +="\u2500";
		    code_count--; 
		    }
	    if (prog != null) {
		//speechFrame_d.rows_d.setForeground(Color.blue);
		speechFrame_d.setText(unicode_line + "\n",
				      DialogNavigationUIFrame.APPEND);
		speechFrame_d.setText(" File:" + prog + "\n",
				      DialogNavigationUIFrame.APPEND);
	    }
	    
	    // check if the string equals audio_ready
	    //
	    if (check.equals("audio_ready") && audio_ready_ack_count == 0) {
		audio_ready_ack_count++;
		System.out.println("Received acknowledgement for audio ready");
	    }	    
	    
	    // check if the string equals audio_ready
	    //
	    else if (check.equals("audio_ready") && audio_ready_ack_count == 1) {
		audio_ready_ack_count++;
		System.out.println("Received acknowledgement for audio ready");
	    }	    
	    
	    // check if the acknowledgement has been got from both the servers
	    //
	    else if(check.equals("audio_ready") && audio_ready_ack_count == 2) {
		
		System.out.println("Received acknowledgement for audio ready");
		
		// set the mode to READ
		//
		audioInterface.mode_d = audioInterface.READ;
		
		// initialise the audio_data object that will contain
		// all the recorded audio data on the client side
		//
		audio_data = new float [audioInterface.getNumOfChannels()] [0];
	
		// try opening a line with the user defined audio
		// parameters. If audioFormat not supported then pop
		// up an error window
		//
		if (audioInterface.open() == false) {
		    frame = new JFrame(); 
		    JOptionPane.showMessageDialog(frame, "Format not Supported." + 
						  "Click ok to restart the demo");
		    	
		    // local variable
		    //	    
		    String prog_reset = MAIN;
		    
		    // create a new Galaxy frame
		    //
		    GFrame reset_frame = new Clause(prog_reset);
		    
		    // set the key and value to reset
		    //
		    reset_frame.setProperty(":reset", "reset");
		    
		    // send the reset frame so that all the states in
		    // the other servers get resetted.
		    //
		    send(reset_frame);
		    finite_state_d = AUDIO_READY;
		    enableButton();
		    
		    // return a null galaxy frame
		    //
		    return (GFrame)null;
		    
		}

		// start the line
		//
		audioInterface.start();
		
		System.out.println("System is ready to record");
		
		// start the recording thread and keep writing the audio data
		// into the circular buffer
		//
		audioInterface.startRecording();
	      
		// set the action state to recording
		//
		action_state_d = RECORDING;
		
		// print the status to the output window
		//
		speechFrame_d.setText("\n" + "    Recording..." + "\n",
				      DialogNavigationUIFrame.APPEND);

		System.out.println("before send data");

		// send the first block of data to the server side
		//
		sendData();
		System.out.println("after send data");
		System.out.println("DATA SENT"+System.currentTimeMillis());
		
		// set the state to raw_data_transfer
		//
		finite_state_d = RAW_DATA_TRANSFER;
	    }	
	}  
	
	else if (finite_state_d == RAW_DATA_TRANSFER) {
	    
	    // get the string value from the "dialog_navigation_ack" key 
	    //
	    String check = (String)frame_a.getProperty(":dialog_navigation_ack");
	    
	    // Put in so that we won't get a null pointer error
	    //
	    if (check == null) {
		check = "";
	    }

	    // check if the string is raw_data
	    //
	     if(check.equals("raw_data")) {
		 
		 System.out.println("ack for raw frame");
		 Integer frame_index = (Integer) frame_a.
		     getProperty(":frame_index");
		 
		 // Put in so that we won't get a null pointer error
		 //
		 if (frame_index == null) {
		     System.out.println("frame_index is NULL! Major problem " +
					"that would have crashed. Need to " +
					"restart demo now.\n");
		     speechFrame_d.setTextDebug("LOCAL ERROR: while in " +
						"RAW_DATA_TRANSFER 'frame_index' was not present in the"
						+ " frame! Setting to -1 but this is a major error\n",
						DialogNavigationUIFrame.APPEND);
		     frame_index = new Integer(-1);
		 }

		 System.out.println("Acknowledgement for the " + 
				    frame_index + " data frame.");

		 // check whether the end points have been detected or
		 // the stop button has been pressed
		 //
		 if (stop_d == 1 || end_points == 1) {
		     
		     // stop the recording
		     //
		     audioInterface.stopRecording();
		     
		     // stop the line
		     //
		     audioInterface.stop();
		     
		     // close the line
		     //
		     audioInterface.close();  
		     
		     speechFrame_d.setText("    End of Recoding." + "\n" ,
					   DialogNavigationUIFrame.APPEND);
		     // local variable
		     //		 
		     String prog = MAIN;
		     
		     // create a Galaxy frame
		     //
		     GFrame fr = new Clause(prog);
		     
		     // set the properties for this frame
		     //
		     fr.setProperty(":input_string", "End of Decoding.");
		     fr.setProperty(":utterance_status", "end");
		     fr.setProperty(":source_name","dialog_navigation");
		     
		     // send the frame through Hub
		     //
		     send(fr);
		     System.out.println("end of utterance"+
					System.currentTimeMillis());
		     System.out.println("The end of utterance has been sent.");
		     
		     // get the user defined channel of data from the
		     // audio_data object
		     //
		     float[] data_float = new float [audio_data[0].length];
		     for(int i = 0; i <audio_data[0].length;i++) {
			 data_float[i] = audio_data[channel_display - 1][i];
		     } 
		     
		     // set the overall plot
		     //
		     speechFrame_d.speech_plot_d.setMode(false);
		     float percentage_1 = (float) 0.5;
		     float percentage_2 = (float) 0.5;
		     float percentage_3 = (float) 0;
		     
		     speechFrame_d.speech_plot_d.setStartTime(start_index.
							      floatValue());
		     speechFrame_d.speech_plot_d.setEndTime(stop_index.
							    floatValue()); 
		     speechFrame_d.speech_plot_d.setData(data_float);
		     
		     javax.swing.SwingUtilities.invokeLater(new Runnable() {
			     public void run() {				
				 speechFrame_d.speech_plot_d.run();
			     }
			 });
		     
		     // change the finite state
		     //
		     finite_state_d = END_OF_UTTERANCE;
		     action_state_d = IDLE;
		 }
		 else {
		     
		     // keep sending the data till the end points are
		     // detected or stop is pressed
		     //
		     sendData();
		     System.out.println("DATA SENT"+System.currentTimeMillis());
		 } 
	     } 
	}
	
	// check if the state is end_of_utterance state
	//
	else if (finite_state_d == END_OF_UTTERANCE) {
	    
	    String check = (String)frame_a.getProperty(":dialog_navigation_ack");

	    // Put in so that we won't get a null pointer error
	    //
	    if (check == null) {
		check = "";
	    }
	    
	    // (needs change in comments) here we wait for two acks
	    // one from the signal detector server and the other from
	    // the data_recorder server
	    //
	    if (check.equals("utterance_status") && 
		utterance_end_ack_count == 0) {
		
		System.out.println("ack for end of utterance");
		utterance_end_ack_count++;
	    }
	    
	    // here we wait for two acks one from the signal detector
	    // server and the other from the data_recorder server
	    //
	    else if (check.equals("utterance_status") && 
		     utterance_end_ack_count == 1) {
		
		System.out.println("ack for end of utterance");
		utterance_end_ack_count++;
	    }
	    
	    // waiting on the second ack
	    // 
	    else if(check.equals("utterance_status") && 
		    utterance_end_ack_count == 2) {
		
		System.out.println("Acknowledgement for the end of " +
				   "utterance signal.");
		
		speechFrame_d.setText("    End of Decoding.\n",
				      DialogNavigationUIFrame.APPEND);
		int code_count = unicode_count;
		unicode_line = new String();
		while (code_count > 0) {
		    
		    unicode_line +="\u2500";
		    code_count--; 
		}
		speechFrame_d.setText(unicode_line + "\n" +"\n",
				      DialogNavigationUIFrame.APPEND);
		
		System.out.println("*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_" +
				   "*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*");
		enableButton();
	    }
	}
	
	// return a null galaxy frame
	//
	return (GFrame)null;
    }
    
    
     /**
      * Get the database parameters
      *
      * @param frame_a a galaxy frame
      *
      * @return a Galaxy frame for acknowledgement a signal.
      */
    public GFrame serverOpDatabaseParams(GFrame frame_a) {
	
	 
	// Make sure we have nice window decorations.
	//
	JFrame.setDefaultLookAndFeelDecorated(true);
	
	// Create and set up the window.
	//
	JFrame frame = new JFrame("Database Parameters");
	frame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
	frame.getContentPane().setLayout(new FlowLayout());
	
	// Create and set up the content pane.
	//
	database_parameters = new DatabasePanel(frame);
	database_parameters.setOpaque(true); // content panes must be opaque
	frame.setContentPane(database_parameters);
	
	// Display the window.
	//
	frame.pack();
	frame.setVisible(true);
	
	// the actionlistener for the databasewindow object
	//
	database_parameters.okButton.addActionListener
	    (new ActionListener() {
		    
		    public void actionPerformed(ActionEvent e) {
			String cmd = e.getActionCommand();
			boolean flag = true;
			
			String username;
			char[] password;
			String hostname;
			String database;
			
			if (database_parameters.OK.equals(cmd)) { // Process the password.
			    
			    
			    username = database_parameters.userField.getText();
			    
			    if (username.equals(null)) {
				JOptionPane.showMessageDialog(database_parameters.controllingFrame,
							      "Please enter a username.");
			    }	
			    
			    password = database_parameters.passwordField.getPassword();
			    
			    if (password.length == 0) {
				JOptionPane.showMessageDialog(database_parameters.controllingFrame,
							      "Password Field cannot be empty.");
			    }
			    
			    
			    hostname = database_parameters.hostField.getText();
			    
			    if (hostname.equals(null)) {
				JOptionPane.showMessageDialog(database_parameters.controllingFrame,
							      "Please enter a hostname.");
			    }	
			    
			    database = database_parameters.databaseField.getText();
			    
			    if (database.equals(null)) {
				JOptionPane.showMessageDialog(database_parameters.controllingFrame,
							      "Please enter a database name.");
			    }	
			    
			    if(!(username.equals(null)|| (password.length == 0) || 
				 (hostname.equals(null)) || (database.equals(null)) )) {
				
				// local variable
				//		 
				String prog = MAIN;
				
				// create a Galaxy frame
				//
				GFrame fr = new Clause(prog);
				
				// set the properties for this frame
				//
				fr.setProperty(":username", username);
				String password_string = new String(password);
				fr.setProperty(":password", (String) password_string);
				fr.setProperty(":hostname",hostname);
				fr.setProperty(":database",database);
				
				// send the frame through Hub
				//
				send(fr);
				database_parameters.dispose();
			    }
			    
			    else {
				JOptionPane.showMessageDialog(database_parameters.controllingFrame,
							      "You need to enter the username, password, hostname and database to which you want to connect");
			    }  
			}
		    }
		});
	
	return (GFrame) null;
    }
    
    /**
     * get the hypothesis and display it on the screen
     *
     * @param frame_a a galaxy frame
     *
     * @return a Galaxy frame for acknowledgement a signal.
     */
    public GFrame serverOpReceiveHypothesis(GFrame frame_a) {

	// debug window
	//
	speechFrame_d.setTextDebug("IN: Receiving speech hypothesis " +
				   "frame--\n" +
				   frame_a.toFormattedString() +
				   "\n-----------------------------------\n\n",
				   DialogNavigationUIFrame.APPEND);
	// end debug window
	
	String hypothesis = (String) frame_a.getProperty(":input_string");
	if (hypothesis == null || hypothesis.equals("!!NO HYPO!!")) {
	    speechFrame_d.setText("Unable to understand hypothesis, " +
				  "repeat yourself please.\n",
			      DialogNavigationUIFrame.APPEND);
	    return (GFrame) null;
	}

	speechFrame_d.setText("Hypothesis:[ " + hypothesis + " ]\n",
			      DialogNavigationUIFrame.APPEND);

	// Create new frame to send this data to the NLP
	// Going to send :input_string, :input_string, :sysid, :path_score
	//	
	GFrame fr = new Clause("main");
	fr.setProperty(":input_to_nlp", (String)hypothesis);
	fr.setProperty(":input_string", (String)hypothesis);
	
	String sysid = (String)frame_a.getProperty(":sysid");
	if (sysid == null) sysid = "";
	fr.setProperty(":sysid", sysid);

	Float float_num = (Float)frame_a.getFloat(":path_score");
	if (float_num == null) float_num = new Float(0.00);
	fr.setProperty(":path_score", float_num);
	
	// send the frame
	//
	send(fr);

	return (GFrame) null;
    }

     /**
     * display the frame
     *
     * @param frame_a a galaxy frame
     *
     * @return a Galaxy frame for acknowledgement a signal.
     */
    public GFrame serverOpDisplayDebug(GFrame frame_a) {
	
	// debug window
	//
	speechFrame_d.setTextDebug("DEBUG: Receiving debug frame--\n" +
				   frame_a.toFormattedString() +
				   "\n-----------------------------------\n\n",
				   DialogNavigationUIFrame.APPEND);
	// end debug window
	
	return (GFrame) null;
	
    }
    

     /**
      * display the frame
     *
     * @param frame_a a galaxy frame
     *
     * @return a Galaxy frame for acknowledgement a signal.
     */
    public GFrame serverOpDisplayNlp(GFrame frame_a) {
	
	// debug window
	//
	speechFrame_d.setTextDebug("IN: Receiving display nlp frame--\n" +
				   frame_a.toFormattedString() +
				   "\n-----------------------------------\n\n",
				   DialogNavigationUIFrame.APPEND);
	// end debug window

	// This frame should only contain
	// :parse_input - string that contains the hypothesis or user input
	// :parse_output - string that is the parsed output
	//
	String parse_in = frame_a.getString(":parse_input");
	String parse_out = frame_a.getString(":parse_output");
	
	// Check each for null
	//
	if (parse_in == null) {
	    parse_in = "<Empty!>";
	}
	if (parse_out == null) {
	    parse_out = "<Empty!>";
	}
	
	// Print each out to NLP screen
	//
	speechFrame_d.setTextDialog("-----Parse input:----------------\n",
				 DialogNavigationUIFrame.APPEND);
	speechFrame_d.setTextDialog(parse_in + 
				    "\n------------------------------\n\n",
				 DialogNavigationUIFrame.APPEND);
	
	speechFrame_d.setTextDialog("-----Parse output:----------------\n",
				 DialogNavigationUIFrame.APPEND);
	// Make sure it does't have any trailing whitespace
	//
	speechFrame_d.setTextDialog(parse_out.trim() + 
				    "\n-------------------------------\n\n",
				 DialogNavigationUIFrame.APPEND);
	    
	return (GFrame) null;
    }

    /**
     * display the frame
     *
     * @param frame_a a galaxy frame
     *
     * @return a Galaxy frame for acknowledgement a signal.
     */
    public GFrame serverOpDisplayPrompt(GFrame frame_a) {
	
	// debug window
	//
	speechFrame_d.setTextDebug("IN: Receiving display prompt frame--\n" +
				   frame_a.toFormattedString() +
				   "\n-----------------------------------\n\n",
				   DialogNavigationUIFrame.APPEND);
	// end debug window

	String prompt_d = (String) frame_a.getProperty(":prompt");
	if (prompt_d != null) {
	    
	    // This now has output from the system so display to the user
	    //
	    speechFrame_d.setText("Response to user:[" + 
				  prompt_d.trim() + "]\n",
				  DialogNavigationUIFrame.APPEND);
	}

	return (GFrame) null;
    }
    
    /**
      * display the frame
      *
      * @param frame_a a galaxy frame
      *
     * @return a Galaxy frame for acknowledgement a signal.
     */
    public GFrame serverOpDisplayDatabase(GFrame frame_a) {

	// debug window
	//
	speechFrame_d.setTextDebug("IN: Receiving display database frame--\n" +
				   frame_a.toFormattedString() +
				   "\n-----------------------------------\n\n",
				   DialogNavigationUIFrame.APPEND);
	// end debug window
	
	String text_to_display = null;
	String db_query_d = null;
	String task_d = null;
	String query_params_d = null;
	Integer access_web_d = null;
	String object_info = "";

	GFrame query_result_d = (GFrame) frame_a.getProperty(":query_result");
	
	// This can accept both the query and the response
	// This first section is the query
	//
	if (query_result_d == null) {

	    db_query_d = frame_a.getString(":db_query");
	    if (db_query_d == null)
		db_query_d = "<Empty!>";
	    
	    task_d = frame_a.getString(":task");
	    if (task_d == null)
		task_d = "<Empty!>";
	    
	    query_params_d = frame_a.getString(":query_params");
	    if (query_params_d == null)
		query_params_d = "<Empty!>";
	    
	    access_web_d = frame_a.getInt(":access_web");
	    if (access_web_d == null)
		access_web_d = new Integer(0);
	    
	    text_to_display = "IN: ------Database request-------------\n" +
		"  query: " + db_query_d + "\n" +
		"  task:  " + task_d + "\n" +
		"  params: " + query_params_d + "\n" +
		"  access_web: " + access_web_d + "\n" +
		"-----------------------------------\n\n";
	}
	// This section means this is a query_result
	//
	else {
	    // Find out how many results
	    //
	    Integer num_found_d;
	    num_found_d = query_result_d.getInt(":nfound");
	    if (num_found_d == null)
		num_found_d = new Integer(0);
	    
	    // If not 0 results, -2 is web error, -1 is sql error
	    //	 
	    if (num_found_d.intValue() > 0) {
	
		// Print out the first object to get an idea of the type of
		// answer.
		//
		GVector gvec = (GVector)query_result_d.getProperty(":values");
		if (gvec != null) {
		    GalaxyObject gal_obj = (GalaxyObject)gvec.elementAt(0);
		    object_info = 
			"  first element: type is:[" + 
			gal_obj.getTypeName() + "]\n" +
			"                 value is:[" + 
			gal_obj.toString() + "]\n";
		}
	    }
	    
	    text_to_display =  "FROM PACKED FRAME: ------Database Response" +
		"------------\n" +
		"  number of elements found: " + num_found_d + "\n" + 
		object_info + //this will either contain data or ""
		"-----------------------------------\n\n";
	}

	speechFrame_d.setTextDebug(text_to_display,
				    DialogNavigationUIFrame.APPEND);
	return (GFrame) null;
    }

    /**
     * This function gets the endpoints from the signal detector and
     * sets the end_points flag to 1. It also sends the endpointed
     * data to the data recorder.
     *
     * @param frame_a a Galaxy frame to be sent.
     *
     * @return a boolean value indicating status.
     */
    public GFrame serverOpEndPoints(GFrame frame_a) {
	
	// get the start and the stop times
	//
	 start_index = (Float) frame_a.getProperty(":start_time");
	 stop_index = (Float) frame_a.getProperty(":stop_time");
	
	// set the decimal format
	//
	DecimalFormat df = new DecimalFormat("0.###");
	
	// round off the values to third significant decimal point
	//
	Float  start = Float.valueOf(df.format((double)
					       (start_index.floatValue())));
	Float  stop = Float.valueOf(df.format((double)
					      (stop_index.floatValue())));
	
	// display the start and the stop time on the text field
	//
	speechFrame_d.setText("        Start Time: "+start+ " secs" +"    ",
			      DialogNavigationUIFrame.APPEND);
	speechFrame_d.setText("End Time: "+stop + " secs" + "\n",
			      DialogNavigationUIFrame.APPEND);
	System.out.println("end points calculated" + 
			   System.currentTimeMillis());
	
	// get the start and the stop sample number
	//
	int start_sample = (int)(start_index.floatValue() * 
				 (float)format_d.getSampleRate());
	int stop_sample = (int)(stop_index.floatValue() * 
				(float)format_d.getSampleRate());
	
	// initialise the float array
	//
	float[] data_endpointed = new float [stop_sample-start_sample+1];
	int f = 0;
	
	int channel_count = 0;

	
	// local variable
	//	    
	String prog = MAIN;
	
	// create a new Galaxy frame
	//
	GFrame data_frame = new Clause(prog);


	int num_channels = audioInterface.getNumOfChannels();
		
	// put each channel in a separate float array and set
	// it to the frame that is to be sent
	//
	while(channel_count < num_channels) {
	    for( int i=start_sample;i< stop_sample;i++) {
		data_endpointed[f] = audio_data[channel_count][i];
		f++;
	    }
	    data_frame.setProperty(":channel"+channel_count,
				   new Float32(data_endpointed));
	    f = 0;
	    channel_count++;
	}    
	
	// set the endpointed data in the frame
	//
	data_frame.setProperty(":raw_data_endpointed", "endpointed");
	data_frame.setProperty(":source_name", "dialog_navigation");
	
	// send the data frame to the data recorder
	//
	send(data_frame);

	// set the end_pointd flag to 1
	//
	end_points = 1;
	
	return (GFrame) null;
    }


     /**
     * Displays the start of decoding
     *
     * @param frame_a a Galaxy frame to be sent.
     *
     * @return a boolean value indicating status.
     */
    public GFrame serverOpDisplayDecoding(GFrame frame_a) {
	
	speechFrame_d.setText("    Decoding......" + "\n",
			      DialogNavigationUIFrame.APPEND);
	return (GFrame)null;
    }
    
    /**
     * resets the client side all the other servers
     *
     * @param frame_a a Galaxy frame to be sent.
     *
     * @return a boolean value indicating status.
     */
    public GFrame serverOpResetState(GFrame frame_a) {
	
	finite_state_d = AUDIO_READY;
	enableButton();
	
	// local variable
	//	    
	String prog_reset = MAIN;
	GFrame reset_frame = new Clause(prog_reset);
	
	// set the key and value to reset
	//
	reset_frame.setProperty(":reset_all", "reset");
	
	// send the reset frame so that all the states in
	// the other servers get resetted.
	//
	send(reset_frame);
	JFrame frame = new JFrame("Error Checking"); 
	String warningMessage = new 
	    String("Model Compatibility: The sampling frequency you have" + 
		   "choosen is not compatible with the model.Please go to" + 
		   " the Configuration menu to change it");
	JOptionPane.showMessageDialog(frame, warningMessage, 
				      "Error Checking", JOptionPane.WARNING_MESSAGE);
	speechFrame_d.setText(" ",DialogNavigationUIFrame.CLEAR);
	
	return (GFrame)null;

    }
    
    //-------------------------------------------------------------------------
    //
    // class-specific other private methods:
    //  
    //
    //-------------------------------------------------------------------------
    
    /** 
     * Convert float array to byte array.
     *
     * @param byteSound User provided byte array to return result in.
     * @param dbuf User provided float array to convert.
     */
    private void floatToByte(byte[] byteSound, float [][] dbuf) {
	
	// check whether this will work
	//
	int bufsz = dbuf[0].length;
	int ib = 0;
	if(format_d.isBigEndian()) {
	    for(int i = 0; i < bufsz; i++) {
		short y = (short)Math.round(32767 * dbuf[0][i]);
		byteSound[ib] = (byte)(y >> 8);
		ib++;
		byteSound[ib] = (byte)(y & 0x00ff);
		ib++;
	    }
	} 
	else {
	    for(int i=0;i<bufsz;i++) {
		short y = (short)Math.round(32767 * dbuf[0][i]);
		byteSound[ib] = (byte)(y & 0x00ff);
		ib++;
		byteSound[ib] = (byte)(y >> 8);
		ib++;
            }
	}
    }
    
    /** 
     * Convert byte array to float array.
     *
     * @param dbuf User provided float array to return result in.
     * @param dbuf User provided byte array to convert.
     * @param bufsz Number of float samples to write.
     */
    private void byteToFloat(float [][] dbuf, byte[] bbuf, int bufsz) {
        int ib=0;
        if(format_d.isBigEndian()) {
            for(int i = 0; i < bufsz; i++) {
                short y = bbuf[ib];
                y = (short)(y << 8);
                y |= (bbuf[ib+1]&0x00ff);
                ib += 2;
                dbuf[0][i] = y / 32767.f;
            }
        } 
	else {
            for(int i = 0; i < bufsz; i++) {
                short y = (short)bbuf[ib+1];
                y = (short)(y << 8);
                y |= (bbuf[ib]&0x00ff);
                ib += 2;
                dbuf[0][i] = y / 32767.f;
            }
        }
    }	
    
     


    //------------------------------------------------------------------------
    //
    // class-specific public methods:
    //  error handler code
    //
    //------------------------------------------------------------------------
  
    /**
     * Method to simply print the passed error, a stack trace, and then exit.
     *
     *@param e Exception that was caught
     */
    private void handleError(Exception e){
	System.out.println("Exception in ISIPAudioRecorder: " + e);
	e.printStackTrace();
    }   
}

