//GMM Demo: This Program is intended to show some of "ISIP Production System" capibilities in training and  defining GMMs
//In this Demo, we train 2 GMM Models with 2 mixtures using data generated by a Matlab script.
//To make things simple, we confined ourselves to just 2 Models, each with 2 Mixtures.
//Amir Harati, Temple University 2011

#include <MixtureModel.h>
#include <GaussianModel.h>
#include<Console.h>
#include<CommandLine.h>
#include "isip_gmm_demo_support.h"

int main(int argc, const char** argv) 
{

 Sdb sdb1;  
 CommandLine cmdl(sdb1);
 cmdl.setUsage(
#include "usage_message.text"
                );
 cmdl.setHelp(
#include "help_message.text"
             );
 DebugLevel verbose;
 cmdl.addOptionParam(verbose,L"verbosity",Integral::BRIEF);
 String logFile;
 cmdl.addOptionParam(logFile,L"log_file",L"isip_gmm_demo.log");
 String outFile;
 cmdl.addOptionParam(outFile,L"out_file",L"isip_gmm_demo_out.sof");
 String initMethod;
 cmdl.addOptionParam(initMethod,L"init_method",L"default");
 String initFile;
 cmdl.addOptionParam(initFile,L"init_file",String::NULL_STRING);
 Long BUFFER_SIZE;
 cmdl.addOptionParam(BUFFER_SIZE,L"buffer_size",(long)100000);
 Long MAX_ITR;
 cmdl.addOptionParam(MAX_ITR,L"max_itr",(long)10);
 Float var_floor0;
 cmdl.addOptionParam(var_floor0,L"var_floor",(float)0.001);
 Long min_count;
 cmdl.addOptionParam(min_count,L"min_model_count",(long)3);
 DebugLevel debug_level;
 cmdl.addOptionParam(debug_level, L"debug" ,Integral::NONE);

String train_file_1;String train_file_2;String test_file_data;String test_file_class;

if (!cmdl.parse(argc, argv)) {
   cmdl.printUsage();
  };
Filename filename[4];
{
int i=0;
for (bool8 file = sdb1.gotoFirst(); file; file = sdb1.gotoNext()) {
    	
    	    Sof sof1;
   	        	    
    	    // open file in read-only mode
    	    //
    	    sdb1.getName(filename[i]);
    	    if (!sof1.open(filename[i], File::READ_ONLY)) {
    	      return Error::handle(sof1.name(), L"open",
    				   Error::TEST, __FILE__, __LINE__);
    	    }
    	    
  	    
    	    // close the file
    	    //
    	    sof1.close();
i++;    	  
}
if(i!=4){
cmdl.printUsage();
}
}
filename[0].getName(train_file_1);
filename[1].getName(train_file_2);
filename[2].getName(test_file_data);
filename[3].getName(test_file_class);


long dataDim;
{
 //Deterine data dimension from the data
 VectorFloat tempBuff[1];
 long start_tag_tmp=-1,buff_size_tmp=-1;
 loadData(train_file_1,start_tag_tmp,tempBuff,buff_size_tmp,1,L"VectorFloat");
 dataDim=tempBuff[0].length();
}

VectorFloat var_floor;
var_floor.setLength(dataDim);
for(int i=0;i<dataDim;i++)
 var_floor(i)=var_floor0;
//Check command line to be correct
if(BUFFER_SIZE<1)
{
 cmdl.printUsage();
}



if(initMethod.ne(L"default") && initMethod.ne(L"file"))
{ 
 cmdl.printUsage();
}

if(initMethod.eq(L"file") && initFile.eq(String::NULL_STRING))
{ 
 cmdl.printUsage();
}
//
//Open the log file 
File::remove(logFile);
Console::open(logFile);
//buffers
long MAX_BUFF=BUFFER_SIZE,buff_size1=-1,buff_size2=-1;
VectorFloat trainBuff_class1[MAX_BUFF];
VectorFloat trainBuff_class2[MAX_BUFF];

long start_tag1=-1,start_tag2=-1;
long data_chunck=1;

long total1=0;
VectorFloat mean1;
MatrixFloat cov1;

long total2=0;
VectorFloat mean2;
MatrixFloat cov2;


mean1.setLength(dataDim);
mean2.setLength(dataDim);
cov1.setDimensions(dataDim,dataDim);
cov2.setDimensions(dataDim,dataDim);

//Initlization: Load all training data to compute initila mean and coverinace of each mixture
//We first assume a single gaussian and compute mean and covariance for that  and then  split it 
//into to part
//The data is loaded in blocks 
sendMessage(L"Start GMM Demo",verbose,0);
sendMessage(L"",verbose,0);

sendMessage(L"Start Initilization ...",verbose,0);

//Compute mean and Covariance of each class by assuming a Single Gaussain
//This is used for defaul initlization method
if(initMethod.eq(L"default"))
{
sendMessage(L"Default Method ...",verbose,1);
while(buff_size1!=0 || buff_size2!=0)
{

loadData(train_file_1,start_tag1,trainBuff_class1,buff_size1,MAX_BUFF,L"VectorFloat");
loadData(train_file_2,start_tag2,trainBuff_class2,buff_size2,MAX_BUFF,L"VectorFloat");

computeMeanCov(trainBuff_class1,buff_size1,total1,mean1,cov1);
computeMeanCov(trainBuff_class2,buff_size2,total2,mean2,cov2);
data_chunck++;
}
Convert2Diagonal(cov1);
Convert2Diagonal(cov2);
}

///////////////////////////////////////////

//Define the Models and init

//Model 1
MixtureModel model1;
GaussianModel gauss1,gauss2;

if(initMethod.eq(L"default"))
{
// set the GaussianModel object
VectorFloat mean;
cov1.getDiagonal(mean);
mean.sqrt();
mean.add(mean1);

gauss1.setMean(mean);
gauss1.setCovariance(cov1);


cov1.getDiagonal(mean);
mean.sqrt();
mean.mult(-1);
mean.add(mean1);
gauss2.setMean(mean);
gauss2.setCovariance(cov1);
}
if(initMethod.eq(L"file"))
{
sendMessage(L"Read Initial Values From File ...",verbose,1);
VectorFloat mean1_m1,mean2_m1,mean1_m2,mean2_m2;
MatrixFloat cov1_m1,cov2_m1,cov1_m2,cov2_m2;
mean1_m1.setLength(dataDim);
mean2_m1.setLength(dataDim);
mean1_m2.setLength(dataDim);
mean2_m2.setLength(dataDim);
cov1_m1.setDimensions(dataDim,dataDim);
cov2_m1.setDimensions(dataDim,dataDim);
cov1_m2.setDimensions(dataDim,dataDim);
cov2_m2.setDimensions(dataDim,dataDim);

byte8 *tmp;
tmp=(byte8*)initFile;
string str((char*)tmp);

loadInit(str,mean1_m1,mean2_m1,cov1_m1,cov2_m1,mean1_m2,mean2_m2,cov1_m2,cov2_m2);
gauss1.setMean(mean1_m1);
gauss1.setCovariance(cov1_m1);
gauss2.setMean(mean2_m1);
gauss2.setCovariance(cov2_m1);

}
VectorFloat weights(L".5 .5");
model1.setWeights(weights);

// add model objects into the MixtureModel object
model1.add(gauss1);
model1.add(gauss2);

//Model 2
MixtureModel model2;
if(initMethod.eq(L"default"))
{
// set the GaussianModel object
VectorFloat mean;
cov2.getDiagonal(mean);
mean.sqrt();
mean.add(mean2);

gauss1.setMean(mean);
gauss1.setCovariance(cov2);


cov2.getDiagonal(mean);
mean.sqrt();
mean.mult(-1);
mean.add(mean2);
gauss2.setMean(mean);
gauss2.setCovariance(cov2);
}
if(initMethod.eq(L"file"))
{
VectorFloat mean1_m1,mean2_m1,mean1_m2,mean2_m2;
MatrixFloat cov1_m1,cov2_m1,cov1_m2,cov2_m2;
mean1_m1.setLength(dataDim);
mean2_m1.setLength(dataDim);
mean1_m2.setLength(dataDim);
mean2_m2.setLength(dataDim);
cov1_m1.setDimensions(dataDim,dataDim);
cov2_m1.setDimensions(dataDim,dataDim);
cov1_m2.setDimensions(dataDim,dataDim);
cov2_m2.setDimensions(dataDim,dataDim);
byte8 *tmp;
tmp=(byte8*)initFile;
string str((char*)tmp);

loadInit(str,mean1_m1,mean2_m1,cov1_m1,cov2_m1,mean1_m2,mean2_m2,cov1_m2,cov2_m2);
gauss1.setMean(mean1_m2);
gauss1.setCovariance(cov1_m2);
gauss2.setMean(mean2_m2);
gauss2.setCovariance(cov2_m2);

}
weights.assign(L".5 .5");
model2.setWeights(weights);

// add model objects into the MixtureModel object
model2.add(gauss1);
model2.add(gauss2);


//////////////////////////////////////////////

sendMessage(L"Initilization is Done ...",verbose,0);

if(debug_level>=Integral::BRIEF)
{
sendMessage(L"Models After Iniltilization",verbose,0);
model1.debug(L"Model1 after Initilization");
model2.debug(L"Model2 after Initilization");
}



VectorDouble params1(L"0,0,0,1,0,-10");
VectorDouble params2(L"0,0,0,1,0,-10");

double L10,L11;
L10=0;L11=-200000;

VectorDouble LL1;
LL1.setLength(MAX_ITR);
long LL1_len=0;

sendMessage(L"Start Training .....",verbose,0);
sendMessage(L"Start Training Model 1 ...",verbose,1);
for(long itr=0;itr<MAX_ITR;itr++)
{
  String str(L"Iteration : ");
  str.concat((uint64)itr);
  sendMessage(str,verbose,2);
//Accumulate 
buff_size1=-1;
start_tag1=-1;
data_chunck=1;
sendMessage(L"Re-Estimation ...",verbose,3);
while(buff_size1!=0)
{
loadData(train_file_1,start_tag1,trainBuff_class1,buff_size1,MAX_BUFF,L"VectorFloat");
trainModel_step0(model1,trainBuff_class1,buff_size1,params1);
params1.setLength(8,false);
data_chunck++;
}
//Update the Model
sendMessage(L"Update The Model ...",verbose,3);
model1.update(var_floor,min_count);

//Compute Likelihood
buff_size1=-1;
start_tag1=-1;
data_chunck=1;
sendMessage(L"Computing Likelihood ...",verbose,3);
while(buff_size1!=0)
{
loadData(train_file_1,start_tag1,trainBuff_class1,buff_size1,MAX_BUFF,L"VectorFloat");
trainModel_step1(model1,trainBuff_class1,buff_size1,L10);
data_chunck++;
}
if(round(10000*L10)>round(10000*L11))
{
L11=L10;
L10=0;
params1(5)=L11;
String tmp(L"Likelihood :");
tmp.concat(L11);
sendMessage(tmp,verbose,4);
LL1(itr)=L11;
LL1_len++;
}
else
{
String tmp(L"Likelihood :");
tmp.concat(L11);
sendMessage(tmp,verbose,4);
break;
}

}



double L20,L21;
L20=0;L21=-200000;
VectorDouble LL2;
LL2.setLength(MAX_ITR);
long LL2_len=0;
sendMessage(L"",verbose,0);
sendMessage(L"Training Model 2...",verbose,0);
for(long itr=0;itr<MAX_ITR;itr++)
{
String str(L"Iteration : ");
str.concat((uint64)itr);
sendMessage(str,verbose,2);
//Accumulate 
buff_size2=-1;
start_tag2=-1;
data_chunck=1;
sendMessage(L"Re-Estimation ...",verbose,3);
while(buff_size2!=0)
{
loadData(train_file_2,start_tag2,trainBuff_class2,buff_size2,MAX_BUFF,L"VectorFloat");
trainModel_step0(model2,trainBuff_class2,buff_size2,params2);
params2.setLength(8,false);
data_chunck++;
}
//Update the Model
sendMessage(L"Update the Model...",verbose,3);
model2.update(var_floor,min_count);

//Compute Likelihood
buff_size2=-1;
start_tag2=-1;
data_chunck=1;
sendMessage(L"Computing Likelihood ...",verbose,3);
while(buff_size2!=0)
{
loadData(train_file_2,start_tag2,trainBuff_class2,buff_size2,MAX_BUFF,L"VectorFloat");
trainModel_step1(model2,trainBuff_class2,buff_size2,L20);
data_chunck++;
}
if(round(10000*L20)>round(10000*L21))
{
L21=L20;
L20=0;
params2(5)=L21;
String tmp(L"Likelihood :");
tmp.concat(L21);
sendMessage(tmp,verbose,4);
LL2(itr)=L21;
LL2_len++;
}
else
{
String tmp(L"Likelihood :");
tmp.concat(L21);
sendMessage(tmp,verbose,4);
break;
}


}
sendMessage(L"",verbose,0);
sendMessage(L"Finish Training...",verbose,0);

if(debug_level>=Integral::BRIEF)
{
sendMessage(L"Models After Training",verbose,0);
model1.debug(L"Model1 after Training");
model2.debug(L"Model2 after Training");
}

sendMessage(L"",verbose,0);
sendMessage(L"Start Decoding....",verbose,0);
Double error=0;
buff_size1=-1;start_tag1=-1,start_tag2=-1;
long tmp;
VectorFloat testBuff_data[MAX_BUFF];
VectorFloat testBuff_class[MAX_BUFF];
VectorDouble LL_1,LL_2;
VectorFloat result;
while(buff_size1!=0)
{
loadData(test_file_data,start_tag1,testBuff_data,buff_size1,MAX_BUFF,L"VectorFloat");
loadData(test_file_class,start_tag2,testBuff_class,tmp,MAX_BUFF,L"VectorFloat");
computeLikelihood(model1,testBuff_data,buff_size1,LL_1);
computeLikelihood(model2,testBuff_data,buff_size1,LL_2);
decode(LL_1,LL_2,result);
errorCompute(result,testBuff_class,error);
}
sendMessage(L"Finish Decoding...",verbose,0);
String tmp_str(L"Error on test set  : ");
tmp_str.concat((float)error);
sendMessage(tmp_str,verbose,1);

tmp_str.assign(L"Writing output in : ");
tmp_str.concat(outFile);
sendMessage(tmp_str,verbose,0);

Sof out;
out.open(outFile,File::WRITE_ONLY, File::TEXT);
model1.write(out,1,L"Model");
model2.write(out,2,L"Model");
LL1.setLength(LL1_len);
LL1.write(out,1,L"Training_Likelihood_Model");
LL2.setLength(LL2_len);
LL2.write(out,2,L"Training_Likelihood_Model");
error.write(out,0,L"Error_on_Test_data");
out.close();
Console::close();


}
 
