// file: tr_latn_cstr_0.cc
//

// isip include files
//
#include "train_lattice_node.h"
#include "train_lattice_node_constants.h"

#include <train_lex_tree.h>
#include <train_lex_tree_constants.h>

// method: destructor
//
// arguments: none
//
// return: none
//
// this is the default destructor
//
Train_Lattice_node::~Train_Lattice_node() {

  // reset node statistics
  //
  type_d = TRAIN_LATTICE_NODE_DEFAULT_TYPE;
  word_d = (Train_Word*)NULL;
  pron_variant_d = TRAIN_LATTICE_NODE_DEFAULT_PRON_VAR;
  node_index_d = TRAIN_LATTICE_NODE_DEFAULT_INDEX;
  frame_d = (int_4)0;
  num_arcs_in_d = (int_4)0;
  num_arcs_out_d = (int_4)0;
  
  // delete node pointers (note that we need not delete the items in
  // in the nodes because they are only pointers to nodes held by the
  // lnode_d structure in Train_Lattice)
  //
  if (prev_nodes_d != (Train_Link_list*)NULL) {
    delete prev_nodes_d;
    prev_nodes_d = (Train_Link_list*)NULL;
  }
  if (next_nodes_d != (Train_Link_list*)NULL) {
    delete next_nodes_d;
    next_nodes_d = (Train_Link_list*)NULL;
  }

  // delete lm scores
  //
  if (lm_scores_d != (float_8*)NULL) {
    delete [] lm_scores_d;
    lm_scores_d = (float_8*)NULL;
  }
  
  // delete ac scores
  //
  if (ac_scores_d != (float_8*)NULL) {
    delete [] ac_scores_d;
    ac_scores_d = (float_8*)NULL;
  }
  
  // the lexical tree contained here will be deleted externally
  //
  if (lex_tree_d != (Train_Lex_tree*)NULL) {
    lex_tree_d = (Train_Lex_tree*)NULL;
  }
  
  // exit gracefully
  //
}





